@@ -73,7 +73,7 @@ TRACK_HEIGHT_SMALL = 25  # track height in canvas and column
 TRACK_HEIGHT_SMALLEST = 25 # DEPRECATED
 
 # Main layout minimum heights. Changing these will have effect on layout.
-TOP_ROW_HEIGHT = 300
+TOP_ROW_HEIGHT = 270
 TLINE_HEIGHT = 260
 
 # Main layout minimum widths. Changing these will have effect on layout.
@@ -169,6 +169,7 @@ CC_CLIP_FRAMES_DIR = "/clip_frames"
 CC_RENDERED_FRAMES_DIR = "/rendered_frames"
 CC_PREVIEW_RENDER_DIR = "/preview_frames"
 USER_SHORTCUTS_DIR =  "user_shortcuts/"
+USER_USBHID_DIR =  "user_usbhid/"
 SCRIP_TOOL_DIR = "scripttool"
 TEMP_RENDER_DIR = "temprender/"
 
@@ -371,3 +372,49 @@ PRESET_GROUP_IMAGE_SEQUENCE = "Image Sequence"
 PRESET_GROUP_AUDIO = "Audio" 
 PRESET_GROUP_MISC = "webmetc"
 PRESET_GROUP_ALPHA = "Alpha"
+
+# Media file ratings.
+MEDIA_FILE_UNRATED = 0
+MEDIA_FILE_FAVORITE = 1
+MEDIA_FILE_BAD = 2
+
+MEDIA_RATINGS_SHOW_ALL = 0
+MEDIA_RATINGS_SHOW_FAVORITES = 1
+MEDIA_RATINGS_HIDE_BAD = 2
+
+# Extensions for data files.
+STABILIZE_DATA_EXTENSION = ".stabdata"
+MOTION_TRACKING_DATA_EXTENSION = ".trackdata"
+
+# magic values string propertie that have not been set yet.
+FILE_PATH_NOT_SET = "!!##PATHNOTSET##!!"
+TRACKING_DATA_NOT_SET = "!!##TRACKINGDATANOTSET##!!"
+
+# GUI scaling for larger and HIDPI screens.
+TRACKS_SCALE_DEFAULT = 0
+TRACKS_SCALE_ONE_AND_HALF = 1
+TRACKS_SCALE_DOUBLE = 2
+
+ICONS_SCALE_DEFAULT = 0
+ICONS_SCALE_DOUBLE = 1
+
+GUI_ITEMS_SCALE_DEFAULT = 1.0
+GUI_ITEMS_SCALE_ONE_AND_QUATER = 1.25
+GUI_ITEMS_SCALE_ONE_AND_HALF = 1.5
+
+SLOWMO_SLOW_FAST = 0
+SLOWMO_REVERSE = 1
+
+SLOWMO_MEDIA_RANGE_CLIP_AREA = 0
+SLOWMO_MEDIA_RANGE_FULL_MEDIA = 1
+
+PROGRAM_OUT_MODE = 0
+VECTORSCOPE_MODE = 1
+RGB_PARADE_MODE = 2
+
+# values set in /res/mediaplugins/plugins.json
+# and use in containerclip.py whencreating new 
+# ContainerClipMediaItem objects for Generators.
+DEFAULT_RENDER_CLIP = "clip"
+DEFAULT_RENDER_ALPHA_CLIP = "alphaclip"
+DEFAULT_RENDER_FRAME_SEQUENCE = "framesequecne"
