@@ -2,7 +2,7 @@
     Flowblade Movie Editor is a nonlinear video editor.
     Copyright 2012 Janne Liljeblad.
 
-    This file is part of Flowblade Movie Editor <http://code.google.com/p/flowblade>.
+    This file is part of Flowblade Movie Editor <https://github.com/jliljebl/flowblade/>.
 
     Flowblade Movie Editor is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -29,23 +29,24 @@ except ImportError:
 import gi
 
 import cairo
-import mlt
-import time
+try:
+    import mlt7 as mlt
+except:
+    import mlt
 
 from gi.repository import Gtk, GObject
 from gi.repository import GLib
-from gi.repository import Gdk
 from gi.repository import Pango
 gi.require_version('PangoCairo', '1.0')
 from gi.repository import PangoCairo
 
 import appconsts
 import cairoarea
-import editorpersistance
 import editorstate
 import mltrefhold
 import guiutils
 import utils
+import utilsgtk
 
 SLOT_W = 60
 METER_SLOT_H = 458
@@ -61,7 +62,7 @@ METER_LIGHTS = 143 #57
 METER_HEIGHT = METER_LIGHTS * DASH_INK + (METER_LIGHTS - 1) * DASH_SKIP
 METER_WIDTH = 10
 
-# These are calculated using IEC_Scale function in MLT and correspond to level values received here
+# These are calculated using IEC_Scale function in MLT and correspond to level values received here.
 DB_IEC_MINUS_2 = 0.95
 DB_IEC_MINUS_4 = 0.9
 DB_IEC_MINUS_6 = 0.85
@@ -75,7 +76,8 @@ OVER_FRAMES = 30
 
 # Colors
 METER_BG_COLOR = (0.15, 0.15, 0.15)
-OVERLAY_COLOR = (0.70, 0.70, 0.70) #utils.get_cairo_color_tuple_255_rgb(63, 145, 188)#59, 140, 174) #(0.7, 0.7, 1.0)
+OVERLAY_COLOR = (0.70, 0.70, 0.70)
+CHANNEL_ID_COLOR = (0.06, 0.34, 0.66)
 
 # Color gradient used to draw "LED" colors
 rr, rg, rb = utils.get_cairo_color_tuple_255_rgb(219, 69, 69)
@@ -92,7 +94,7 @@ RIGHT_CHANNEL = "_audio_level.1"
 
 MONITORING_AVAILABLE = False
 
-# GUI compoents displaying levels
+# GUI components displaying levels
 _monitor_window = None
 _master_volume_meter = None
 
@@ -119,27 +121,31 @@ def init(profile):
             METER_LIGHTS = 100
             METER_HEIGHT = METER_LIGHTS * DASH_INK + (METER_LIGHTS - 1) * DASH_SKIP
         else:
-            METER_SLOT_H = 275
+            METER_SLOT_H = 250
             CONTROL_SLOT_H = 240
             METER_LIGHTS = 82
             METER_HEIGHT = METER_LIGHTS * DASH_INK + (METER_LIGHTS - 1) * DASH_SKIP
 
+    # We want a global ticker object to always exist, so we can use it to know if it is running.
+    launch_update_ticker()
+    _update_ticker.destroy_ticker()
+            
+def launch_update_ticker():
     # We want this to be always present when closing app or we'll need to handle it being missing.
     global _update_ticker
-    _update_ticker = utils.Ticker(_audio_monitor_update, 0.04)
+    _update_ticker = utilsgtk.GtkTicker(_audio_monitor_update, 40)
     _update_ticker.start_ticker()
-    _update_ticker.stop_ticker()    
-
+  
 def init_for_project_load():
     # Monitor window is quaranteed to be closed
     if _update_ticker.running:
-        _update_ticker.stop_ticker()    
+        _update_ticker.destroy_ticker()    
         
     global _level_filters
     _level_filters = None
     _init_level_filters(False)
 
-    _update_ticker.start_ticker()
+    launch_update_ticker()
 
 def update_mute_states():
     if _monitor_window != None:
@@ -148,7 +154,7 @@ def update_mute_states():
 def close():
     close_audio_monitor()
     close_master_meter()
-    _update_ticker.stop_ticker()
+    _update_ticker.destroy_ticker()
 
 def show_audio_monitor():
     global _monitor_window
@@ -161,7 +167,7 @@ def show_audio_monitor():
         
     global _update_ticker
     if _update_ticker.running == False:
-        _update_ticker.start_ticker()
+        launch_update_ticker()
 
 def close_audio_monitor():
     
@@ -195,14 +201,13 @@ def get_master_meter():
     _master_volume_meter = MasterVolumeMeter()
 
     if _update_ticker.running == False:
-        _update_ticker.start_ticker()
+        launch_update_ticker()
 
     align = guiutils.set_margins(_master_volume_meter.widget, 3, 3, 3, 3)
     
     frame = Gtk.Frame()
     frame.add(align)
-    frame.set_shadow_type(Gtk.ShadowType.ETCHED_OUT)
-
+    guiutils.set_margins(frame, 0, 0, 1, 1)
     return frame
 
 def close_master_meter():
@@ -234,7 +239,7 @@ def _init_level_filters(create_track_filters):
     # so when Sequence is saved these filters will automatically be removed.
     # Filters are not part of sequence.Sequence object because they just used for monitoring,
     #
-    # Track/master gain values are persistant, they're also editing desitions 
+    # Track/master gain values are persistent, they're also editing decisions 
     # and are therefore part of sequence.Sequence objects.
     
     # Create levels filters array if it deosn't exist
@@ -258,7 +263,7 @@ def _destroy_level_filters(destroy_track_filters=False):
 
     # We need to be sure that audio level updates are stopped before
     # detaching and destroying them
-    _update_ticker.stop_ticker()
+    _update_ticker.destroy_ticker()
 
     # Detach filters
     if len(_level_filters) != 0:
@@ -267,7 +272,7 @@ def _destroy_level_filters(destroy_track_filters=False):
         if _monitor_window == None and _master_volume_meter == None:
             seq.tractor.detach(_level_filters[0])
 
-        # Track filters are onlty detached when this called from wondow close
+        # Track filters are only detached when this called from window close
         if destroy_track_filters:
             for i in range(1, len(seq.tracks) - 1):
                 seq.tracks[i].detach(_level_filters[i])
@@ -278,10 +283,30 @@ def _destroy_level_filters(destroy_track_filters=False):
         _audio_levels = []
     elif _monitor_window == None:
         _level_filters = [_level_filters[0]]
-        _audio_levels[0] = 0.0
+        _audio_levels[0] = (0.0, 0.0)
 
     if _master_volume_meter != None or _monitor_window != None:
-        _update_ticker.start_ticker()
+        launch_update_ticker()
+
+def recreate_master_meter_filter_for_new_sequence():
+    global _level_filters, _audio_levels
+
+    # We need to be sure that audio level updates are stopped before
+    # detaching and destroying them
+    _update_ticker.destroy_ticker()
+
+    if len(_level_filters) != 0:
+        seq = editorstate.current_sequence()
+        # Only detach master filter if we are displaying audio master meter
+        if _master_volume_meter != None:
+            seq.tractor.detach(_level_filters[0])
+            _level_filters.pop(0)
+            _audio_levels[0] = (0.0, 0.0)
+            master_level_filter = _add_audio_level_filter(seq.tractor, seq.profile)
+            _level_filters.insert(0, master_level_filter)
+
+    if _master_volume_meter != None or _monitor_window != None:
+        launch_update_ticker()
 
 def _add_audio_level_filter(producer, profile):
     audio_level_filter = mlt.Filter(profile, "audiolevel")
@@ -289,18 +314,15 @@ def _add_audio_level_filter(producer, profile):
     producer.attach(audio_level_filter)
     return audio_level_filter
 
-def _audio_monitor_update():
-    # This is not called from gtk thread
-
+def _audio_monitor_update(data):
+    # 'data' is not used here. 
+    # This is called holding Gdk lock so we can do Gtk updates.
     if _monitor_window == None and _master_volume_meter == None:
         return
 
-    Gdk.threads_enter()
-
     global _audio_levels
     _audio_levels = []
     for i in range(0, len(_level_filters)):
-        #print i
         audio_level_filter = _level_filters[i]
         l_val = _get_channel_value(audio_level_filter, LEFT_CHANNEL)
         r_val = _get_channel_value(audio_level_filter, RIGHT_CHANNEL)
@@ -311,8 +333,6 @@ def _audio_monitor_update():
     if _master_volume_meter != None:
         _master_volume_meter.canvas.queue_draw()
 
-    Gdk.threads_leave()
-
 def _get_channel_value(audio_level_filter, channel_property):
     level_value = audio_level_filter.get(channel_property)
     if level_value == None:
@@ -364,7 +384,6 @@ class AudioMonitorWindow(Gtk.Window):
         self.set_title(_("Audio Mixer"))
         self.show_all()
         self.set_resizable(False)
-        self.set_keep_above(True) # Perhaps configurable later
 
     def update_tracks_mute_states(self):
         for i in range(1, len(self.gain_controls)):
@@ -391,10 +410,7 @@ class MetersArea:
     def _draw(self, event, cr, allocation):
         x, y, w, h = allocation
 
-        if editorpersistance.prefs.theme == appconsts.LIGHT_THEME:
-            cr.set_source_rgb(*METER_BG_COLOR)
-        else:
-            cr.set_source_rgb(0.0, 0.0, 0.0)
+        cr.set_source_rgb(0.0, 0.0, 0.0)
         cr.rectangle(0, 0, w, h)
         cr.fill()
 
@@ -422,6 +438,10 @@ class AudioMeter:
         self.x_pad_r = SLOT_W / 2 + 6 - 2
         self.meter_width = METER_WIDTH
 
+    def set_height(self, h):
+        self.left_channel.set_height(h)
+        self.right_channel.set_height(h)
+        
     def display_value(self, cr, x, value_left, value_right, grad):
         cr.set_source(grad)
         cr.set_dash(DASHES, 0) 
@@ -446,6 +466,9 @@ class ChannelMeter:
         self.y_top_pad = Y_TOP_PAD
         self.over_countdown = 0
 
+    def set_height(self, height):
+        self.height = height
+
     def display_value(self, cr, x, value):
         if value > 1.0:
             self.over_countdown = OVER_FRAMES
@@ -492,7 +515,7 @@ class ChannelMeter:
             self.draw_value_line(cr, x, DB_IEC_MINUS_12, "-12", 1)
             self.draw_value_line(cr, x, DB_IEC_MINUS_20, "-20", 1)
             self.draw_value_line(cr, x, DB_IEC_MINUS_40, "-40", 1)
-            self.draw_value_line(cr, x, 0.0,  u"\u221E", 5)
+            self.draw_value_line(cr, x, 0.0,  "\u221E", 5)
             
     def get_meter_y_for_value(self, value):
         y = self.get_y_for_value(value)
@@ -514,7 +537,7 @@ class ChannelMeter:
         self.draw_text(val_text, "Sans 8", cr, x + self.dB_x_pad + x_fine_tune, y - 8 + self.y_top_pad, OVERLAY_COLOR)
         
     def draw_channel_identifier(self, cr, x):
-        self.draw_text(self.channel_text, "Sans Bold 8", cr, x - 4, self.height + 2 +  self.y_top_pad, OVERLAY_COLOR)
+        self.draw_text(self.channel_text, "Sans Bold 8", cr, x - 4, self.height + 2 +  self.y_top_pad, CHANNEL_ID_COLOR)
 
     def draw_text(self, text, font_desc, cr, x, y, color):
         layout = PangoCairo.create_layout(cr)
@@ -544,7 +567,7 @@ class GainControl(Gtk.Frame):
             gain_value = producer.audio_gain # track
         gain_value = gain_value * 100
         
-        self.adjustment = Gtk.Adjustment(value=gain_value, lower=0, upper=100, step_incr=1)
+        self.adjustment = Gtk.Adjustment(value=gain_value, lower=0, upper=100, step_increment=1)
         self.slider = Gtk.VScale()
         self.slider.set_adjustment(self.adjustment)
         self.slider.set_size_request(SLOT_W - 10, CONTROL_SLOT_H - 105)
@@ -559,19 +582,15 @@ class GainControl(Gtk.Frame):
             pan_value = 0.5 # center
         pan_value = (pan_value - 0.5) * 200 # from range 0 - 1 to range -100 - 100
 
-        self.pan_adjustment = Gtk.Adjustment(value=pan_value, lower=-100, upper=100, step_incr=1)
+        self.pan_adjustment = Gtk.Adjustment(value=pan_value, lower=-100, upper=100, step_increment=1)
         self.pan_slider = Gtk.HScale()
         self.pan_slider.set_adjustment(self.pan_adjustment)
         self.pan_slider.connect("value-changed", self.pan_changed)
 
-        self.pan_button = Gtk.ToggleButton(_("Pan"))
+        self.pan_button = Gtk.ToggleButton()
+        surface = guiutils.get_cairo_image("pan_track")
+        self.pan_button.set_image(Gtk.Image.new_from_surface (surface))
         self.pan_button.connect("toggled", self.pan_active_toggled)
-        
-        if pan_value == 0.0:
-            self.pan_slider.set_sensitive(False)
-        else:
-            self.pan_button.set_active(True)
-            self.pan_adjustment.set_value(pan_value) # setting button active sets value = 0, set correct value again
 
         label = guiutils.bold_label(name)
 
@@ -589,6 +608,12 @@ class GainControl(Gtk.Frame):
 
         self.mute_changed()
 
+        if pan_value == 0.0:
+            self.pan_slider.set_sensitive(False)
+        else:
+            self.pan_button.set_active(True)
+            self.pan_adjustment.set_value(pan_value) # setting button active sets value = 0, set correct value again
+            
     def gain_changed(self, slider):
         gain = slider.get_value() / 100.0
         if self.is_master == True:
@@ -632,13 +657,17 @@ class GainControl(Gtk.Frame):
 
 class MasterVolumeMeter:
     def __init__(self):
+        self.H_CUT = 30
+        self.GRAD_CUT = 37
+        self.top_pad = 14
+        
         self.meter = AudioMeter(METER_HEIGHT + 40)
         self.meter.x_pad_l = 6
         self.meter.x_pad_r = 14
         self.meter.right_channel.draw_dB = True
         self.meter.right_channel.dB_x_pad = -14
         self.meter.meter_width = 5
-        self.top_pad = 14
+
         self.meter.right_channel.y_top_pad = self.top_pad 
         self.meter.left_channel.y_top_pad = self.top_pad 
 
@@ -649,20 +678,19 @@ class MasterVolumeMeter:
                                                     self._draw)
 
         self.widget = Gtk.VBox(False, 0)
-        self.widget.pack_start(self.canvas, False, False, 0)
+        self.widget.pack_start(self.canvas, True, True, 0)
 
     def _draw(self, event, cr, allocation):
         x, y, w, h = allocation
 
-        if editorpersistance.prefs.theme == appconsts.LIGHT_THEME:
-            cr.set_source_rgb(*METER_BG_COLOR)
-        else:
-            cr.set_source_rgb(0.0, 0.0, 0.0)
+        self.meter.set_height(h - self.H_CUT)
+ 
+        cr.set_source_rgb(0.0, 0.0, 0.0)
         cr.fill_preserve()
         cr.rectangle(0, 0, w, h)
         cr.fill()
 
-        grad = cairo.LinearGradient (0, self.top_pad, 0, METER_HEIGHT + self.top_pad)
+        grad = cairo.LinearGradient (0, self.top_pad, 0, self.top_pad + h - self.GRAD_CUT)
         grad.add_color_stop_rgba(*RED_1)
         grad.add_color_stop_rgba(*RED_2)
         grad.add_color_stop_rgba(*YELLOW_1)
@@ -671,5 +699,6 @@ class MasterVolumeMeter:
         grad.add_color_stop_rgba(*GREEN_2)
 
         l_value, r_value = _audio_levels[0]
+
         x = 0
         self.meter.display_value(cr, x, l_value, r_value, grad)
