@@ -26,8 +26,6 @@ import copy
 from gi.repository import GLib
 from gi.repository import Gtk
 import pickle
-import threading
-import time
 
 import appconsts
 import atomicfile
@@ -39,6 +37,8 @@ import editorpersistance
 import editorstate
 from editorstate import PROJECT
 from editorstate import PLAYER
+from editorstate import current_sequence
+import extraeditors
 import gui
 import guicomponents
 import guipopover
@@ -46,6 +46,7 @@ import guiutils
 import mltfilters
 import propertyedit
 import propertyeditorbuilder
+import propertyparse
 import respaths
 import translations
 import updater
@@ -90,62 +91,45 @@ class FilterFooterRow:
     def __init__(self, filter_object, filter_stack):
         self.filter_object = filter_object
         self.filter_stack = filter_stack
-        
-        surface = guiutils.get_cairo_image("filter_save")
-        if editorpersistance.prefs.double_track_hights  == False:
-            save_button = guicomponents.PressLaunch(self.save_pressed, surface, w=22, h=22)
+
+        if editorpersistance.prefs.icons_scale == appconsts.ICONS_SCALE_DEFAULT:
+            w=22
+            h=22
         else:
-            save_button = guicomponents.PressLaunch(self.save_pressed, surface, w=44, h=44)
+            w=44
+            h=44
+            
+        surface = guiutils.get_cairo_image("filter_save")
+        save_button = guicomponents.PressLaunch(self.save_pressed, surface, w, h)
         save_button.widget.set_tooltip_markup(_("Save effect values"))
         
         surface = guiutils.get_cairo_image("filter_load")
-        if editorpersistance.prefs.double_track_hights  == False:
-            load_button = guicomponents.PressLaunch(self.load_pressed, surface, w=22, h=22)
-        else:
-            load_button = guicomponents.PressLaunch(self.load_pressed, surface, w=44, h=44)
+        load_button = guicomponents.PressLaunch(self.load_pressed, surface, w, h)
         load_button.widget.set_tooltip_markup(_("Load effect values"))
 
         surface = guiutils.get_cairo_image("filter_reset")
-        if editorpersistance.prefs.double_track_hights  == False:
-            reset_button = guicomponents.PressLaunch(self.reset_pressed, surface, w=22, h=22)
-        else:
-            reset_button = guicomponents.PressLaunch(self.reset_pressed, surface, w=44, h=44)
+        reset_button = guicomponents.PressLaunch(self.reset_pressed, surface, w, h)
         reset_button.widget.set_tooltip_markup(_("Reset effect values"))
         
         surface = guiutils.get_cairo_image("filters_mask_add")
-        if editorpersistance.prefs.double_track_hights  == False:
-            mask_button = guicomponents.PressLaunch(self.add_mask_pressed, surface, w=22, h=22)
-        else:
-            mask_button = guicomponents.PressLaunch(self.add_mask_pressed, surface, w=44, h=44)
+        mask_button = guicomponents.PressLaunch(self.add_mask_pressed, surface, w, h)
         mask_button.widget.set_tooltip_markup(_("Add Filter Mask"))
         self.mask_button = mask_button
         
         surface = guiutils.get_cairo_image("filters_move_up")
-        if editorpersistance.prefs.double_track_hights  == False:
-            move_up_button = guicomponents.PressLaunch(self.move_up_pressed, surface, w=22, h=22)
-        else:
-            move_up_button = guicomponents.PressLaunch(self.move_up_pressed, surface, w=44, h=44)
+        move_up_button = guicomponents.PressLaunch(self.move_up_pressed, surface, w, h)
         move_up_button.widget.set_tooltip_markup(_("Move Filter Up"))
 
         surface = guiutils.get_cairo_image("filters_move_down")
-        if editorpersistance.prefs.double_track_hights  == False:
-            move_down_button = guicomponents.PressLaunch(self.move_down_pressed, surface, w=22, h=22)
-        else:
-            move_down_button = guicomponents.PressLaunch(self.move_down_pressed, surface, w=44, h=44)
+        move_down_button = guicomponents.PressLaunch(self.move_down_pressed, surface, w, h)
         move_down_button.widget.set_tooltip_markup(_("Move Filter Down"))
 
         surface = guiutils.get_cairo_image("filters_move_top")
-        if editorpersistance.prefs.double_track_hights  == False:
-            move_top_button = guicomponents.PressLaunch(self.move_top_pressed, surface, w=22, h=22)
-        else:
-            move_top_button = guicomponents.PressLaunch(self.move_top_pressed, surface, w=44, h=44)
+        move_top_button = guicomponents.PressLaunch(self.move_top_pressed, surface, w, h)
         move_top_button.widget.set_tooltip_markup(_("Move Filter To Top"))
 
         surface = guiutils.get_cairo_image("filters_move_bottom")
-        if editorpersistance.prefs.double_track_hights  == False:
-            move_bottom_button = guicomponents.PressLaunch(self.move_bottom_pressed, surface, w=22, h=22)
-        else:
-            move_bottom_button = guicomponents.PressLaunch(self.move_bottom_pressed, surface, w=44, h=44)
+        move_bottom_button = guicomponents.PressLaunch(self.move_bottom_pressed, surface, w, h)
         move_bottom_button.widget.set_tooltip_markup(_("Move Filter To Bottom"))
         
         self.widget = Gtk.HBox(False, 0)
@@ -304,7 +288,7 @@ class ClipFilterStack:
             stack_item = FilterStackItem(filter_object, edit_panel, self)
             self.filter_stack.append(stack_item)
             self.widget.pack_start(stack_item.widget, False, False, 0)
-        
+
         self.widget.show_all()
 
     def get_filters(self):
@@ -384,6 +368,10 @@ class ClipFilterStack:
         filter_stack_item = self.filter_stack[filter_index]
         filter_stack_item.expander.set_expanded(True)
 
+    def set_filter_item_minified(self, filter_index):
+        filter_stack_item = self.filter_stack[filter_index]
+        filter_stack_item.expander.set_expanded(False)
+        
     def set_all_filters_expanded_state(self, expanded):
         for i in range(0, len(self.filter_stack)):
             stack_item = self.filter_stack[i]
@@ -400,6 +388,13 @@ class ClipFilterStack:
             stack_item = self.filter_stack[i]
             stack_item.expander.set_expanded(state_list[i])
 
+    def set_single_expanded(self, expanded_index):
+        for i in range(0, len(self.filter_stack)):
+            stack_item = self.filter_stack[i]
+            stack_item.expander.set_expanded(False)
+        
+        stack_item = self.filter_stack[expanded_index]
+        stack_item.expander.set_expanded(True)
 
 # -------------------------------------------------------------- GUI INIT
 def get_clip_effects_editor_info_row():
@@ -466,14 +461,6 @@ def set_clip(clip, track, clip_index, show_tab=True):
 
     gui.editor_window.edit_multi.set_visible_child_name(appconsts.EDIT_MULTI_FILTERS)
 
-    #global _edit_polling_thread
-    # Close old polling
-    #if _edit_polling_thread != None:
-    #    _edit_polling_thread.shutdown()
-    # Start new polling
-    #_edit_polling_thread = PropertyChangePollingThread()
-    #_edit_polling_thread.start()
-
 def refresh_clip():
     if _filter_stack == None:
         return 
@@ -571,7 +558,6 @@ def clear_clip():
     show_text_in_edit_area(_("No Clip"))
 
     set_enabled(False)
-    # shutdown_polling()
 
     global keyframe_editor_widgets
     keyframe_editor_widgets = []
@@ -610,14 +596,6 @@ def update_stack(clip, track, clip_index):
 
     widgets.value_edit_box = scroll_window
 
-"""
-def update_stack_changed_blocked():
-    global _block_changed_update
-    _block_changed_update = True
-    update_stack()
-    _block_changed_update = False
-"""
-
 def _alpha_filter_add_maybe_info(filter_info):
     if editorpersistance.prefs.show_alpha_info_message == True and \
        editorstate. current_sequence().compositing_mode != appconsts.COMPOSITING_MODE_STANDARD_FULL_TRACK:
@@ -718,8 +696,9 @@ def do_stack_move(clip, insert_row, delete_row):
     action = edit.move_filter_action(data)
     set_stack_update_blocked()
     action.do_edit()
+    _filter_stack.set_single_expanded(insert_row)
     set_stack_update_unblocked()
-    
+
 def reinit_stack_if_needed(force_update):
     clip, track, clip_index = _filter_stack.get_clip_data()
     if _filter_stack.stack_changed(clip) == True or force_update == True:
@@ -759,6 +738,7 @@ def _get_filter_panel(clip, filter_object, filter_index, track, clip_index):
                 or (editor_type == propertyeditorbuilder.KEYFRAME_EDITOR_RELEASE)
                 or (editor_type == propertyeditorbuilder.KEYFRAME_EDITOR_CLIP)
                 or (editor_type == propertyeditorbuilder.FILTER_RECT_GEOM_EDITOR)
+                or (editor_type == propertyeditorbuilder.NO_KF_RECT)
                 or (editor_type == propertyeditorbuilder.KEYFRAME_EDITOR_CLIP_FADE_FILTER)):
                     keyframe_editor_widgets.append(editor_row)
                     filter_keyframe_editor_widgets.append(editor_row)
@@ -772,20 +752,36 @@ def _get_filter_panel(clip, filter_object, filter_index, track, clip_index):
             if not hasattr(editor_row, "no_separator"):
                 vbox.pack_start(guicomponents.EditorSeparator().widget, False, False, 0)
             
+            # Some editors need to be accessed by extraeditors for controlling their state.
+            try:
+                name = ep.args[propertyedit.ACCESSABLE_EDITOR]
+                filter_index = ep.filter_index
+                extraeditors.accessable_editors[name + ":" + str(filter_index)] = editor_row
+            except Exception as e:
+                pass
+
         # Create NonMltEditableProperty wrappers for properties
         non_mlteditable_properties = propertyedit.get_non_mlt_editable_properties( clip, 
                                                                                    filter_object,
-                                                                                   filter_index)
+                                                                                   filter_index,
+                                                                                   track,
+                                                                                   clip_index)
 
         # Extra editors. Editable properties may have already been created 
         # with "editor=no_editor" and now extra editors may be created to edit those
         # Non mlt properties are added as these are only needed with extraeditors
         editable_properties.extend(non_mlteditable_properties)
-        editor_rows = propertyeditorbuilder.get_filter_extra_editor_rows(filter_object, editable_properties)
+        editor_rows = propertyeditorbuilder.get_filter_extra_editor_rows(filter_object, editable_properties, track, clip_index)
         for editor_row in editor_rows:
             vbox.pack_start(editor_row, False, False, 0)
             if not hasattr(editor_row, "no_separator"):
                 vbox.pack_start(guicomponents.EditorSeparator().widget, False, False, 0)
+            if hasattr(editor_row, "kf_edit_geom_editor"):
+                # kf_edit_geom_editor is keyframeeditor.FilterRotatingGeometryEditor,
+                # the widget that actually needs to be updated.
+                # editor_row is just Gtk.VBox.
+                keyframe_editor_widgets.append(editor_row.kf_edit_geom_editor)
+                filter_keyframe_editor_widgets.append(editor_row.kf_edit_geom_editor)
     else:
         vbox.pack_start(Gtk.Label(label=_("No editable parameters")), True, True, 0)
     vbox.show_all()
@@ -906,7 +902,7 @@ def _filter_mask_item_activated(action, variant, data):
     set_stack_update_unblocked()
 
     set_clip(clip, track, clip_index)
-    _filter_stack.set_filter_item_expanded(current_filter_index + 1)
+    _filter_stack.set_single_expanded(index_1)
 
 
 # ------------------------------------------------ SAVE, LOAD etc. from hamburger menu
@@ -1022,6 +1018,8 @@ def _load_effect_stack_values_dialog_callback(dialog, response_id):
 
 def _reset_filter_values(filter_object):
         filter_object.properties = copy.deepcopy(filter_object.info.properties)
+        propertyparse.replace_value_keywords(filter_object.properties, current_sequence().profile)
+        
         filter_object.non_mlt_properties = copy.deepcopy(filter_object.info.non_mlt_properties)
         filter_object.update_mlt_filter_properties_all()
                 
