@@ -18,7 +18,6 @@
     along with Flowblade Movie Editor. If not, see <http://www.gnu.org/licenses/>.
 """
 from gi.repository import Gtk
-from gi.repository import Gdk
 from gi.repository import GLib
 
 import cairo
@@ -37,7 +36,7 @@ import threading
 import time
 
 import appconsts
-import atomicfile
+import callbackbridge
 import dialogutils
 import edit
 import editorstate
@@ -82,8 +81,6 @@ FLUXITY_TYPE_ICON = None
 
 NEWLINE = '\n'
 
-set_plugin_to_be_edited_func = None
-get_edited_plugin_clip = None
 
 # ----------------------------------------------------- interface
 def get_action_object(container_data):
@@ -355,7 +352,7 @@ class AbstractContainerActionObject:
 
         return rendered_clip
 
-    def set_video_endoding(self, clip, callback=None):
+    def set_video_endoding(self, clip, callback=None, show_render_buttons=False):
         self.external_encoding_callback = callback
         current_profile_index = mltprofiles.get_profile_index_for_profile(current_sequence().profile)
         # These need to re-initialized always when using this module.
@@ -367,27 +364,51 @@ class AbstractContainerActionObject:
             self.container_data.render_data = toolsencoding.create_container_clip_default_render_data_object(current_sequence().profile)
             
         encoding_panel = toolsencoding.get_encoding_panel(self.container_data.render_data, True)
+        if show_render_buttons == True:
+            render_combo = Gtk.ComboBoxText()
+            render_combo.append_text(_("Render Full Media"))
+            render_combo.append_text(_("Render Clip Length"))
+            render_combo.set_active(0)
 
-        align = dialogutils.get_default_alignment(encoding_panel)
+        vbox = Gtk.VBox()
+        if show_render_buttons == True:
+            vbox.pack_start(render_combo, False, False, 0)
+        vbox.pack_start(encoding_panel, False, False, 0)
         
-        dialog = Gtk.Dialog(_("Container Clip Render Settings"),
+        align = dialogutils.get_default_alignment(vbox)
+
+        if show_render_buttons == True:
+            title = _("Container/Generator Clip Render")
+            ok_text =  _("Render")
+        else:
+            title = _("Set Render Encoding Settings")
+            ok_text =  _("Set Encoding")
+                        
+        dialog = Gtk.Dialog(title,
                             gui.editor_window.window,
                             Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
                             (_("Cancel"), Gtk.ResponseType.REJECT,
-                             _("Set Encoding"), Gtk.ResponseType.ACCEPT))
+                             ok_text, Gtk.ResponseType.ACCEPT))
         dialog.vbox.pack_start(align, True, True, 0)
         dialogutils.set_outer_margins(dialog.vbox)
         dialog.set_resizable(False)
-
         dialog.connect('response', self.encode_settings_callback)
+                    
+        if show_render_buttons == True:
+            self.render_combo = render_combo
+        else:
+            self.render_combo = None
         dialog.show_all()
 
     def encode_settings_callback(self, dialog, response_id):
         if response_id == Gtk.ResponseType.ACCEPT:
             self.container_data.render_data = toolsencoding.get_render_data_for_current_selections()
             if self.external_encoding_callback != None:
-                self.external_encoding_callback(self.container_data.render_data)
-
+                if self.render_combo == None:
+                    self.external_encoding_callback(self.container_data.render_data)
+                else:
+                    self.external_encoding_callback(self.render_combo)
+                    
         dialog.destroy()
         
     def clone_clip(self, old_clip):
@@ -454,7 +475,7 @@ class GMicContainerActions(AbstractContainerActionObject):
             script_file = open(self.container_data.program)
             user_script = script_file.read()
             test_out_file = userfolders.get_cache_dir()  + "/gmic_cont_clip_test.png"
-            test_in_file = str(respaths.IMAGE_PATH + "unrendered_blender.png") # we just need some valid input
+            test_in_file = str(respaths.IMAGE_PATH + "black.jpg") # we just need some valid input
 
             # Create command list and launch process.
             command_list = [editorstate.gmic_path, test_in_file]
@@ -546,7 +567,7 @@ class GMicContainerActions(AbstractContainerActionObject):
                 steps_count = 3
                 if  self.container_data.render_data.do_video_render == False:
                     steps_count = 2
-                msg = _("Step ") + str(step) + " / " + str(steps_count) + " - "
+                msg = _("G'Mic Render Step ") + str(step) + " / " + str(steps_count) + " - "
                 if step == "1":
                     msg += _("Writing Clip Frames")
                 elif step == "2":
@@ -724,7 +745,7 @@ class FluxityContainerActions(AbstractContainerActionObject):
                 steps_count = 2
                 if self.container_data.render_data.do_video_render == False:
                     steps_count = 1
-                msg = _("Step ") + str(step) + " / " + str(steps_count) + " - "
+                msg = _("Generator Render ") + str(step) + " / " + str(steps_count) + " - "
                 if step == "1":
                     msg += _("Writing Clip Frames")
                 else:
@@ -766,8 +787,8 @@ class FluxityContainerActions(AbstractContainerActionObject):
 
     def plugin_tline_render_comlete(self):
         clip = self.create_producer_and_do_update_edit(None)
-        # Reopen in edit panel, doing callback to avoid circular imports
-        set_plugin_to_be_edited_func(clip, self)
+        # Reopen in edit panel.
+        callbackbridge.mediaplugin_set_plugin_to_be_edited(clip, self)
 
     def abort_render(self):
         fluxityheadless.abort_render(self.get_container_program_id())
@@ -786,13 +807,13 @@ class FluxityContainerActions(AbstractContainerActionObject):
         return (surface, length, icon_path)
 
     def edit_program(self, clip):
-        set_plugin_to_be_edited_func(clip, self)
+        callbackbridge.mediaplugin_set_plugin_to_be_edited(clip, self)
         gui.editor_window.edit_multi.set_visible_child_name(appconsts.EDIT_MULTI_PLUGINS)
 
     def apply_editors(self, editors):
         new_editors_list = self.get_editors_data_as_editors_list(editors)
         self.container_data.data_slots["fluxity_plugin_edit_data"]["editors_list"] = new_editors_list
-        get_edited_plugin_clip().container_data.data_slots["fluxity_plugin_edit_data"]["editors_list"] = new_editors_list
+        callbackbridge.mediaplugin_get_clip().container_data.data_slots["fluxity_plugin_edit_data"]["editors_list"] = new_editors_list
 
     def render_fluxity_preview(self, callbacks, editors, preview_frame):
         self.create_data_dirs_if_needed() # This could be first time we are writing
@@ -918,9 +939,7 @@ class MLTXMLContainerActions(AbstractContainerActionObject):
                 fraction, elapsed = status
 
                 if self.container_data.render_data.do_video_render == True:
-                    msg = _("Video for: ") + self.clip.name 
-                #elif step == "2":
-                #    msg = _("Image Sequence for: ") + self.clip.name 
+                    msg = _("Video for Selection Clip: ") + self.clip.name 
 
                 job_msg = self.get_job_queue_message()
                 job_msg.progress = float(fraction)
