@@ -18,11 +18,9 @@
     along with Flowblade Movie Editor. If not, see <http://www.gnu.org/licenses/>.
 """
 
-from gi.repository import Gtk, Gdk, GLib
+from gi.repository import Gtk, GLib
 
-import copy
 import hashlib
-import json
 import os
 import shutil
 import threading
@@ -30,16 +28,15 @@ import time
 
 import appconsts
 import containeractions
-import dialogs
 import dialogutils
-import editorstate
 from editorstate import PROJECT
 from editorstate import current_sequence
 import gui
-import guicomponents
 import guiutils
+import gtkbuilder
 import projectaction
 import respaths
+import toolsencoding
 import updater
 import userfolders
 import utils
@@ -112,6 +109,11 @@ class ContainerClipData:
 
 
 # -------------------------------------------------------- Clip menu actions
+
+def render_tline_generator_clip(clip, callback):
+    action_object = containeractions.get_action_object(clip.container_data)
+    action_object.set_video_endoding(None, callback, True)
+
 def render_full_media(data):
     clip, track, item_id, item_data = data
     action_object = containeractions.get_action_object(clip.container_data)
@@ -152,7 +154,7 @@ def clone_clip(clip):
 def _get_file_select_row_and_editor(label_text, file_filter=None, title=None):
     if title == None:
         title = _("Select A File")
-    file_chooser = Gtk.FileChooserButton.new(title, Gtk.FileChooserAction.OPEN)
+    file_chooser = gtkbuilder.get_file_chooser_button(title, Gtk.FileChooserAction.OPEN)
     file_chooser.set_size_request(250, 25)
     file_chooser.set_current_folder(os.path.expanduser("~") + "/")
 
@@ -167,8 +169,8 @@ def _open_rows_dialog(callback, title, rows, data):
     cancel_str = _("Cancel")
     ok_str = _("Ok")
     dialog = Gtk.Dialog(title,
-                        parent_window,
-                        Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
+                        None,
+                        None,
                         (cancel_str, Gtk.ResponseType.CANCEL,
                         ok_str, Gtk.ResponseType.ACCEPT))
 
@@ -284,11 +286,21 @@ def _fluxity_clip_create_dialog_callback(dialog, response_id, data):
 
 # ------------------------------------------------------- ADDING GENERATOR FROM DIALOG AS MEDIA ITEM
 # Called when user selects 'Add Generator' in with option 'Add as Container Clip'.
-def create_fluxity_media_item_from_plugin(script_file, screenshot_file, plugin_data, import_callback=None):
+def create_fluxity_media_item_from_plugin(script_file, screenshot_file, plugin_data, import_callback=None, default_render=None):
     container_data = ContainerClipData(appconsts.CONTAINER_CLIP_FLUXITY, script_file, None)
     container_data.data_slots["icon_file"] = screenshot_file
     container_data.data_slots["fluxity_plugin_edit_data"] = plugin_data
-    
+
+    # Create and set default render options data.
+    if default_render == appconsts.DEFAULT_RENDER_CLIP:
+        container_data.render_data = toolsencoding.create_container_clip_default_render_data_object(PROJECT().profile)
+        container_data.render_data.do_video_render = True
+    elif default_render == appconsts.DEFAULT_RENDER_ALPHA_CLIP:
+        render_data = toolsencoding.create_container_clip_default_alpha_render_data_object(PROJECT().profile)
+        if render_data != None:
+            container_data.render_data = render_data
+            container_data.render_data.do_video_render = True
+
     # Set callback
     global _media_import_callback
     _media_import_callback = import_callback
@@ -420,6 +432,7 @@ class ContainerClipMediaItem:
         self.type = container_data.unrendered_type
         self.icon = None # cairo.ImageSurface
         self.icon_path = None
+        self.rating = appconsts.MEDIA_FILE_UNRATED
         
         self.mark_in = -1
         self.mark_out = -1
