@@ -32,6 +32,7 @@ import shutil
 
 import appconsts
 import audiosync
+import callbackbridge
 import clipeffectseditor
 import clipenddragmode
 import compositeeditor
@@ -65,9 +66,9 @@ import userfolders
 
 
 # functions are monkeypatched in at app.py 
-display_clip_menu_pop_up = None
-compositor_menu_item_activated = None
-set_compositor_data = None
+
+#compositor_menu_item_activated = None
+#set_compositor_data = None
 
 # ----------------------------- module funcs
 def do_clip_insert(track, new_clip, tline_pos, use_clip_in=False):
@@ -291,7 +292,7 @@ def tline_canvas_mouse_pressed(event, frame):
                 PLAYER().seek_frame(frame)
             # Right mouse on timeline seeks frame
             else:
-                success = display_clip_menu_pop_up(event.y, event, frame)
+                success = callbackbridge.clipmenuaction_display_clip_menu(event.y, event, frame)
                 if not success:
                     PLAYER().seek_frame(frame)
         return
@@ -306,27 +307,20 @@ def tline_canvas_mouse_pressed(event, frame):
     if editorstate.current_is_move_mode() and timeline_visible():
         hit_compositor = tlinewidgets.compositor_hit(frame, event.x, event.y, current_sequence().compositors)
         if hit_compositor != None:
-            if editorstate.get_compositing_mode() == appconsts.COMPOSITING_MODE_STANDARD_AUTO_FOLLOW:
-                compositeeditor.set_compositor(hit_compositor)
-                compositormodes.set_compositor_selected(hit_compositor)
-                movemodes.clear_selected_clips()
-                editorstate.timeline_mouse_disabled = True
+            movemodes.clear_selected_clips()
+            if event.button == 1 or (event.button == 3 and event.get_state() & Gdk.ModifierType.CONTROL_MASK):
+                compositormodes.set_compositor_mode(hit_compositor)
+                mode_funcs = EDIT_MODE_FUNCS[editorstate.COMPOSITOR_EDIT]
+                press_func = mode_funcs[TL_MOUSE_PRESS]
+                press_func(event, frame)
                 return
-            elif editorstate.auto_follow_active() == False or hit_compositor.obey_autofollow == False:
-                movemodes.clear_selected_clips()
-                if event.button == 1 or (event.button == 3 and event.get_state() & Gdk.ModifierType.CONTROL_MASK):
-                    compositormodes.set_compositor_mode(hit_compositor)
-                    mode_funcs = EDIT_MODE_FUNCS[editorstate.COMPOSITOR_EDIT]
-                    press_func = mode_funcs[TL_MOUSE_PRESS]
-                    press_func(event, frame)
-                    return
             if event.button == 3:
                 compositormodes.set_compositor_selected(hit_compositor)
-                set_compositor_data(hit_compositor)
+                callbackbridge.clipmenuaction_set_compositor_data(hit_compositor)
                 guipopoverclip.compositor_popover_menu_show(gui.tline_canvas.widget,
                                                             event.x, event.y, 
                                                             hit_compositor, 
-                                                            compositor_menu_item_activated)
+                                                            callbackbridge.clipmenuaction_compositor_menu_item_activated)
                 return
             elif event.button == 2:
                 updater.zoom_project_length()
@@ -350,7 +344,7 @@ def tline_canvas_mouse_pressed(event, frame):
     if (event.button == 3 and EDIT_MODE() != editorstate.CLIP_END_DRAG and EDIT_MODE() != editorstate.KF_TOOL):
         if ((not editorstate.current_is_active_trim_mode()) and timeline_visible()):
             if not(event.get_state() & Gdk.ModifierType.CONTROL_MASK):
-                success = display_clip_menu_pop_up(event.y, event, frame)
+                success = callbackbridge.clipmenuaction_display_clip_menu(event.y, event, frame)
                 if not success:
                     PLAYER().seek_frame(frame)
         else:
@@ -657,6 +651,7 @@ def maybe_autorender_plugin(clip):
     if clip.container_data == None:
         return
 
+    print("---------------renderdata:", clip.container_data.render_data)
     if editorpersistance.prefs.auto_render_media_plugins == True:
         if clip.container_data.container_type == appconsts.CONTAINER_CLIP_FLUXITY:
             action_object = containeractions.get_action_object(clip.container_data)
