@@ -27,10 +27,9 @@ import pickle
 
 import appconsts
 import atomicfile
-import mltprofiles
+import databridge
 import userfolders
-import utils # this needs to also go to not load Gtk for background rendering process
-import usbhid
+import utils
 
 
 PREFS_DOC = "prefs"
@@ -42,6 +41,12 @@ UNDO_STACK_MIN = 10
 UNDO_STACK_MAX = 100
 FILTER_SELECT_WIDTH_MIN = 220
 FILTER_SELECT_WIDTH_MAX = 350
+PROJECT_PANEL_WIDTH_MIN = 225
+PROJECT_PANEL_WIDTH_MAX = 325
+EDIT_PANEL_WIDTH_MIN = appconsts.PANEL_MULTI_EDIT_MINIMUM_SIZE
+EDIT_PANEL_WIDTH_MAX = 650
+MEDIA_PANEL_WIDTH_MIN = appconsts.PANEL_MEDIA_MINIMUM_SIZE
+MEDIA_PANEL_WIDTH_MAX = 510
 
 GLASS_STYLE = 0
 SIMPLE_STYLE = 1
@@ -67,7 +72,7 @@ def load():
             write_file = afw.get_file()
             pickle.dump(prefs, write_file)
 
-    # Override deprecated preferences to default values.
+    # Override deprecated preferences/values to default values.
     prefs.delta_overlay = True
     prefs.auto_play_in_clip_monitor = False
     prefs.empty_click_exits_trims = True
@@ -76,6 +81,7 @@ def load():
     prefs.buttons_style = NO_DECORATIONS 
     prefs.buttons_style = NO_DECORATIONS 
     prefs.show_tool_tooltips = True
+    prefs.theme = appconsts.FLOWBLADE_THEME_NEUTRAL
 
     try:
         recent_projects = utils.unpickle(recents_file_path)
@@ -111,7 +117,7 @@ def load():
             write_file = afw.get_file()
             pickle.dump(prefs, write_file)
         print("prefs updated to new version, new param count:", len(prefs.__dict__))
-
+        
 def save():
     """
     Write out prefs and recent_projects files
@@ -186,8 +192,8 @@ def update_prefs_from_widgets(widgets_tuples_tuple):
     auto_center_check, play_pause_button, timeline_start_end_button, auto_center_on_updown, \
     ffwd_rev_shift_spin, ffwd_rev_caps_spin, follow_move_range, loop_clips = playback_prefs_widgets
     
-    force_language_combo, disp_splash, window_mode_combo, full_names, double_track_hights, \
-    top_row_layout, layout_monitor, filter_select_width_spin = view_prefs_widgets
+    force_language_combo, disp_splash, window_mode_combo, full_names, tracks_combo, icons_combo, project_panel_width_spin, \
+    edit_panel_width_spin, media_panel_width_spin, layout_monitor, filter_select_width_spin = view_prefs_widgets
 
     perf_render_threads, perf_drop_frames = performance_widgets
 
@@ -196,7 +202,7 @@ def update_prefs_from_widgets(widgets_tuples_tuple):
     global prefs
     prefs.open_in_last_opended_media_dir = open_in_last_opened_check.get_active()
     prefs.remember_last_render_dir = open_in_last_rendered_check.get_active()
-    prefs.default_profile_name = mltprofiles.get_profile_name_for_index(default_profile_combo.get_active())
+    prefs.default_profile_name = databridge.mltprofiles_get_profile_name_for_index(default_profile_combo.get_active())
     prefs.undos_max = undo_max_spin.get_adjustment().get_value()
     prefs.media_load_order = load_order_combo.get_active()
 
@@ -205,9 +211,7 @@ def update_prefs_from_widgets(widgets_tuples_tuple):
     prefs.trans_cover_delete = cover_delete.get_active()
 
     prefs.play_pause = play_pause_button.get_active()
-# ------------------------------ timeline_start_end_button
     prefs.timeline_start_end = timeline_start_end_button.get_active()
-# -------------------------------End of timeline_start_end_button
     prefs.hide_file_ext = hide_file_ext_button.get_active()
     prefs.mouse_scroll_action_is_zoom = (mouse_scroll_action.get_active() == 0)
     prefs.scroll_horizontal_dir_up_forward = (hor_scroll_dir.get_active() == 0)
@@ -225,9 +229,9 @@ def update_prefs_from_widgets(widgets_tuples_tuple):
     prefs.perf_drop_frames = perf_drop_frames.get_active()
     prefs.show_full_file_names = full_names.get_active()
     prefs.center_on_arrow_move = auto_center_on_updown.get_active()
-    prefs.double_track_hights = (double_track_hights.get_active() == 1)
+    prefs.tracks_scale = tracks_combo.get_active()
+    prefs.icons_scale = icons_combo.get_active()
     prefs.playback_follow_move_tline_range = follow_move_range.get_active()
-    prefs.top_row_layout = top_row_layout.get_active()
     prefs.auto_save_delay_value_index = autosave_combo.get_active()
     prefs.layout_display_index = layout_monitor.get_active()
     if len(render_folder_select.get_filenames()) != 0:
@@ -240,7 +244,7 @@ def update_prefs_from_widgets(widgets_tuples_tuple):
     if usbhid_config_index == 0:
         prefs.usbhid_config = None
     else:
-        usbhid_config_metadata_list = usbhid.get_usb_hid_device_config_metadata_list()
+        usbhid_config_metadata_list = databridge.usbhid_get_usb_hid_device_config_metadata_list()
         if len(usbhid_config_metadata_list) >= (usbhid_config_index - 1):
             # index 0 in the combo box is "None", the -1 offsets the zero-based index from the list
             # with the 1-based index from the combo box in the preferences GUI
@@ -250,6 +254,9 @@ def update_prefs_from_widgets(widgets_tuples_tuple):
     # --------------------------------- USB HID END
     prefs.dnd_action = dnd_action.get_active()
     prefs.filter_select_width = int(filter_select_width_spin.get_value())
+    prefs.editor_panel_width = int(edit_panel_width_spin.get_value())
+    prefs.project_panel_width = int(project_panel_width_spin.get_value())
+    prefs.media_panel_width = int(media_panel_width_spin.get_value())
 
 def get_graphics_default_in_out_length():
     in_fr = int(15000/2) - int(prefs.default_grfx_length/2)
@@ -330,7 +337,7 @@ class EditorPreferences:
         self.ffwd_rev_ctrl = 10 # DEPRECATED, CTRL + arrow now does 10 fps slowmo playback
         self.ffwd_rev_caps = 1
         self.shortcuts = "flowblade.xml"
-        self.double_track_hights = False
+        self.double_track_hights = False # DEPRECATED, tracks_size, icons_size and gui_items_size replace this
         self.delta_overlay = True # DEPRECATED, NOT USER SETTABLE ANYMORE.
         self.show_alpha_info_message = True
         self.playback_follow_move_tline_range = True
@@ -375,3 +382,8 @@ class EditorPreferences:
         self.usbhid_config = None
         self.zoom_to_playhead = True
         self.filter_select_width = 220
+        self.tracks_scale = appconsts.TRACKS_SCALE_DEFAULT
+        self.icons_scale = appconsts.ICONS_SCALE_DEFAULT
+        self.project_panel_width = PROJECT_PANEL_WIDTH_MIN
+        self.editor_panel_width = EDIT_PANEL_WIDTH_MIN 
+        self.media_panel_width = MEDIA_PANEL_WIDTH_MIN
