@@ -47,10 +47,6 @@ CUT = 13
 KF_TOOL = 14
 MULTI_TRIM = 15
 
-# SDL version (Not used currently)
-SDL_1 = 1
-SDL_2 = 2
-
 # Gtk.Application object for main app. 
 app = None
 
@@ -100,7 +96,7 @@ SCREEN_WIDTH = -1
 # Runtime environment data
 gtk_version = None
 mlt_version = None
-appversion = "2.16.3"
+appversion = "2.18.1"
 RUNNING_FROM_INSTALLATION = 0
 RUNNING_FROM_DEV_VERSION = 1
 RUNNING_FROM_FLATPAK = 2
@@ -153,6 +149,9 @@ _trim_clips_cache = {}
 # Value set in gmic.py module. 
 gmic_path = None
 
+# For development and test use.
+force_sdl2 = False 
+
 def current_is_move_mode():
     if ((edit_mode == INSERT_MOVE) or (edit_mode == OVERWRITE_MOVE) or (edit_mode == MULTI_MOVE)):
         return True
@@ -197,12 +196,6 @@ def EDIT_MODE():
 def MONITOR_MEDIA_FILE():
     return _monitor_media_file
 
-def auto_follow_active():
-    if get_compositing_mode() == appconsts.COMPOSITING_MODE_TOP_DOWN_FREE_MOVE:
-        return False
-    else:
-        return True
-
 def get_compositing_mode():
     if project.c_seq == None:
         print ("get_compositing_mode(), trying to get compositing mode when no current sequence available!") 
@@ -229,7 +222,7 @@ def mlt_version_is_greater_correct(test_version):
         if runtime_ver[1] > test_ver[1]:
             return True
         elif runtime_ver[1] == test_ver[1]:
-            if  runtime_ver[2] >  test_ver[2]:
+            if  runtime_ver[2] > test_ver[2]:
                 return True
     
     return False
@@ -279,7 +272,7 @@ def screen_size_large_width():
         return True
     else:
         return False
-        
+
 def screen_size_small():
     if screen_size_small_height() == True or screen_size_small_width() == True:
         return True
@@ -304,8 +297,9 @@ def add_cached_trim_clip(clip):
 def clear_trim_clip_cache():
     global _trim_clips_cache
     _trim_clips_cache = {}
-
-
+ 
+ 
+ 
 # Called from tline "motion_notify_event" when drag is not on.
 # This is only used by multitrimmode.py to have data to enter trims with keyboard correctly.
 def set_mouse_current_non_drag_pos(x, y):
