@@ -22,24 +22,20 @@
 Module contains the main editor window object and timeline cursor handling.
 """
 
-import cairo
-
 from gi.repository import Gtk
-from gi.repository import Gdk
 
-import app
+
 import appconsts
 import audiomonitoring
 import audiosync
 import batchrendering
-import boxmove
+import callbackbridge
 import clipeffectseditor
 import clipmenuaction
 import compositeeditor
 import containerclip
 import dialogs
 import dialogutils
-import diskcachemanagement
 import dnd
 import editevent
 import editorlayout
@@ -50,8 +46,8 @@ import glassbuttons
 import gmic
 import gui
 import guicomponents
-import guipopover
 import guiutils
+import gtkbuilder
 import jobs
 import keyevents
 import medialinker
@@ -76,6 +72,7 @@ import projectinfogui
 import proxyediting
 import scripttool
 import shortcuts
+import shortcutsdialog
 import singletracktransition
 import titler
 import tlineaction
@@ -124,10 +121,10 @@ class EditorWindow:
             self.window2 = Gtk.Window(Gtk.WindowType.TOPLEVEL)
             self.window2.set_icon_from_file(respaths.IMAGE_PATH + "flowbladeappicon.png")
             self.window2.set_border_width(5)
-            self.window2.connect("delete-event", lambda w, e:app.shutdown())
+            self.window2.connect("delete-event", lambda w, e:callbackbridge.app_shutdown())
 
         # To ask confirmation for shutdown
-        self.window.connect("delete-event", lambda w, e:app.shutdown())
+        self.window.connect("delete-event", lambda w, e:callbackbridge.app_shutdown())
 
         # Player consumer has to be stopped and started when window resized
         self.window.connect("window-state-event", lambda w, e:updater.refresh_player(e))
@@ -190,7 +187,7 @@ class EditorWindow:
                     self.window2.maximize()
                 else:
                     self.window2.resize(w, h)
-
+ 
             self.window2.move(x, y)
             self.window2.show_all()
 
@@ -207,7 +204,7 @@ class EditorWindow:
 
     def _get_app_pane(self): 
         # VPaned top row / timeline
-        self.app_v_paned = Gtk.VPaned()
+        self.app_v_paned = gtkbuilder.VPaned()
         self.app_v_paned.pack1(self.top_row_hbox, resize=False, shrink=False)
         self.app_v_paned.pack2(self.tline_pane, resize=True, shrink=False)
 
@@ -231,11 +228,14 @@ class EditorWindow:
         monitor_source_box.pack_start(Gtk.Label(), True, True, 0)
         monitor_source_box.pack_start(self.monitor_tc_info.monitor_source, False, False, 0)
         monitor_source_box.pack_start(self.monitor_tc_info.monitor_tc, False, False, 0)
+        monitor_desc_panel = projectinfogui.get_top_level_project_info_panel()
+        self.monitor_desc_label = projectinfogui.widgets.monitor_desc_label
+        monitor_source_box.pack_start(monitor_desc_panel, False, False, 0)
         monitor_source_box.pack_start(Gtk.Label(), True, True, 0)
 
         fullscreen_icon = guiutils.get_cairo_image("fullscreen")
         fullscreen_exit_icon = guiutils.get_cairo_image("fullscreen_exit")
-        if editorpersistance.prefs.double_track_hights == False:
+        if guiutils.double_icon_size() == False:
             self.fullscreen_press = guicomponents.PressLaunch(menuactions.toggle_fullscreen, fullscreen_icon, 20, 12)
         else:
             self.fullscreen_press = guicomponents.PressLaunch(menuactions.toggle_fullscreen, fullscreen_icon, 40, 24)
@@ -249,7 +249,7 @@ class EditorWindow:
         self.fullscreen_press.fullscreen_exit_icon = fullscreen_exit_icon
 
         icon_2 = guiutils.get_cairo_image("layout")
-        if editorpersistance.prefs.double_track_hights == False:
+        if guiutils.double_icon_size() == False:
             layout_press = guicomponents.PressLaunchPopover(editorlayout.show_layout_press_menu, icon_2, 24, 12)
         else:
             layout_press = guicomponents.PressLaunchPopover(editorlayout.show_layout_press_menu, icon_2, 48, 24)
@@ -287,9 +287,7 @@ class EditorWindow:
                 menu_vbox.pack_start(monitor_source_box, False, False, 0)
                 menu_vbox.pack_start(guiutils.pad_label(40, 2), False, False, 0)
             menu_vbox.pack_start(tline_info_box, True, True, 0)
-            
-            if editorpersistance.prefs.theme == appconsts.FLOWBLADE_THEME_NEUTRAL:
-                menu_vbox.override_background_color(Gtk.StateFlags.NORMAL, gui.get_mid_neutral_color())
+            menu_vbox.override_background_color(Gtk.StateFlags.NORMAL, gui.get_mid_neutral_color())
         else:
             menubar_box.pack_start(self.tools_buttons.widget, False, False, 0)
             menubar_box.pack_start(guiutils.pad_label(8, 2), False, False, 0)
@@ -355,7 +353,7 @@ class EditorWindow:
             self.mm_paned.add(media_panel)
 
         else:
-            self.mm_paned = Gtk.HPaned()
+            self.mm_paned = gtkbuilder.HPaned()
             guiutils.set_margins(self.bins_panel, 6, 6, 8, 0)
             self.mm_paned.pack1(self.bins_panel, resize=True, shrink=True)
             self.mm_paned.pack2(media_panel, resize=True, shrink=False)
@@ -483,12 +481,13 @@ class EditorWindow:
         # Project panel
         if editorlayout.top_level_project_panel() == True:
             # Project info
-            project_info_panel = projectinfogui.get_top_level_project_info_panel()
+            #project_info_panel = projectinfogui.get_top_level_project_info_panel()
             top_project_vbox = Gtk.VBox()
-            top_project_vbox.pack_start(project_info_panel, False, False, 0)
+            #top_project_vbox.pack_start(project_info_panel, False, False, 0)
             top_project_vbox.pack_start(self.bins_panel, True, True, 0)
             top_project_vbox.pack_start(seq_panel, True, True, 0)
-            self.top_project_panel = guiutils.set_margins(top_project_vbox, 0, 0, 0, 4)
+
+            self.top_project_panel = guiutils.set_margins(top_project_vbox, 4, 4, 0, 4)
             self.project_panel = None
         else:
             # Notebook project panel for smallest screens
@@ -575,8 +574,7 @@ class EditorWindow:
         self.edit_buttons_frame.add(self.edit_buttons_row)
         guiutils.set_margins(self.edit_buttons_frame, 1, 0, 0, 0)
 
-        if editorpersistance.prefs.theme == appconsts.FLOWBLADE_THEME_NEUTRAL:
-            self.edit_buttons_frame.override_background_color(Gtk.StateFlags.NORMAL, gui.get_mid_neutral_color())
+        self.edit_buttons_frame.override_background_color(Gtk.StateFlags.NORMAL, gui.get_mid_neutral_color())
             
     def _init_tline(self):
         self.tline_scale = tlinewidgets.TimeLineFrameScale(modesetting.set_default_edit_mode,
@@ -770,7 +768,7 @@ class EditorWindow:
             self.left_column_frame = guiutils.get_empty_panel_etched_frame() # to be filled later if panels are added into this position.
 
         # Top row paned
-        self.top_paned = Gtk.HPaned()
+        self.top_paned = gtkbuilder.HPaned()
         if editorpersistance.prefs.global_layout == appconsts.SINGLE_WINDOW:
             self.top_paned.pack1(self.notebook_frame , resize=True, shrink=False)
             self.top_paned.pack2(self.monitor_frame, resize=True, shrink=False)
@@ -810,7 +808,7 @@ class EditorWindow:
             ('ExportScreenshot', None, _('Current Frame'), None, None, lambda a:exporting.screenshot_export()),
             ('ExportToArdour', None, _('Current Sequence Audio As Ardour Session'), None, None, lambda a:exporting.ardour_export()),
             ('Close', None, _('_Close'), None, None, lambda a:projectaction.close_project()),
-            ('Quit', None, _('_Quit'), '<control>Q', None, lambda a:app.shutdown()),
+            ('Quit', None, _('_Quit'), '<control>Q', None, lambda a:callbackbridge.app_shutdown()),
             ('EditMenu', None, _('_Edit')),
             ('Undo', None, _('_Undo'), '<control>Z', None, undo.do_undo_and_repaint),
             ('Redo', None, _('_Redo'), '<control>Y', None, undo.do_redo_and_repaint),
@@ -827,7 +825,7 @@ class EditorWindow:
             ('SequenceSplit', None, _('Split to new Sequence at Playhead Position'), None, None, lambda a:tlineaction.sequence_split_pressed()),
             ('DeleteClip', None, _('Lift'), None, None, lambda a:tlineaction.lift_button_pressed()),
             ('SpliceOutClip', None, _('Splice Out'), None, None, lambda a:tlineaction.splice_out_button_pressed()),
-            ('ResyncSelected', None, _('Resync'),  resync_shortcut, None, lambda a:tlineaction.resync_button_pressed()),
+            ('ResyncSelected', None, _('Resync Track'),  resync_shortcut, None, lambda a:tlineaction.resync_button_pressed()),
             ('SetSyncParent', None, _('Set Sync Parent'), None, None, lambda a:_this_is_not_used()),
             ('AddTransition', None, _('Add Single Track Transition'), None, None, lambda a:singletracktransition.add_transition_menu_item_selected()),
             ('ClearFilters', None, _('Clear Filters'), clear_filters_shortcut, None, lambda a:clipmenuaction.clear_filters()),
@@ -870,14 +868,13 @@ class EditorWindow:
             ('CreateGMicContainerItem', None, _("From G'Mic Script"), None, None, lambda a:containerclip.create_gmic_media_item()),
             ('CombineSequences', None, _('Import Another Sequence Into This Sequence...'), None, None, lambda a:projectaction.combine_sequences()),
             ('LogClipRange', None, _('Log Marked Clip Range'), '<control>L', None, lambda a:medialog.log_range_clicked()),
-            ('ViewProjectEvents', None, _('View Project Events...'), None, None, lambda a:projectaction.view_project_events()),
             ('RecreateMediaIcons', None, _('Recreate Media Icons...'), None, None, lambda a:menuactions.recreate_media_file_icons()),
+            ('ShowProjectInfo', None, _('Project Info and Data'), None, None, lambda a:projectaction.show_project_info()),
             ('RemoveUnusedMedia', None, _('Remove Unused Media...'), None, None, lambda a:projectaction.remove_unused_media()),
             ('ChangeProfile', None, _("Change Project Profile..."), None, None, lambda a: projectaction.change_project_profile()),
             ('ProxyManager', None, _('Proxy Manager'), None, None, lambda a:proxyediting.show_proxy_manager_dialog()),
             ('DataStoreManager', None, _('Data Store Manager'), None, None, lambda a:projectdatavaultgui.show_project_data_manager_window()),
             ('ProjectDataInfo', None, _('Project Data'), None, None, lambda a:projectdatavaultgui.show_current_project_data_store_info_window()),
-            ('ProjectInfo', None, _('Project Info'), None, None, lambda a:menuactions.show_project_info()),
             ('RenderMenu', None, _('Render')),
             ('AddToQueue', None, _('Add To Batch Render Queue...'), None, None, lambda a:projectaction.add_to_render_queue()),
             ('BatchRender', None, _('Batch Render Queue'), None, None, lambda a:batchrendering.launch_batch_rendering()),
@@ -893,7 +890,7 @@ class EditorWindow:
             ('QuickReference', None, _('Contents'), None, None, lambda a:menuactions.quick_reference()),
             ('QuickReferenceWeb', None, _('Contents Web'), None, None, lambda a:menuactions.quick_reference_web()),
             ('Environment', None, _('Runtime Environment'), None, None, lambda a:menuactions.environment()),
-            ('KeyboardShortcuts', None, _('Keyboard Shortcuts'), None, None, lambda a:dialogs.keyboard_shortcuts_dialog(self.window, workflow.get_tline_tool_working_set, menuactions.keyboard_shortcuts_callback, keyevents.change_single_shortcut, menuactions.keyboard_shortcuts_menu_item_selected_callback)),
+            ('KeyboardShortcuts', None, _('Keyboard Shortcuts'), None, None, lambda a:shortcutsdialog.keyboard_shortcuts_dialog(self.window, workflow.get_tline_tool_working_set, menuactions.keyboard_shortcuts_callback, keyevents.change_single_shortcut, menuactions.keyboard_shortcuts_menu_item_selected_callback)),
             ('About', None, _('About'), None, None, lambda a:menuactions.about()),
             ('TOOL_ACTION_KEY_1', None, None, '1', None, lambda a:_this_is_not_used()),
             ('TOOL_ACTION_KEY_2', None, None, '2', None, lambda a:_this_is_not_used()),
@@ -994,7 +991,7 @@ class EditorWindow:
                     <separator/>
                     <menuitem action='ChangeProfile'/>
                     <separator/>
-                    <menuitem action='ProjectDataInfo'/>
+                    <menuitem action='ShowProjectInfo'/>
                     <separator/>
                     <menuitem action='ProxyManager'/>
                 </menu>
@@ -1056,8 +1053,8 @@ class EditorWindow:
 
         # Get menu bar
         self.menubar = self.ui.get_widget('/MenuBar')
-        if editorpersistance.prefs.theme == appconsts.FLOWBLADE_THEME_NEUTRAL:
-            self.menubar .override_background_color(Gtk.StateFlags.NORMAL, gui.get_mid_neutral_color())
+        self.menubar .override_background_color(Gtk.StateFlags.NORMAL, gui.get_mid_neutral_color())
+
         # Set reference to UI manager and acclegroup
         self.uimanager = ui
         self.accel_group = accel_group
@@ -1384,7 +1381,7 @@ class EditorWindow:
 
         # Aug-2019 - SvdB - BB
         offset = 2
-        if editorpersistance.prefs.double_track_hights:
+        if guiutils.double_icon_size():
            offset = 4
            buttons_row.set_margin_bottom(offset)
 
@@ -1405,6 +1402,15 @@ class EditorWindow:
     def top_paned_resized(self, w, req):
         pass
 
+# testing
+def _apply_test_dimensions(window, test_index, scale):
+    dims = [(1920, 1200), (2048, 1152), (2048, 1536), (2560, 1440), (2560, 1600), (3440, 1440), (3840, 2160)]
+    w, h = dims[test_index]
+    print("unscaled", w, h)
+    w = int(w/scale)
+    h = int(h/scale)
+    print("scaled", w, h)
+    window.set_size_request(w, h)
 
 def _this_is_not_used():
     print("THIS WAS USED!!!!!")
