@@ -22,20 +22,28 @@
 This module contains complex property editors.
 """
 
+from PIL import Image
+
+from datetime import datetime
 import math
+import os
+import webbrowser
 
 from gi.repository import Gtk
 
 import appconsts
 import cairo
 import cairoarea
-import editorpersistance
+from editorstate import PROJECT
 import guiutils
 import guicomponents
 import glassbuttons
+import jobs
 import lutfilter
+import motiontracking
 import respaths
 import translations
+import utils
 
 SHADOW = 0
 MID = 1
@@ -86,6 +94,9 @@ CIRCLE_GRAD_2 = (0, 0.8, 0.8, 0.8, 1)
 FX_GRAD_1 = (0, 1.0, 1.0, 1.0, 0.4)
 FX_GRAD_2 = (1, 0.3, 0.3, 0.3, 0.4)
 
+
+accessable_editors = {}
+
 def _p(name):
     try:
         return translations.param_names[name]
@@ -160,7 +171,6 @@ def _draw_select_line(cr, x, y):
     cr.line_to(x + 0.5, y + height)
     cr.stroke()
 
-
 def _draw_cursor_indicator(cr, x, y, radius):
     degrees = math.pi / 180.0
 
@@ -1088,208 +1098,6 @@ class ColorGrader:
             self.hs_label.set_text(sat_str)
 
 
-"""
-# NON_ MLT PROPERTY SLIDER DEMO CODE
-def hue_changed(self, ep, value):
-    ep.write_property_value(str(value))
-    self.update_properties()
-
-def saturation_changed(self, ep, value):
-    ep.write_property_value(str(value))
-    self.update_properties()
-
-def value_changed(self, ep, value):
-    ep.write_property_value(str(value))
-    self.update_properties()
-"""
-
-"""
-class AbstractColorWheel:
-
-    def __init__(self, edit_listener):
-        self.widget = cairoarea.CairoDrawableArea2( 260,
-                                                    260,
-                                                    self._draw)
-        self.widget.press_func = self._press_event
-        self.widget.motion_notify_func = self._motion_notify_event
-        self.widget.release_func = self._release_event
-        self.X_PAD = 3
-        self.Y_PAD = 3
-        self.CENTER_X = 130
-        self.CENTER_Y = 130
-        self.MAX_DIST = 123
-        self.twelwe_p = (self.CENTER_X , self.CENTER_Y - self.MAX_DIST)
-        self.CIRCLE_HALF = 6
-        self.cursor_x = self.CENTER_X
-        self.cursor_y = self.CENTER_Y
-        self.WHEEL_IMG = GdkPixbuf.Pixbuf.new_from_file(respaths.IMAGE_PATH + "color_wheel.png")
-        self.edit_listener = edit_listener
-        self.angle = 0.0
-        self.distance = 0.0
-
-    def _press_event(self, event):
-
-        self.cursor_x, self.cursor_y = self._get_legal_point(event.x, event.y)
-        self._save_point()
-        self.widget.queue_draw()
-
-    def _motion_notify_event(self, x, y, state):
-
-        self.cursor_x, self.cursor_y = self._get_legal_point(x, y)
-        self._save_point()
-        self.widget.queue_draw()
-
-    def _release_event(self, event):
-        self.cursor_x, self.cursor_y = self._get_legal_point(event.x, event.y)
-        self._save_point()
-        self.edit_listener()
-        self.widget.queue_draw()
-
-    def _get_legal_point(self, x, y):
-        vec = viewgeom.get_vec_for_points((self.CENTER_X, self.CENTER_Y), (x, y))
-        dist = vec.get_length()
-        if dist < self.MAX_DIST:
-            return (x, y)
-
-        new_vec = vec.get_multiplied_vec(self.MAX_DIST / dist )
-        return new_vec.end_point
-
-    def get_angle(self, p):
-        angle = viewgeom.get_angle_in_deg(self.twelwe_p, (self.CENTER_X, self.CENTER_Y), p)
-        clockwise = viewgeom.points_clockwise(self.twelwe_p, (self.CENTER_X, self.CENTER_Y), p)
-
-        if clockwise:
-             angle = 360.0 - angle;
-
-        # Color circle starts from 11 o'clock
-        angle = angle - 30.0
-        if angle  < 0.0:
-            angle = angle + 360.0
-
-        return angle
-
-    def get_distance(self, p):
-        vec = viewgeom.get_vec_for_points((self.CENTER_X, self.CENTER_Y), p)
-        dist = vec.get_length()
-        return dist/self.MAX_DIST
-
-    def _save_point(self):
-        print "_save_point not implemented"
-        pass
-
-    def get_angle_and_distance(self):
-        if self.band == SHADOW:
-            x = self.shadow_x
-            y = self.shadow_y
-        elif self.band == MID:
-            x = self.mid_x
-            y = self.mid_y
-        else:
-            x = self.hi_x
-            y = self.hi_y
-        p = (x, y)
-        angle = self._get_angle(p)
-        distance = self._get_distance(p)
-        return (angle, distance)
-
-    def _draw(self, event, cr, allocation):
-        x, y, w, h = allocation
-
-        # Draw bg
-        cr.set_source_rgb(*(gui.bg_color_tuple))
-        cr.rectangle(0, 0, w, h)
-        cr.fill()
-
-        cr.set_source_pixbuf(self.WHEEL_IMG, self.X_PAD, self.Y_PAD)
-        cr.paint()
-
-
-class SimpleColorWheel(AbstractColorWheel):
-
-    def __init__(self, edit_listener):
-        AbstractColorWheel.__init__(self, edit_listener)
-        self.value_x = self.cursor_x
-        self.value_y = self.cursor_y
-
-    def _save_point(self):
-        self.value_x = self.cursor_x
-        self.value_y = self.cursor_y
-
-    def get_angle_and_distance(self):
-        p = (self.value_x, self.value_y)
-        angle = self.get_angle(p)
-        distance = self.get_distance(p)
-        return (angle, distance)
-
-    def _draw(self, event, cr, allocation):
-        AbstractColorWheel._draw(self, event, cr, allocation)
-
-        _draw_select_circle(cr, self.cursor_x - self.CIRCLE_HALF, self.cursor_y - self.CIRCLE_HALF, (1,1,1), ACTIVE_RING_COLOR)
-
-
-class SMHColorWheel(AbstractColorWheel):
-
-    def __init__(self, edit_listener):
-        AbstractColorWheel.__init__(self, edit_listener)
-        self.band = SHADOW
-        self.shadow_x = self.cursor_x
-        self.shadow_y = self.cursor_y
-        self.mid_x = self.cursor_x
-        self.mid_y = self.cursor_y
-        self.hi_x = self.cursor_x
-        self.hi_y = self.cursor_y
-
-    def set_band(self, band):
-        self.band = band
-        if self.band == SHADOW:
-            self.cursor_x = self.shadow_x
-            self.cursor_y = self.shadow_y
-        elif self.band == MID:
-            self.cursor_x = self.mid_x
-            self.cursor_y = self.mid_y
-        else:
-            self.cursor_x = self.hi_x
-            self.cursor_y = self.hi_y
-
-    def _save_point(self):
-        if self.band == SHADOW:
-            self.shadow_x = self.cursor_x
-            self.shadow_y = self.cursor_y
-        elif self.band == MID:
-            self.mid_x = self.cursor_x
-            self.mid_y = self.cursor_y
-        else:
-            self.hi_x = self.cursor_x
-            self.hi_y = self.cursor_y
-
-    def get_angle_and_distance(self):
-        if self.band == SHADOW:
-            x = self.shadow_x
-            y = self.shadow_y
-        elif self.band == MID:
-            x = self.mid_x
-            y = self.mid_y
-        else:
-            x = self.hi_x
-            y = self.hi_y
-        p = (x, y)
-        angle = self.get_angle(p)
-        distance = self.get_distance(p)
-        return (angle, distance)
-
-    def _draw(self, event, cr, allocation):
-        AbstractColorWheel._draw(self, event, cr, allocation)
-
-        if self.band == SHADOW:
-            band_color = ACTIVE_SHADOW_COLOR
-        elif self.band == MID:
-            band_color = ACTIVE_MID_COLOR
-        else:
-            band_color = ACTIVE_HI_COLOR
-
-        _draw_select_circle(cr, self.cursor_x - self.CIRCLE_HALF, self.cursor_y - self.CIRCLE_HALF, band_color, ACTIVE_RING_COLOR)
-"""
-
 class ColorBandSelector:
     def __init__(self):
         self.band = SHADOW
@@ -1357,3 +1165,341 @@ class ColorBandSelector:
         cr.close_path()
         cr.fill()
 
+
+class InfoAndTipsEditor:
+    
+    def __init__(self, uri, link_text):
+        self.uri = uri
+        self.widget = Gtk.LinkButton.new_with_label(uri, link_text)
+        self.widget.connect("activate-link", self._open_info)
+        self.widget.set_margin_top(24)
+        self.widget.set_margin_bottom(4)
+
+    def _open_info(self, widget):
+        webbrowser.open(self.uri)
+        return True
+
+
+class AnalyzeStabilizeFilterEditor:
+    def __init__(self, filter, editable_properties):
+        self.filter = filter
+        self.editable_properties = editable_properties
+        self.widget = Gtk.HBox()
+        self.button = Gtk.Button(label=_("Analyze video"))
+        self.button.connect("clicked", self.analyze_button_clicked)
+        self.no_data_text = "<small>" + _("No stabilizing data") + "</small>"
+        self.loaded_data_text = "<small>" + _("Stabilizing data loaded") + "</small>"
+        prop_name, prop_value, prop_type = filter.non_mlt_properties[0]
+        if prop_value == appconsts.FILE_PATH_NOT_SET:
+            text = self.no_data_text 
+        else:
+            text = self.loaded_data_text 
+        self.info_label = Gtk.Label(label=text)
+        self.info_label.set_use_markup(True)
+        self.info_label.set_margin_right(4)
+        self.widget.pack_start(self.info_label, False, False, 0) 
+        self.widget.pack_start(self.button, False, False, 0) 
+
+        self.widget.set_margin_top(24)
+        self.widget.set_margin_bottom(24)
+
+    def analyze_button_clicked(self, button):
+        session_id = utils.create_render_session_uid()
+        profile_desc = PROJECT().profile_desc.replace(" ", "_")
+        clip_path = self.editable_properties[0].clip.path
+        accuracy_prop = [ep for ep in self.editable_properties if ep.name == "accuracy"][0]
+        shakiness_prop = [ep for ep in self.editable_properties if ep.name == "shakiness"][0]
+        
+        args = ("session_id:" + str(session_id),
+                "profile_desc:" + str(profile_desc),
+                "clip_path:" + str(clip_path),
+                "shakiness:" + str(shakiness_prop.value),
+                "accuracy:" + str(accuracy_prop.value))
+
+        job = jobs.StablizeDataRenderJobQueueObject(session_id, self.filter, self.editable_properties, self, args)
+        job.add_to_queue()
+    
+    def analysis_complete(self):
+        self.info_label.set_text(self.loaded_data_text)
+        self.info_label.set_use_markup(True)
+        self.info_label.queue_draw()
+
+
+
+class AnalyzeMotionTrackingFilterEditor:
+    def __init__(self, filter, editable_properties):
+        self.filter = filter
+        # We need to turn this because otherwise filter keeps attemting to 
+        # analyze tracking continuosly. We want analyzing to happen in the 
+        # dedicated process and only results be displayed.
+        # ...aaand this only has effect if we have some results?
+        self.filter.mlt_filter.set("analyze", "0")
+
+        self.editable_properties = editable_properties
+
+        # We need access to GUI box editor to control its state.
+        box_editor_key ="trackerrect" + ":" + str(editable_properties[0].filter_index)
+        self.box_gui_editor = accessable_editors[box_editor_key]
+        
+        label_info_label = Gtk.Label(label=_("Tracking Data Label:"))
+        label_info_label.set_margin_right(4)
+
+        def_label = self.get_default_data_label()
+        
+        self.label_entry = Gtk.Entry()
+        self.label_entry.set_width_chars(28)
+        self.label_entry.set_text(def_label)
+        self.label_entry.set_activates_default(True)
+
+        hbox1 = Gtk.HBox()
+        hbox1.pack_start(label_info_label, False, False, 0) 
+        hbox1.pack_start(self.label_entry, True, True, 0)
+        
+        self.button = Gtk.Button(label=_("Create Motion Tracking Data"))
+        self.button.connect("clicked", self.analyze_button_clicked)
+
+        self.clear_button = Gtk.Button(label=_("Clear Data"))
+        self.clear_button.connect("clicked", self.clear_button_clicked)
+        
+        self.info_label = Gtk.Label("")
+        self.info_label.set_use_markup(True)
+        self.info_label.set_margin_right(4)
+
+        hbox2 = Gtk.HBox()
+        hbox2.pack_start(Gtk.Label(), True, True, 0) 
+        hbox2.pack_start(self.info_label, False, False, 0) 
+        hbox2.set_margin_top(4)
+        
+        hbox3 = Gtk.HBox()
+        hbox3.pack_start(self.clear_button, False, False, 0)
+        hbox3.pack_start(Gtk.Label(), True, True, 0) 
+        hbox3.pack_start(self.button, False, False, 0)
+        hbox3.set_margin_top(4)
+
+        vbox = Gtk.VBox()
+        
+        vbox.pack_start(hbox1, False, False, 0) 
+        vbox.pack_start(hbox2, False, False, 0)
+        vbox.pack_start(hbox3, False, False, 0)
+
+        self.widget = Gtk.HBox()
+        self.widget.pack_start(vbox, True, True, 0)
+
+        self.widget.set_margin_top(24)
+        self.widget.set_margin_bottom(24)
+        
+        self.set_buttons_state()
+            
+    def get_default_data_label(self):
+        clip_path = self.editable_properties[0].clip.path
+        file_name = os.path.splitext(os.path.basename(clip_path))[0]
+        if len(file_name) < 8:
+            def_file_part = file_name
+        else:
+            def_file_part = file_name[:8]
+        
+        return def_file_part + "-" + _("tracking")
+        
+    def analyze_button_clicked(self, button):
+        session_id = utils.create_render_session_uid()
+        profile_desc = PROJECT().profile_desc.replace(" ", "_")
+        clip_path = self.editable_properties[0].clip.path
+        clip_in = str(self.editable_properties[0].clip.clip_in)
+        clip_out = str(self.editable_properties[0].clip.clip_out)
+        step_prop = [ep for ep in self.editable_properties if ep.name == "steps"][0]
+        algo_prop = [ep for ep in self.editable_properties if ep.name == "algo"][0]
+        rect_prop = [ep for ep in self.editable_properties if ep.name == "rect"][0]
+        rect_value = rect_prop.value.replace(" ", "_")
+
+        args = ("session_id:" + str(session_id),
+                "profile_desc:" + str(profile_desc),
+                "clip_path:" + clip_path,
+                "clip_in:" + clip_in,
+                "clip_out:" + clip_out,
+                "rect:" + str(rect_value),
+                "algo:" + str(algo_prop.value),
+                "step:" + str(step_prop.value))
+
+        data_label = self.label_entry.get_text()
+        if len(data_label) == 0:
+            data_label = self.get_default_data_label()
+        
+        job = jobs.MotionTrackingDataRenderJobQueueObject(session_id, self.filter, self.editable_properties, self, args, data_label)
+        job.add_to_queue()
+
+    def analysis_complete(self, final_label, data_file_path):
+        f = open(data_file_path, 'r')
+        results = f.read()
+        f.close()
+
+        self.filter.mlt_filter.set("results", results)
+        self.filter.mlt_filter.set("reload", "1")
+        self.filter.mlt_filter.set("shape_color", "#00ff00")
+
+        try:
+            create_info_text = "<small>" + _("Motion Tracking Data created: ") + final_label + "</small>"
+            self.info_label.set_text(create_info_text)
+            self.info_label.set_use_markup(True)
+            self.info_label.queue_draw()
+            self.box_gui_editor.geom_kf_edit.active = False
+            self.box_gui_editor.geom_kf_edit.widget.queue_draw()
+            self.set_buttons_state()
+        except Exception as e :
+            # GUI might havenn destoyed during rendering.
+            print(e)
+
+    def clear_button_clicked(self, button):
+        self.filter.mlt_filter.set("analyze", "0")
+        self.filter.mlt_filter.set("results", "")
+        self.filter.mlt_filter.set("reload", "1")
+        self.filter.mlt_filter.set("shape_color", "#aa2222")
+        self.box_gui_editor.geom_kf_edit.active = True
+        self.box_gui_editor.geom_kf_edit.widget.queue_draw()
+        self.set_buttons_state()
+
+    def set_buttons_state(self):
+        results = self.filter.mlt_filter.get("results") 
+        if results == None or len(results) == 0:
+            self.button.set_sensitive(True)
+            self.clear_button.set_sensitive(False)
+        else:
+            self.button.set_sensitive(False)
+            self.clear_button.set_sensitive(True)
+        
+
+class ApplyMotionTrackingFilterEditor:
+    def __init__(self, filter, editable_properties, non_mlt_editors, non_mlt_properties):
+        self.filter = filter
+        self.editable_properties = editable_properties
+        self.non_mlt_editors = non_mlt_editors
+        self.non_mlt_properties = non_mlt_properties
+
+        select_label = Gtk.Label(label=_("Select Motion Tracking Data:"))
+        select_label.set_margin_right(4)
+
+        selected_tracking_data = [ep for ep in self.non_mlt_properties if ep.name == "selected_tracking_data"][0].value
+        self.data_select_keys, self.data_select_combo = motiontracking.get_tracking_data_select_combo(_("No Tracking Data Available"), selected_tracking_data)
+
+        hbox1 = Gtk.HBox()
+        hbox1.pack_start(select_label, False, False, 0) 
+        hbox1.pack_start(self.data_select_combo, True, True, 0)
+        hbox1.set_margin_bottom(24)
+
+        self.info_label = Gtk.Label("<small>No Tracking Data Applied</small>")
+        last_applied_date_str = [ep for ep in self.non_mlt_properties if ep.name == "last_applied_tracking_data"][0].value
+        if last_applied_date_str != appconsts.TRACKING_DATA_NOT_SET:
+            self.show_last_applied_date(last_applied_date_str)
+        self.info_label.set_use_markup(True)
+        self.info_label.set_margin_right(4)
+
+        self.button = Gtk.Button(label=_("Apply Motion Tracking Data"))
+        self.button.connect("clicked", self.apply_tracking)
+
+        hbox2 = Gtk.HBox()
+        hbox2.pack_start(Gtk.Label(), True, True, 0) 
+        hbox2.pack_start(self.info_label, False, False, 0) 
+        hbox2.pack_start(self.button, False, False, 0)
+        hbox2.set_margin_top(24)
+        hbox2.set_margin_bottom(24)
+        
+        self.widget = Gtk.VBox()
+        self.widget.pack_start(hbox1, False, False, 0)
+        for row in self.non_mlt_editors:
+            self.widget.pack_start(row, False, False, 0)
+        self.widget.pack_start(hbox2, False, False, 0)
+
+    def apply_tracking(self, button):
+        tracking_data_id = self.data_select_keys[self.data_select_combo.get_active()]
+        xoff = [ep for ep in self.non_mlt_properties if ep.name == "xoff"][0].value
+        yoff = [ep for ep in self.non_mlt_properties if ep.name == "yoff"][0].value
+        interpretation = [ep for ep in self.non_mlt_properties if ep.name == "interpretation"][0].value
+        size = [ep for ep in self.non_mlt_properties if ep.name == "size"][0].value
+        clip_in = self.editable_properties[0].clip.clip_in
+        
+        info = utils.get_file_producer_info(self.editable_properties[0].clip)
+        source_w = int(info["width"])
+        source_h = int(info["height"])
+        if self.editable_properties[0].clip.media_type == appconsts.IMAGE:
+            graphic_img = Image.open(self.editable_properties[0].clip.path)
+            source_w, source_h = graphic_img.size
+            
+        motiontracking.apply_tracking(  tracking_data_id, self.filter, self.editable_properties, 
+                                        int(float(xoff)), int(float(yoff)), interpretation, size, clip_in,
+                                        source_w, source_h)
+        
+        selected_tracking_data_prop = [ep for ep in self.non_mlt_properties if ep.name == "selected_tracking_data"][0]
+        selected_tracking_data_prop.write_value(tracking_data_id)
+
+        date_str = datetime.now().strftime('%y.%m - %H:%M:%S')
+        last_applied_tracking_data = [ep for ep in self.non_mlt_properties if ep.name == "last_applied_tracking_data"][0]
+        last_applied_tracking_data.write_value(date_str)
+        self.show_last_applied_date(date_str)
+
+    def show_last_applied_date(self, date_str):
+        self.info_label.set_text("<small>" + _("Last applied ") + date_str + "</small>")
+        self.info_label.set_use_markup(True)
+        self.info_label.queue_draw()
+
+
+class FilterMaskApplyMotionTrackingEditor:
+    def __init__(self, filter, editable_properties, non_mlt_editors, non_mlt_properties):
+        self.filter = filter
+        self.editable_properties = editable_properties
+        self.non_mlt_properties = non_mlt_properties
+        self.non_mlt_editors = non_mlt_editors
+        
+        select_label = Gtk.Label(label=_("Select Motion Tracking Data:"))
+        select_label.set_margin_right(4)
+
+        selected_tracking_data = [ep for ep in self.non_mlt_properties if ep.name == "selected_tracking_data"][0].value
+        self.data_select_keys, self.data_select_combo = motiontracking.get_tracking_data_select_combo(_("No Tracking Data Available"), selected_tracking_data)
+
+        hbox1 = Gtk.HBox()
+        hbox1.pack_start(select_label, False, False, 0) 
+        hbox1.pack_start(self.data_select_combo, True, True, 0)
+        hbox1.set_margin_bottom(24)
+
+        self.info_label = Gtk.Label("<small>No Tracking Data Applied</small>")
+        last_applied_date_str = [ep for ep in self.non_mlt_properties if ep.name == "last_applied_tracking_data"][0].value
+        if last_applied_date_str != appconsts.TRACKING_DATA_NOT_SET:
+            self.show_last_applied_date(last_applied_date_str)
+        self.info_label.set_use_markup(True)
+        self.info_label.set_margin_right(4)
+
+        self.button = Gtk.Button(label=_("Apply Motion Tracking Data"))
+        self.button.connect("clicked", self.apply_tracking)
+
+        hbox2 = Gtk.HBox()
+        hbox2.pack_start(Gtk.Label(), True, True, 0) 
+        hbox2.pack_start(self.info_label, False, False, 0) 
+        hbox2.pack_start(self.button, False, False, 0)
+        hbox2.set_margin_top(24)
+        hbox2.set_margin_bottom(24)
+        
+        self.widget = Gtk.VBox()
+        self.widget.pack_start(hbox1, False, False, 0)
+        for row in self.non_mlt_editors:
+            self.widget.pack_start(row, False, False, 0)
+        self.widget.pack_start(hbox2, False, False, 0)
+
+    def apply_tracking(self, button):
+        tracking_data_id = self.data_select_keys[self.data_select_combo.get_active()]
+        clip_in = self.editable_properties[0].clip.clip_in
+        xoff = [ep for ep in self.non_mlt_properties if ep.name == "xoff"][0].value
+        yoff = [ep for ep in self.non_mlt_properties if ep.name == "yoff"][0].value
+        scale = [ep for ep in self.non_mlt_properties if ep.name == "scale"][0].value
+
+        motiontracking.apply_filter_mask_tracking(tracking_data_id, self.filter, self.editable_properties, int(float(xoff)), int(float(yoff)), float(scale) / 100.0, clip_in)
+        
+        selected_tracking_data_prop = [ep for ep in self.non_mlt_properties if ep.name == "selected_tracking_data"][0]
+        selected_tracking_data_prop.write_value(tracking_data_id)
+
+        date_str = datetime.now().strftime('%y.%m - %H:%M:%S')
+        last_applied_tracking_data = [ep for ep in self.non_mlt_properties if ep.name == "last_applied_tracking_data"][0]
+        last_applied_tracking_data.write_value(date_str)
+        self.show_last_applied_date(date_str)
+
+    def show_last_applied_date(self, date_str):
+        self.info_label.set_text("<small>" + _("Last applied ") + date_str + "</small>")
+        self.info_label.set_use_markup(True)
+        self.info_label.queue_draw()
\ No newline at end of file
