@@ -21,12 +21,10 @@
 import cairo
 import math
 
-import appconsts
 import cairoarea
 import editorpersistance
 import gui
-import guiutils # Aug-2019 - SvdB - BB
-import respaths
+import guiutils
 
 BUTTONS_GRAD_STOPS = [   (1, 1, 1, 1, 0.2),
                         (0.8, 1, 1, 1, 0),
@@ -305,8 +303,7 @@ class PlayerButtons(AbstractGlassButtons):
         size_adj = 1
         prefs = editorpersistance.prefs
 
-        # End of Colorized icons
-        if prefs.double_track_hights:
+        if guiutils.double_icon_size():
            size_ind = 1
            size_adj = 2
         AbstractGlassButtons.__init__(self, MB_BUTTON_WIDTH[size_ind], MB_BUTTON_HEIGHT[size_ind], MB_BUTTON_Y, MB_BUTTONS_WIDTH[size_ind], MB_BUTTONS_HEIGHT[size_ind] - 2)
@@ -435,7 +432,7 @@ class GmicButtons(AbstractGlassButtons):
         prefs = editorpersistance.prefs
 
         # NOT IMPLEMENTED
-        if prefs.double_track_hights:
+        if guiutils.double_icon_size():
            size_ind = 1
            size_adj = 2
         AbstractGlassButtons.__init__(self, MB_BUTTON_WIDTH[size_ind], MB_BUTTON_HEIGHT[size_ind], MB_BUTTON_Y, MB_BUTTONS_WIDTH[size_ind], MB_BUTTONS_HEIGHT[size_ind] - 2)
