@@ -19,7 +19,7 @@
 """
 
 
-from gi.repository import Gtk, GLib, Gdk, GdkPixbuf
+from gi.repository import Gtk, GLib
 from gi.repository import GObject
 from gi.repository import Pango
 
@@ -34,18 +34,19 @@ except:
     import mlt
 
 import appconsts
+import callbackbridge
 import editorlayout
 import editorpersistance
 from editorstate import PROJECT
-import gui
 import guicomponents
 import guipopover
 import guiutils
 import motionheadless
-import persistance
 import proxyheadless
 import renderconsumer
 import respaths
+import stabilizeheadless
+import trackingheadless
 import userfolders
 import utils
 
@@ -61,14 +62,14 @@ CONTAINER_CLIP_RENDER_BLENDER = 3 # Deprecated
 MOTION_MEDIA_ITEM_RENDER = 4
 PROXY_RENDER = 5
 CONTAINER_CLIP_RENDER_FLUXITY = 6
+STABILIZE_DATA_RENDER = 7
+MOTION_TRACKING_DATA_RENDER = 8
 
 FFMPEG_ATTR_SOURCEFILE = "%SOURCEFILE"
 FFMPEG_ATTR_SCREENSIZE = "%SCREENSIZE"
 FFMPEG_ATTR_SCREENSIZE_2 = "%SCREEN%SIZE%TWO%"
 FFMPEG_ATTR_PROXYFILE = "%PROXYFILE"
 
-open_media_file_callback = None
-
 _status_polling_thread = None
 
 _jobs_list_view = None
@@ -79,7 +80,7 @@ _remove_list = [] # objects are removed from GUI with delay to give user time to
 _jobs_render_progress_window = None
 
 
-class JobProxy: # This object represents job in job queue. 
+class JobProxy: # This object represents a job in the job queue. 
 
 
     def __init__(self, uid, callback_object):
@@ -98,21 +99,7 @@ class JobProxy: # This object represents job in job queue.
 
     def get_elapsed_str(self):
         return utils.get_time_str_for_sec_float(self.elapsed)
-
-    def get_type_str(self):
-        if self.type == NOT_SET_YET:
-            return "NO TYPE SET" # this just error info, application has done something wrong.
-        elif self.type == CONTAINER_CLIP_RENDER_GMIC:
-            return _("G'Mic Clip")
-        elif self.type == CONTAINER_CLIP_RENDER_MLT_XML:
-            return _("Selection Clip")
-        elif self.type == MOTION_MEDIA_ITEM_RENDER:
-            return _("Motion Clip")
-        elif self.type == PROXY_RENDER:
-            return _("Proxy Clip")
-        elif self.type == CONTAINER_CLIP_RENDER_FLUXITY:
-            return _("Generator Clip")
-            
+    
     def get_progress_str(self):
         if self.progress < 0.0:
             return "-"
@@ -194,10 +181,9 @@ def update_job_queue(job_msg): # We're using JobProxy objects as messages to upd
     tree_path = Gtk.TreePath.new_from_string(str(row))
     store_iter = _jobs_list_view.storemodel.get_iter(tree_path)
 
-    _jobs_list_view.storemodel.set_value(store_iter, 0, _jobs[row].get_type_str())
-    _jobs_list_view.storemodel.set_value(store_iter, 1, _jobs[row].text)
-    _jobs_list_view.storemodel.set_value(store_iter, 2, _jobs[row].get_elapsed_str())
-    _jobs_list_view.storemodel.set_value(store_iter, 3, _jobs[row].get_progress_str())
+    _jobs_list_view.storemodel.set_value(store_iter, 0, _jobs[row].text)
+    _jobs_list_view.storemodel.set_value(store_iter, 1, _jobs[row].get_elapsed_str())
+    _jobs_list_view.storemodel.set_value(store_iter, 2, _jobs[row].get_progress_str())
 
     _jobs_list_view.scroll.queue_draw()
 
@@ -324,7 +310,7 @@ class JobsQueueView(Gtk.VBox):
     def __init__(self):
         GObject.GObject.__init__(self)
         
-        self.storemodel = Gtk.ListStore(str, str, str, str)
+        self.storemodel = Gtk.ListStore(str, str, str)
         
         # Scroll container
         self.scroll = Gtk.ScrolledWindow()
@@ -336,9 +322,6 @@ class JobsQueueView(Gtk.VBox):
         tree_sel = self.treeview.get_selection()
         tree_sel.set_mode(Gtk.SelectionMode.MULTIPLE)
 
-        self.text_rend_1 = Gtk.CellRendererText()
-        self.text_rend_1.set_property("ellipsize", Pango.EllipsizeMode.END)
-
         self.text_rend_2 = Gtk.CellRendererText()
         self.text_rend_2.set_property("yalign", 0.0)
         self.text_rend_2.set_property("ellipsize", Pango.EllipsizeMode.END)
@@ -350,33 +333,24 @@ class JobsQueueView(Gtk.VBox):
         self.text_rend_4.set_property("yalign", 0.0)
 
         # Column views
-        self.text_col_1 = Gtk.TreeViewColumn(_("Job Type"))
-        self.text_col_2 = Gtk.TreeViewColumn(_("Info"))
+        self.text_col_2 = Gtk.TreeViewColumn(_("Job Info"))
         self.text_col_3 = Gtk.TreeViewColumn(_("Render Time"))
         self.text_col_4 = Gtk.TreeViewColumn(_("Progress"))
 
-        #self.text_col_1.set_expand(True)
-        self.text_col_1.set_spacing(5)
-        self.text_col_1.set_sizing(Gtk.TreeViewColumnSizing.GROW_ONLY)
-        self.text_col_1.set_min_width(200)
-        self.text_col_1.pack_start(self.text_rend_1, True)
-        self.text_col_1.add_attribute(self.text_rend_1, "text", 0) # <- note column index
-
         self.text_col_2.set_expand(True)
         self.text_col_2.pack_start(self.text_rend_2, True)
-        self.text_col_2.add_attribute(self.text_rend_2, "text", 1)
+        self.text_col_2.add_attribute(self.text_rend_2, "text", 0)
         self.text_col_2.set_min_width(90)
 
         self.text_col_3.set_expand(False)
         self.text_col_3.pack_start(self.text_rend_3, True)
-        self.text_col_3.add_attribute(self.text_rend_3, "text", 2)
+        self.text_col_3.add_attribute(self.text_rend_3, "text", 1)
 
         self.text_col_4.set_expand(False)
         self.text_col_4.pack_start(self.text_rend_4, True)
-        self.text_col_4.add_attribute(self.text_rend_4, "text", 3)
+        self.text_col_4.add_attribute(self.text_rend_4, "text", 2)
 
         # Add column views to view
-        self.treeview.append_column(self.text_col_1)
         self.treeview.append_column(self.text_col_2)
         self.treeview.append_column(self.text_col_3)
         self.treeview.append_column(self.text_col_4)
@@ -398,8 +372,7 @@ class JobsQueueView(Gtk.VBox):
         self.storemodel.clear()        
         
         for job in _jobs:
-            row_data = [job.get_type_str(),
-                        job.text,
+            row_data = [job.text,
                         job.get_elapsed_str(),
                         job.get_progress_str()]
             self.storemodel.append(row_data)
@@ -458,14 +431,25 @@ class AbstractJobQueueObject(JobProxy):
 
 class MotionRenderJobQueueObject(AbstractJobQueueObject):
 
-    def __init__(self, session_id, write_file, args):
+    def __init__(self, session_id, write_file, args, tline_clip_data=None):
         
         AbstractJobQueueObject.__init__(self, session_id, MOTION_MEDIA_ITEM_RENDER)
         
         self.write_file = write_file
         self.args = args
         self.parent_folder = userfolders.get_temp_render_dir() # THis is just used for message passing, output file goes where user decided.
+        self.tline_clip_data = tline_clip_data
 
+        # We are using same job object class to render slowmotion media items and 
+        # slowmotion timeline clips, and need to do different things
+        # when self.tline_clip_data != None and we are 
+        # thus working on a timeline clip.
+        if self.tline_clip_data != None:
+            render_data, completed_callback = self.tline_clip_data
+            clip, track, orig_file_path, slowmo_type, slowmo_clip_media_area, \
+            speed, orig_media_in, orig_media_out, new_clip_in, new_clip_out = render_data
+            self.target_clip = clip
+                
     def get_job_name(self):
         folder, file_name = os.path.split(self.write_file)
         return file_name
@@ -493,27 +477,122 @@ class MotionRenderJobQueueObject(AbstractJobQueueObject):
     def _update_from_gui_thread(self):
 
         if motionheadless.session_render_complete(self.parent_folder, self.get_session_id()) == True:
-            #remove_as_status_polling_object(self)
-            
             job_msg = self.get_completed_job_message()
             update_job_queue(job_msg)
             
+            if self.tline_clip_data != None:
+                self.target_clip.render_progress = None
+                callbackbridge.updater_repaint_tline()
+
             motionheadless.delete_session_folders(self.parent_folder, self.get_session_id())
             
             GLib.idle_add(self.create_media_item)
 
         else:
             status = motionheadless.get_session_status(self.parent_folder, self.get_session_id())
+
             if status != None:
                 fraction, elapsed = status
-                
+
                 self.progress = float(fraction)
                 if self.progress > 1.0:
                     # A fix for how progress is calculated in gmicheadless because producers can render a bit longer then required.
                     self.progress = 1.0
 
+                if self.tline_clip_data != None:
+                    self.target_clip.render_progress = self.progress
+                    callbackbridge.updater_repaint_tline()
+
                 self.elapsed = float(elapsed)
-                self.text = self.get_job_name()
+                self.text = _("Motion Clip Render") + " " + self.get_job_name()
+                
+                job_msg = self.get_job_queue_message()
+
+                update_job_queue(job_msg)
+            else:
+                # Process start/stop on their own and we hit trying to get non-existing status for e.g completed renders.
+                
+                # This may not necessery to do here.
+                if self.tline_clip_data != None:
+                    self.target_clip.render_progress = None
+
+    def abort_render(self):
+        #remove_as_status_polling_object(self)
+        if self.tline_clip_data != None:
+            self.target_clip.render_progress = None
+        motionheadless.abort_render(self.parent_folder, self.get_session_id())
+        
+    def create_media_item(self):
+        if self.tline_clip_data == None:
+            callbackbridge.projectaction_open_rendered_file(self.write_file)
+        else:
+            render_data, completed_callback = self.tline_clip_data
+            completed_callback(render_data, self.write_file)
+
+
+
+class StablizeDataRenderJobQueueObject(AbstractJobQueueObject):
+
+    def __init__(self, session_id, filter, editable_properties, analyze_editor, args):
+        
+        AbstractJobQueueObject.__init__(self, session_id, STABILIZE_DATA_RENDER)
+        
+        self.analyze_editor = analyze_editor
+        self.filter = filter
+        self.editable_properties = editable_properties
+        self.args = args
+        self.parent_folder = userfolders.get_temp_render_dir() # This is used for message passing, output file goes to path given by 'write_file'.
+        
+    def start_render(self):
+        
+        job_msg = self.get_job_queue_message()
+        job_msg.text = _("Render Starting...")
+        job_msg.status = RENDERING
+        update_job_queue(job_msg)
+        
+        # Set writefile.
+        data_file_uid = utils.get_uid_str()
+        self.write_file = userfolders.get_render_dir() + data_file_uid + appconsts.STABILIZE_DATA_EXTENSION
+        print(self.write_file)
+        # Create command list and launch process.
+        command_list = [sys.executable]
+        command_list.append(respaths.LAUNCH_DIR + "flowbladestabilizeheadless")
+        for arg in self.args:
+            command_list.append(arg)
+        parent_folder_arg = "parent_folder:" + str(self.parent_folder)
+        command_list.append(parent_folder_arg)
+        write_file_arg = "write_file:" + str(self.write_file)
+        command_list.append(write_file_arg)
+        
+        subprocess.Popen(command_list)
+        
+    def update_render_status(self):
+        GLib.idle_add(self._update_from_gui_thread)
+            
+    def _update_from_gui_thread(self):
+
+        if stabilizeheadless.session_render_complete(self.parent_folder, self.get_session_id()) == True:
+            #remove_as_status_polling_object(self)
+            
+            job_msg = self.get_completed_job_message()
+            update_job_queue(job_msg)
+            
+            stabilizeheadless.delete_session_folders(self.parent_folder, self.get_session_id())
+            
+            GLib.idle_add(self.update_filter_and_gui)
+
+        else:
+            status = stabilizeheadless.get_session_status(self.parent_folder, self.get_session_id())
+            if status != None:
+                fraction, elapsed = status
+                
+                self.progress = float(fraction)
+                if self.progress > 1.0:
+                    # A fix for how progress is calculated because producers can render a bit longer then required.
+                    self.progress = 1.0
+
+                self.elapsed = float(elapsed)
+                self.text = _("Stabilizing Analysis") + " " + self.editable_properties[0].clip.name
                 
                 job_msg = self.get_job_queue_message()
                 
@@ -521,14 +600,100 @@ class MotionRenderJobQueueObject(AbstractJobQueueObject):
             else:
                 # Process start/stop on their own and we hit trying to get non-existing status for e.g completed renders.
                 pass
-    
+
+    def update_filter_and_gui(self):
+        self.filter.mlt_filter.set("results", str(self.write_file))
+        # We have only one of these, so just recreate list.
+        self.filter.non_mlt_properties = [("results_save_data",  str(self.write_file),  appconsts.PROP_EXPRESSION)]
+        self.analyze_editor.analysis_complete()
+        
     def abort_render(self):
         #remove_as_status_polling_object(self)
-        motionheadless.abort_render(self.parent_folder, self.get_session_id())
+        stabilizeheadless.abort_render(self.parent_folder, self.get_session_id())
+
+
+class MotionTrackingDataRenderJobQueueObject(AbstractJobQueueObject):
+
+    def __init__(self, session_id, filter, editable_properties, analyze_editor, args, data_label):
         
-    def create_media_item(self):
-        open_media_file_callback(self.write_file)
- 
+        AbstractJobQueueObject.__init__(self, session_id, MOTION_TRACKING_DATA_RENDER)
+        
+        self.analyze_editor = analyze_editor
+        self.filter = filter
+        self.editable_properties = editable_properties
+        self.args = args
+        self.data_label = data_label
+        self.parent_folder = userfolders.get_temp_render_dir() # This is used for message passing, output file goes to path given by 'write_file'.
+        
+    def start_render(self):
+        job_msg = self.get_job_queue_message()
+        job_msg.text = _("Render Starting...")
+        job_msg.status = RENDERING
+        update_job_queue(job_msg)
+        
+        # Set writefile.
+        self.write_file = userfolders.get_temp_render_dir() + utils.get_uid_str() + ".xml"
+
+        # Set tracking data file path.
+        self.data_file_path = userfolders.get_render_dir() +  utils.get_uid_str() + appconsts.MOTION_TRACKING_DATA_EXTENSION
+            
+        # Create command list and launch process.
+        command_list = [sys.executable]
+        command_list.append(respaths.LAUNCH_DIR + "flowbladetrackingheadless")
+        for arg in self.args:
+            command_list.append(arg)
+            
+        parent_folder_arg = "parent_folder:" + str(self.parent_folder)
+        command_list.append(parent_folder_arg)
+        write_file_arg = "write_file:" + str(self.write_file)
+        command_list.append(write_file_arg)
+        data_file_arg = "data_file_path:" + str(self.data_file_path)
+        command_list.append(data_file_arg)
+        
+        subprocess.Popen(command_list)
+        
+    def update_render_status(self):
+        GLib.idle_add(self._update_from_gui_thread)
+            
+    def _update_from_gui_thread(self):
+
+        if stabilizeheadless.session_render_complete(self.parent_folder, self.get_session_id()) == True:
+            
+            job_msg = self.get_completed_job_message()
+            update_job_queue(job_msg)
+            
+            trackingheadless.delete_session_folders(self.parent_folder, self.get_session_id())
+            
+            GLib.idle_add(self.update_filter_and_gui)
+
+        else:
+            status = trackingheadless.get_session_status(self.parent_folder, self.get_session_id())
+            if status != None:
+                fraction, elapsed = status
+                
+                self.progress = float(fraction)
+                if self.progress > 1.0:
+                    # A fix for how progress is calculated because producers can render a bit longer then required.
+                    self.progress = 1.0
+
+                self.elapsed = float(elapsed)
+                self.text = _("Tracking Data Render") + " " + self.data_label #self.editable_properties[0].clip.name
+
+                job_msg = self.get_job_queue_message()
+                
+                update_job_queue(job_msg)
+            else:
+                # Process start/stop on their own and we hit trying to get non-existing status for e.g completed renders.
+                pass
+
+    def update_filter_and_gui(self):    
+        final_label = PROJECT().add_tracking_data(self.data_label, self.data_file_path)
+        self.analyze_editor.analysis_complete(final_label, self.data_file_path)
+
+    def abort_render(self):
+        trackingheadless.abort_render(self.parent_folder, self.get_session_id())
+
+
 
 class ProxyRenderJobQueueObject(AbstractJobQueueObject):
 
@@ -591,9 +756,7 @@ class ProxyRenderJobQueueObject(AbstractJobQueueObject):
             proxy_attr_str = proxy_attr_str.replace(FFMPEG_ATTR_SOURCEFILE, self.render_data.media_file_path)
             # Set proxy file.
             proxy_attr_str = proxy_attr_str.replace(FFMPEG_ATTR_PROXYFILE, self.render_data.proxy_file_path)
-            
-            print(proxy_attr_str)
-            
+
             ffmpeg_command = "ffmpeg -i " + str(proxy_attr_str)
             
             self.ffmpeg_start = time.monotonic()
@@ -627,7 +790,7 @@ class ProxyRenderJobQueueObject(AbstractJobQueueObject):
                         self.progress = 1.0
 
                     self.elapsed = float(elapsed)
-                    self.text = self.get_job_name()
+                    self.text = _("Proxy Render")  + " " + self.get_job_name()
 
                     job_msg = self.get_job_queue_message()
                     update_job_queue(job_msg)
@@ -648,7 +811,7 @@ class ProxyRenderJobQueueObject(AbstractJobQueueObject):
                 if self.progress > 1.0:
                     self.progress = 0.99
                     
-                self.text = self.get_job_name()
+                self.text = _("Proxy Render")  + " " + self.get_job_name()
 
                 job_msg = self.get_job_queue_message()
                 update_job_queue(job_msg)
