@@ -24,6 +24,8 @@ Module handles keyevents.
 
 from gi.repository import Gdk, Gtk
 
+import re
+
 import appconsts
 import clipeffectseditor
 import compositeeditor
@@ -42,12 +44,10 @@ import menuactions
 import modesetting
 import monitorevent
 import movemodes
-import multitrimmode
 import shortcuts
 import shortcutsquickeffects
-import re
 import render
-import rotomask
+import targetactions
 import tlineaction
 import tlinewidgets
 import tlineypage
@@ -57,14 +57,49 @@ import updater
 import projectaction
 import workflow
 
-
+# TODO: We should consider integrating some parts of this with targetactions.py
+# TODO:
+# TODO: As of this writing, targetactions.py has a superset of targetable
+# TODO: actions, as compared to keyevents.py, totally separate from any keyboard
+# TODO: event handling. There are a few new named target actions in there that
+# TODO: aren't available in here. There is also currently a lot code duplication
+# TODO: between the two modules. See targetactions.py for more details.
+# TODO:
+# TODO: At a minimum, if you add or modify any of the key actions in here,
+# TODO: please consider updating targetactions.py as well. Right now there
+# TODO: is a lot of duplication between these modules, and often a change
+# TODO: in one would warrant a change in the other.
+# TODO:
+# TODO: keyevents.py is all about handling key presses from the keyboard, and
+# TODO: routing those events to trigger actions in various parts of the program.
+# TODO:
+# TODO: targetactions.py is basically a bunch of zero-argument functions with
+# TODO: names based on the shortcut key names found here. It was created as part
+# TODO: of the USB HID work, so that USB jog/shuttle devices could have their
+# TODO: buttons target various actions within the program, without requiring
+# TODO: each USB driver to directly make connections to a dozen different parts
+# TODO: of the program to control it.
+# TODO:
+# TODO: So now we have two collections of shortcut key names which map to
+# TODO: basically the same actions, but in a different way. I originally wanted
+# TODO: to just use keyevents.py as the target for the USB driver actions, but
+# TODO: couldn't use it directly since this module is intertwined with the
+# TODO: main computer keyboard and its events.
+# TODO:
+# TODO: For now, I have integrated the new command targets from
+# TODO: targetactions.py into keyevents.py, both for completeness, and also as
+# TODO: a proof of concept as to how we might migrate some of the other code
+# TODO: in here over to call targetactions.py
+# TODO:
+# TODO:   -- Nathan Rosenquist (@ratherlargerobot)
+# TODO:      Feb 2022
 
 # ------------------------------------- keyboard events
 def key_down(widget, event):
     """
     Global key press listener.
     """
-    
+
     # Handle ESCAPE.
     if event.keyval == Gdk.KEY_Escape:
         if editorstate.current_is_move_mode() == False:
@@ -100,7 +135,7 @@ def key_down(widget, event):
             # by stopping signal.
             gui.editor_window.window.emit_stop_by_name("key_press_event")
         return was_handled
-
+    
     # Insert shortcut keys need more focus then timeline shortcuts.
     # these may already have been handled in timeline focus events.
     was_handled = _handle_extended_monitor_focus_events(event)
@@ -150,20 +185,14 @@ def key_down(widget, event):
     # Select all with CTRL + A in media panel.
     if event.keyval == Gdk.KEY_a:
         if (event.get_state() & Gdk.ModifierType.CONTROL_MASK):
-            if gui.media_list_view.widget.has_focus() or gui.media_list_view.widget.get_focus_child() != None:
+            if gui.media_list_view.widget.has_focus() or gui.media_list_view.widget.get_focus_child() != None: 
                 gui.media_list_view.select_all()
                 return True
-        
+
     if event.keyval == Gdk.KEY_F11:
         menuactions.toggle_fullscreen()
         return True
 
-    #debug.test help 
-    #if event.keyval == Gdk.KEY_F12:
-    #    if (event.get_state() & Gdk.ModifierType.CONTROL_MASK):
-    #        rotomask.show_rotomask()
-    #    return True
-
     # Key event was not handled here.
     return False
     
@@ -211,9 +240,21 @@ def _handle_tline_key_event(event):
     if action == 'to_mark_out':
         monitorevent.to_mark_out_pressed()
         return True
+    if action == 'clear_mark_in':
+        targetactions.clear_mark_in()
+        return True
+    if action == 'clear_mark_out':
+        targetactions.clear_mark_out()
+        return True
     if action == 'clear_io_marks':
         monitorevent.marks_clear_pressed()
         return True
+    if action == 'play':
+        targetactions.play()
+        return True
+    if action == 'stop':
+        targetactions.stop()
+        return True
     if action == 'play_pause':
         if PLAYER().is_playing():
             monitorevent.stop_pressed()
@@ -226,6 +267,12 @@ def _handle_tline_key_event(event):
         else:
             monitorevent.start_marks_looping()
         return True
+    if action == 'display_clip_in_monitor':
+        targetactions.display_clip_in_monitor()
+        return True
+    if action == 'display_sequence_in_monitor':
+        targetactions.display_sequence_in_monitor()
+        return True
     if action == 'switch_monitor':
         updater.switch_monitor_display()
         return True
@@ -259,12 +306,21 @@ def _handle_tline_key_event(event):
     if action == 'trim_end':
         tlineaction.trim_end_pressed()
         return True
+    if action == 'monitor_show_video':
+        tlineaction.set_monitor_display_mode(appconsts.PROGRAM_OUT_MODE)
+        return True
+    if action == 'monitor_show_scope':
+        tlineaction.set_monitor_display_mode(appconsts.VECTORSCOPE_MODE)
+        return True
+    if action == 'monitor_show_rgb':
+        tlineaction.set_monitor_display_mode(appconsts.RGB_PARADE_MODE)
+        return True
+
+    # Key bindings for keyboard trimming
     if action == 'toggle_track_output':
         trackaction.toggle_track_output()
         return True
 
-        
-    # Key bindings for keyboard trimming.
     if editorstate.current_is_active_trim_mode() == True:
         if action == 'prev_frame':
             trimmodes.left_arrow_pressed((event.get_state() & Gdk.ModifierType.CONTROL_MASK))
@@ -332,11 +388,14 @@ def _handle_tline_key_event(event):
                 seek_amount = -1
             else:
                 seek_amount = 1
+
+            if (event.get_state() & Gdk.ModifierType.CONTROL_MASK):
+                PLAYER().slowmo_seek_delta(seek_amount)
+                return True
                 
             if (event.get_state() & Gdk.ModifierType.SHIFT_MASK):
                 seek_amount = seek_amount * prefs.ffwd_rev_shift
-            if (event.get_state() & Gdk.ModifierType.CONTROL_MASK):
-                seek_amount = seek_amount * prefs.ffwd_rev_ctrl
+
             if (event.get_state() & Gdk.ModifierType.LOCK_MASK):
                 seek_amount = seek_amount * prefs.ffwd_rev_caps
             PLAYER().seek_delta(seek_amount)
@@ -451,6 +510,12 @@ def _handle_extended_monitor_focus_events(event):
     if action == 'log_range':
         medialog.log_range_clicked()
         return True
+    if action == 'display_clip_in_monitor':
+        targetactions.display_clip_in_monitor()
+        return True
+    if action == 'display_sequence_in_monitor':
+        targetactions.display_sequence_in_monitor()
+        return True
     if action == 'switch_monitor':
         updater.switch_monitor_display()
         return True
@@ -498,10 +563,10 @@ def _get_shortcut_action(event):
         if modifier == "":
             modifier = 'None'
         try:
-            action = _secondary_dict[''.join(sorted(re.sub('[\s]','',modifier.lower())))]
+            action = _secondary_dict[''.join(sorted(re.sub(r'[\s]','',modifier.lower())))]
         except:
             try:
-                action = _secondary_dict[''.join(sorted(re.sub('[\s]','','Any'.lower())))]
+                action = _secondary_dict[''.join(sorted(re.sub(r'[\s]','','Any'.lower())))]
             except:
                 action = 'None'
         return action
@@ -519,7 +584,25 @@ def _handle_configurable_global_events(event):
     if action == 'open_next':
         projectaction.open_next_media_item_in_monitor()
         return True
-        
+    if action == 'open_prev':
+        projectaction.open_prev_media_item_in_monitor()
+        return True
+    if action == "append_from_bin":
+        if gui.media_list_view.widget.has_focus() or gui.media_list_view.widget.get_focus_child() != None: 
+            projectaction.append_selected_media_clips_into_timeline()
+            return True
+    if action == "move_media":
+        gui.media_list_view.init_move()
+    if action == 'monitor_show_video':
+        tlineaction.set_monitor_display_mode(appconsts.PROGRAM_OUT_MODE)
+        return True
+    if action == 'monitor_show_scope':
+        tlineaction.set_monitor_display_mode(appconsts.VECTORSCOPE_MODE)
+        return True
+    if action == 'monitor_show_rgb':
+        tlineaction.set_monitor_display_mode(appconsts.RGB_PARADE_MODE)
+        return True
+
     return False
     
 def _handle_clip_key_event(event):
@@ -537,10 +620,13 @@ def _handle_clip_key_event(event):
                 seek_amount = -1
             else:
                 seek_amount = 1
+            
+            if (event.get_state() & Gdk.ModifierType.CONTROL_MASK):
+                PLAYER().slowmo_seek_delta(seek_amount)
+                return True
+                
             if (event.get_state() & Gdk.ModifierType.SHIFT_MASK):
                 seek_amount = seek_amount * prefs.ffwd_rev_shift
-            if (event.get_state() & Gdk.ModifierType.CONTROL_MASK):
-                seek_amount = seek_amount * prefs.ffwd_rev_ctrl
             if (event.get_state() & Gdk.ModifierType.LOCK_MASK):
                 seek_amount = seek_amount * prefs.ffwd_rev_caps
             PLAYER().seek_delta(seek_amount)
@@ -570,6 +656,12 @@ def _handle_clip_key_event(event):
             else:
                  monitorevent.down_arrow_seek_on_monitor_clip()
                  return True
+        if action == 'play':
+            targetactions.play()
+            return True
+        if action == 'stop':
+            targetactions.stop()
+            return True
         if action == 'play_pause':
             if PLAYER().is_playing():
                 monitorevent.stop_pressed()
@@ -594,6 +686,12 @@ def _handle_clip_key_event(event):
         if action == 'to_mark_out':
             monitorevent.to_mark_out_pressed()
             return True
+        if action == 'clear_mark_in':
+            targetactions.clear_mark_in()
+            return True
+        if action == 'clear_mark_out':
+            targetactions.clear_mark_out()
+            return True
         if action == 'clear_io_marks':
             monitorevent.marks_clear_pressed()
             return True
@@ -652,6 +750,12 @@ def _handle_geometry_editor_keys(event):
                         return True
                     if event.keyval == Gdk.KEY_plus:
                         pass # not impl
+                    if action == 'play':
+                        targetactions.play()
+                        return True
+                    if action == 'stop':
+                        targetactions.stop()
+                        return True
                     if action == 'play_pause':
                         if PLAYER().is_playing():
                             monitorevent.stop_pressed()
@@ -669,7 +773,25 @@ def _handle_geometry_editor_keys(event):
 def _handle_effects_editor_keys(event):
     action = _get_shortcut_action(event)
     focus_editor = _get_focus_keyframe_editor(clipeffectseditor.keyframe_editor_widgets)
+
+                        
     if focus_editor != None:
+        if focus_editor.get_focus_child() != None:
+            if focus_editor.__class__ == keyframeeditor.FilterRectGeometryEditor or \
+                focus_editor.__class__ == keyframeeditor.FilterRotatingGeometryEditor or \
+                focus_editor.__class__ == keyframeeditor.GeometryNoKeyframes:
+                if ((event.keyval == Gdk.KEY_Left) 
+                    or (event.keyval == Gdk.KEY_Right)
+                    or (event.keyval == Gdk.KEY_Up)
+                    or (event.keyval == Gdk.KEY_Down)):
+                    focus_editor.arrow_edit(event.keyval, (event.get_state() & Gdk.ModifierType.CONTROL_MASK), (event.get_state() & Gdk.ModifierType.SHIFT_MASK))
+                    return True
+        if action == 'play':
+            targetactions.play()
+            return True
+        if action == 'stop':
+            targetactions.stop()
+            return True
         if action == 'play_pause':
             if PLAYER().is_playing():
                 monitorevent.stop_pressed()
@@ -688,10 +810,13 @@ def _handle_effects_editor_keys(event):
                 seek_amount = -1
             else:
                 seek_amount = 1
+            
+            if (event.get_state() & Gdk.ModifierType.CONTROL_MASK):
+                PLAYER().slowmo_seek_delta(seek_amount)
+                return True
+                
             if (event.get_state() & Gdk.ModifierType.SHIFT_MASK):
                 seek_amount = seek_amount * prefs.ffwd_rev_shift
-            if (event.get_state() & Gdk.ModifierType.CONTROL_MASK):
-                seek_amount = seek_amount * prefs.ffwd_rev_ctrl
             if (event.get_state() & Gdk.ModifierType.LOCK_MASK):
                 seek_amount = seek_amount * prefs.ffwd_rev_caps
             PLAYER().seek_delta(seek_amount)
