@@ -21,10 +21,11 @@
 """
 Module handles Keyframe tool functionality
 """
-from gi.repository import Pango, PangoCairo, Gtk
-from gi.repository import  Gdk
+from gi.repository import Pango, PangoCairo
+from gi.repository import Gdk, GLib
 
 import cairo
+import copy
 import math
 
 import appconsts
@@ -34,12 +35,12 @@ import edit
 from editorstate import current_sequence
 from editorstate import PLAYER
 import gui
+import guipopoverclip
 import guiutils
 import mltfilters
 import propertyedit
 import propertyparse
 import respaths
-import sequence
 import tlinewidgets
 import updater
 
@@ -90,15 +91,9 @@ POSITION_DRAG = 1
 KF_DRAG_DISABLED = 2 # Not used currently
 KF_DRAG_FRAME_ZERO_KF = 3
 KF_DRAG_BETWEEN_TWO = 4
+KF_DRAG_MULTIPLE = 5
 
 DRAG_MIN_Y = 4 # To make start value slightly magnetic, makes easier to move position without changing value.
-
-hamburger_menu = Gtk.Menu()
-oor_before_menu = Gtk.Menu()
-oor_after_menu = Gtk.Menu()
-value_snapping_menu = Gtk.Menu()
-params_menu =  Gtk.Menu()
-kf_menu = Gtk.Menu()
   
 edit_data = None
 enter_mode = None
@@ -245,12 +240,7 @@ def get_clip_kftool_editable_params_data(clip):
                 kftool_editable_params.append(param_data)
     
     return kftool_editable_params
-    """
-                editable_params_exist = True
-                item_text = filt.info.name  + ": " +  disp_name.replace("!", " ")
-                param_item = self._get_menu_item(item_text, self._param_edit_item_activated, param_data)
-                menu.add(param_item)
-    """
+
                 
 def update_clip_frame(tline_frame):
     if _kf_editor != None and edit_data != None and edit_data["initializing"] != True:
@@ -390,6 +380,11 @@ def mouse_press(event, frame):
 
     clip = track.clips[clip_index]
 
+    # Exit on pressing blank clip.
+    if clip.is_blanck_clip == True:
+        exit_tool()
+        return
+
     if _has_deprecated_volume_filter(clip) == True:
         set_no_clip_edit_data()
         primary_txt = _("This Clip has a deprecated Volume filter and cannot be edited with Keyframe Tool!")
@@ -642,7 +637,7 @@ class TLineKeyFrameEditor:
                 kf, frame, kf_index, kf_type, kf_pos_x, kf_pos_y = kf_positions[i]
                 # This is trying to get rid of some draw artifacts by limiting x positions.
                 # kf_pos_x can get really large values with long clips and large zooms
-                # and Cairo fails and handling those values.
+                # and Cairo fails at handling those values.
                 if kf_pos_x < -10000:
                     kf_pos_x = -10000
                 if kf_pos_x > 10000:
@@ -1097,7 +1092,7 @@ class TLineKeyFrameEditor:
                 grad.add_color_stop_rgba(*tlinewidgets.BLANK_CLIP_COLOR_GRAD)
                 grad.add_color_stop_rgba(*tlinewidgets.BLANK_CLIP_COLOR_GRAD_L)
                 cr.set_source(grad)
-        elif track.type == sequence.VIDEO:
+        elif track.type == appconsts.VIDEO:
             if clip.container_data != None:
                 if clip.container_data.rendered_media_range_in == -1: 
                     clip_bg_col = (0.7, 0.3, 0.3)
@@ -1107,7 +1102,7 @@ class TLineKeyFrameEditor:
                     clip_bg_col = (0.7, 0.3, 0.3)
                     cr.set_source_rgb(*tlinewidgets.CONTAINER_CLIP_RENDERED_COLOR)
                     clip_bg_col = tlinewidgets.CONTAINER_CLIP_RENDERED_COLOR
-            elif clip.media_type == sequence.VIDEO: 
+            elif clip.media_type == appconsts.VIDEO: 
                 grad = cairo.LinearGradient (0, y, 0, y + track_height)
                 pos,r,g,b,ad = tlinewidgets.CLIP_COLOR_GRAD
                 grad.add_color_stop_rgba(pos,r,g,b,a)
@@ -1137,7 +1132,7 @@ class TLineKeyFrameEditor:
         """
         # Check if menu icons hit
         if self._hamburger_hit(event.x, event.y) == True:
-            self._show_hamburger_menu(gui.tline_canvas.widget, event)
+            self._show_hamburger_popover(gui.tline_canvas.widget, event)
             return
 
         lx = self._legalize_x(event.x)
@@ -1176,16 +1171,22 @@ class TLineKeyFrameEditor:
             return
 
         if hit_kf == - 1:
-            self.edit_value =  self.get_snapped_value(ly)
+            self.edit_value = self.get_snapped_value(ly)
         else:
             frame, value, kf_type = self.keyframes[hit_kf]
-            self.edit_value =  self.get_snapped_value(ly)
+            self.edit_value = self.get_snapped_value(ly)
             self.current_clip_frame = frame
         if hit_kf == 0:
             self.current_mouse_action = KF_DRAG_FRAME_ZERO_KF
         elif hit_kf != -1:
-            self.current_mouse_action = KF_DRAG
-            
+            if event.get_state() & Gdk.ModifierType.SHIFT_MASK:
+                self.current_mouse_action = KF_DRAG_MULTIPLE
+                self.mouse_drag_kfs_copy = copy.deepcopy(self.keyframes)
+                self.mouse_drag_start_frame = self.keyframes[self.active_kf_index][0]
+                self.mouse_drag_start_value = self.get_snapped_value(ly)
+            else:
+                self.current_mouse_action = KF_DRAG
+
             prev_frame, val, kf_type = self.keyframes[hit_kf - 1]
             self.drag_min = prev_frame  + 1
             try:
@@ -1193,7 +1194,10 @@ class TLineKeyFrameEditor:
                 self.drag_max = next_frame - 1
             except:
                 self.drag_max = self.clip_in + self.clip_length
-                
+
+            if self.current_mouse_action == KF_DRAG_MULTIPLE:
+                self.drag_max = self.clip_in + self.clip_length
+                                
         updater.repaint_tline()
 
     def motion_notify_event(self, x, y, state):
@@ -1214,7 +1218,9 @@ class TLineKeyFrameEditor:
             self.current_clip_frame = frame
             self.clip_editor_frame_changed(self.current_clip_frame)
             updater.repaint_tline()
-        elif self.current_mouse_action == KF_DRAG or self.current_mouse_action == KF_DRAG_FRAME_ZERO_KF:
+        elif self.current_mouse_action == KF_DRAG or \
+             self.current_mouse_action == KF_DRAG_FRAME_ZERO_KF or \
+             self.current_mouse_action == KF_DRAG_MULTIPLE:
             frame = self._get_drag_frame(lx)
             if self.current_mouse_action == KF_DRAG_FRAME_ZERO_KF:
                 frame = 0
@@ -1225,6 +1231,19 @@ class TLineKeyFrameEditor:
                 self.set_active_kf_frame_and_value(frame, value)
             else:
                 self.set_active_kf_frame_and_value(frame, self.edit_value)
+        
+            if self.current_mouse_action == KF_DRAG_MULTIPLE:
+                for kf_index in range(self.active_kf_index + 1, len(self.keyframes)):
+                    start_frame = self.mouse_drag_kfs_copy[kf_index][0]
+                    frame_delta = frame - self.mouse_drag_start_frame
+                    start_value = self.mouse_drag_kfs_copy[kf_index][1]
+                    if self.value_drag_on == True:        
+                        value_delta = value - self.mouse_drag_start_value
+                        legal_value = self.get_legalized_value(start_value + value_delta)
+                        self.set_kf_frame_and_value(kf_index, start_frame + frame_delta, legal_value)
+                    else:
+                        self.set_kf_frame_and_value(kf_index, start_frame + frame_delta, start_value)
+
             if _playhead_follow_kf == True:
                 self.current_clip_frame = frame
                 self.clip_editor_frame_changed(self.current_clip_frame)
@@ -1265,6 +1284,17 @@ class TLineKeyFrameEditor:
                 self.set_active_kf_frame_and_value(frame, self.edit_value)
                 self.hack_fix_for_zero_one_keyframe_problem()
 
+            if self.current_mouse_action == KF_DRAG_MULTIPLE:
+                for kf_index in range(self.active_kf_index + 1, len(self.keyframes)):
+                    start_frame = self.mouse_drag_kfs_copy[kf_index][0]
+                    frame_delta = frame - self.mouse_drag_start_frame
+                    start_value = self.mouse_drag_kfs_copy[kf_index][1]
+                    if self.value_drag_on == True:        
+                        value_delta = value - self.mouse_drag_start_value
+                        self.set_kf_frame_and_value(kf_index, start_frame + frame_delta, start_value + value_delta)
+                    else:
+                        self.set_kf_frame_and_value(kf_index, start_frame + frame_delta, start_value)
+
             if _playhead_follow_kf == True:
                 self.current_clip_frame = frame
                 self.clip_editor_frame_changed(self.current_clip_frame)
@@ -1287,6 +1317,17 @@ class TLineKeyFrameEditor:
         
         return value
 
+    def get_legalized_value(self, value):
+        editable_property = edit_data["editable_property"] 
+        adjustment = editable_property.get_input_range_adjustment()
+        lower = adjustment.get_lower()
+        upper = adjustment.get_upper()
+        if value < lower:
+            value = lower 
+        if value > upper:
+            value = upper 
+        return value
+
     def update_between_drag_keyframes_values(self, value):
         # Replace prev and next keyframes with new keyframes with updated values.
         i = self.prev_frame_line(self.between_drag_start_x) - 1
@@ -1409,6 +1450,10 @@ class TLineKeyFrameEditor:
         frame, val, kf_type = self.keyframes.pop(self.active_kf_index)
         self.keyframes.insert(self.active_kf_index,(new_frame, new_value, kf_type))
 
+    def set_kf_frame_and_value(self, kf_index, new_frame, new_value):
+        frame, val, kf_type = self.keyframes.pop(kf_index)
+        self.keyframes.insert(kf_index,(new_frame, new_value, kf_type))
+        
     def set_active_kf_type(self, new_type):
         frame, val, kf_type = self.keyframes.pop(self.active_kf_index)
         self.keyframes.insert(self.active_kf_index,(frame, val, new_type))
@@ -1579,36 +1624,14 @@ class TLineKeyFrameEditor:
 
     # ------------------------------------------------------------ menus
     def _show_kf_menu(self, event):
-        menu = kf_menu
-        guiutils.remove_children(menu)
-        kf_type = self.get_active_kf_type()
-        self._create_keyframe_type_submenu(kf_type, menu, self._kf_menu_callback)
-        menu.popup(None, None, None, None, event.button, event.time)
+        guipopoverclip.kftype_select_popover_menu_show(gui.tline_canvas.widget, self.get_active_kf_type(), event.x, event.y, self._kf_popover_callback)
 
-    def _create_keyframe_type_submenu(self, kf_type, menu, callback):
-        linear_item = Gtk.RadioMenuItem()
-        linear_item.set_label(_("Linear"))
-        if kf_type == appconsts.KEYFRAME_LINEAR:
-            linear_item.set_active(True)
-        linear_item.connect("activate", callback, "linear")
-        linear_item.show()
-        menu.append(linear_item)
-
-        smooth_item = Gtk.RadioMenuItem().new_with_label([linear_item], _("Smooth"))
-        smooth_item.connect("activate", callback, "smooth")
-        if kf_type == appconsts.KEYFRAME_SMOOTH:
-            smooth_item.set_active(True)
-        smooth_item.show()
-        menu.append(smooth_item)
-
-        discrete_item = Gtk.RadioMenuItem.new_with_label([linear_item], _("Discrete"))
-        discrete_item.connect("activate", callback, "discrete")
-        if kf_type == appconsts.KEYFRAME_DISCRETE:
-            discrete_item.set_active(True)
-        discrete_item.show()
-        menu.append(discrete_item)
-
-    def _kf_menu_callback(self, widget, data):
+    def _kf_popover_callback2(self, action, variant):
+        print(action, variant)
+        
+    def _kf_popover_callback(self, action, variant):
+        data = variant.get_string()
+        
         if data == "linear":
             self.set_active_kf_type(appconsts.KEYFRAME_LINEAR)
         elif data == "smooth":
@@ -1616,136 +1639,16 @@ class TLineKeyFrameEditor:
         elif data == "discrete":
             self.set_active_kf_type(appconsts.KEYFRAME_DISCRETE)
 
+        action.set_state(variant)
+        guipopoverclip._kf_select_popover.hide()
         updater.repaint_tline()
 
-    def _build_oor_before_menu(self, menu):
-        guiutils.remove_children(menu)
-        before_kfs = len(self.get_out_of_range_before_kfs())
+    def _show_hamburger_popover(self, widget, event):
+        guipopoverclip.kftool_popover_menu_show(widget, self, event.x, event.y, self._popover_callback, self._snapping_menu_item_item_activated)
 
-        if before_kfs == 0 or (before_kfs == 1 and len(self.keyframes) == 1):
-            item = self._get_menu_item(_("No Edit Actions currently available"), self._oor_menu_item_activated, "noop" )
-            item.set_sensitive(False)
-            menu.add(item)
-
-        if before_kfs > 1:
-            menu.add(self._get_menu_item(_("Delete all but first Keyframe before Clip Range"), self._oor_menu_item_activated, "delete_all_before" ))
-            sep = Gtk.SeparatorMenuItem()
-            sep.show()
-            menu.add(sep)
-
-        if len(self.keyframes) > 1 and before_kfs > 0:
-            menu.add(self._get_menu_item(_("Set Keyframe at Frame 0 to value of next Keyframe"), self._oor_menu_item_activated, "zero_next" ))
-
-    def _build_oor_after_menu(self, menu):
-        guiutils.remove_children(menu)
-        after_kfs = len(self.get_out_of_range_after_kfs())
-
-        if after_kfs == 0:
-            item = self._get_menu_item(_("No Edit Actions currently available"), self._oor_menu_item_activated, "noop" )
-            item.set_sensitive(False)
-            menu.add(item)
-            return
-
-        if self.edit_type == BRIGHTNESS_KF_EDIT:
-            menu.add(self._get_menu_item(_("Delete all Keyframes after Clip Range"), self._oor_menu_item_activated, "delete_all_after" ))
-        elif self.edit_type == VOLUME_KF_EDIT:
-            if after_kfs > 1:
-                menu.add(self._get_menu_item(_("Delete all but last Keyframe after Clip Range"), self._oor_menu_item_activated, "delete_all_but_last_after" ))
-            else:
-                item = self._get_menu_item(_("No Edit Actions currently available"), self._oor_menu_item_activated, "noop" )
-                item.set_sensitive(False)
-                menu.add(item)
-
-    def _show_hamburger_menu(self, widget, event):
-        menu = hamburger_menu
-        guiutils.remove_children(menu)
-
-        if edit_data["track"].type == appconsts.VIDEO:
-            
-            editable_params_exist = False
-            kftool_params = get_clip_kftool_editable_params_data(edit_data["clip"])
-            
-            if len(kftool_params) > 0:
-                editable_params_exist = True
-                for param_data in kftool_params:
-                    p_name, filt, i, disp_name = param_data 
-                                    
-                    item_text = filt.info.name  + ": " +  disp_name
-                    param_item = self._get_menu_item(item_text, self._param_edit_item_activated, param_data)
-                    menu.add(param_item)
-
-            if editable_params_exist == False:
-                sep = Gtk.SeparatorMenuItem()
-                sep.show()
-                menu.add(sep)
-
-        leading_menu_item = Gtk.MenuItem(_("Leading Keyframes"))
-        leading_menu = Gtk.Menu()
-        self._build_oor_before_menu(leading_menu)
-        leading_menu_item.set_submenu(leading_menu)
-        leading_menu_item.show_all()
-        menu.add(leading_menu_item)
-        
-        trailing_menu_item = Gtk.MenuItem(_("Trailing Keyframes"))
-        trailing_menu = Gtk.Menu()
-        self._build_oor_after_menu(trailing_menu)
-        trailing_menu_item.set_submenu(trailing_menu)
-        trailing_menu_item.show_all()
-        menu.add(trailing_menu_item)
-        
-        sep = Gtk.SeparatorMenuItem()
-        sep.show()
-        menu.add(sep)
-        
-        playhead_follow_item = Gtk.CheckMenuItem()
-        playhead_follow_item.set_label(_("Playhead Follows Dragged Keyframe"))
-        playhead_follow_item.set_active(_playhead_follow_kf)
-        playhead_follow_item.connect("activate", self._oor_menu_item_activated, "playhead_follows")
-        playhead_follow_item.show()
-        menu.add(playhead_follow_item)
-
-        value_snapping_item = Gtk.MenuItem(_("Value Snapping"))
-        value_snapping_menu = Gtk.Menu()
-        guiutils.remove_children(value_snapping_menu)
-
-        first_item = Gtk.RadioMenuItem()
-        first_item.set_label("1")
-        if _snapping == 1:
-            first_item.set_active(True)
-        first_item.show()
-        value_snapping_menu.append(first_item)
-        first_item.connect("activate", self._oor_menu_item_activated, "1")
-
-        radio_item = Gtk.RadioMenuItem.new_with_label([first_item], "2")
-        if _snapping == 2:
-            radio_item.set_active(True)
-        radio_item.connect("activate", self._oor_menu_item_activated, "2")
-        value_snapping_menu.append(radio_item)
-        radio_item.show()
-
-        radio_item = Gtk.RadioMenuItem.new_with_label([first_item], "5")
-        if _snapping == 5:
-            radio_item.set_active(True)
-        radio_item.connect("activate", self._oor_menu_item_activated, "5")
-        value_snapping_menu.append(radio_item)
-        radio_item.show()
-
-        value_snapping_item.set_submenu(value_snapping_menu)
-        value_snapping_item.show_all()
-        menu.add(value_snapping_item)
-        
-        sep = Gtk.SeparatorMenuItem()
-        sep.show()
-        menu.add(sep)
-        
-        menu.add(self._get_menu_item(_("Exit Edit"), self._oor_menu_item_activated, "exit" ))
-        
-        menu.popup(None, None, None, None, event.button, event.time)
-
-    def _oor_menu_item_activated(self, widget, data):
-        global _snapping
-        
-        if data == "delete_all_before":
+    def _popover_callback(self, action, variant, data):
+        msg, data_msg = data
+        if msg == "delete_all_before":
             keep_doing = True
             while keep_doing:
                 try:
@@ -1756,8 +1659,7 @@ class TLineKeyFrameEditor:
                         keep_doing = False 
                 except:
                     keep_doing = False
-        
-        elif data == "delete_all_but_last_after":
+        elif msg == "delete_all_but_last_after":
         
             keep_doing = True
             index = 1
@@ -1770,14 +1672,13 @@ class TLineKeyFrameEditor:
                         index += 1
                 except:
                     keep_doing = False
-                    
-        elif data == "zero_next":
-            frame_zero, frame_zero_value = self.keyframes[0]
+        elif msg == "zero_next":
+            frame_zero, frame_zero_value, kf_type = self.keyframes[0]
             frame, value, kf_type = self.keyframes[1]
             self.keyframes.pop(0)
             self.keyframes.insert(0, (frame_zero, value, kf_type))
             self.update_property_value()
-        elif data == "delete_all_after":
+        elif msg == "delete_all_after":
             delete_done = False
             for i in range(0, len(self.keyframes)):
                 frame, value, kf_type = self.keyframes[i]
@@ -1792,33 +1693,40 @@ class TLineKeyFrameEditor:
                     delete_done = True
                 if delete_done:
                     break
-        elif data == "edit_brightness":
-            init_tool_for_clip(edit_data["clip"] , edit_data["track"], BRIGHTNESS_KF_EDIT)
-        elif data == "edit_volume":
-            init_tool_for_clip(edit_data["clip"] , edit_data["track"], VOLUME_KF_EDIT)
-        elif data == "exit":
-            set_no_clip_edit_data()
-        elif data == "playhead_follows":
+        elif msg == "edit_param":
+            params_data = self.get_clip_kftool_editable_params_data()
+            param_data = params_data[int(data_msg)]
+            init_tool_for_clip(edit_data["clip"],  edit_data["track"], PARAM_KF_EDIT, param_data)
+        elif msg == "playhead_follows":
+            new_state = not(action.get_state().get_boolean())
             global _playhead_follow_kf
-            _playhead_follow_kf = widget.get_active()
-        elif data == "1":
-            _snapping = 1
-        elif data == "2":
-            _snapping = 2
-        elif data == "5":
-            _snapping = 5
-
+            _playhead_follow_kf = new_state
+            action.set_state(GLib.Variant.new_boolean(new_state))
+        elif msg == "exit":
+            set_no_clip_edit_data()
+    
         updater.repaint_tline()
 
-    def _param_edit_item_activated(self,  widget, data):
-        init_tool_for_clip(edit_data["clip"],  edit_data["track"], PARAM_KF_EDIT, data)
+    def _snapping_menu_item_item_activated(self, action, new_value_variant):
+        msg = int(new_value_variant.get_string())
+        global _snapping
+        _snapping = msg
+        action.set_state(new_value_variant)
+        guipopoverclip._kftool_popover.hide()
 
-    def _get_menu_item(self, text, callback, data):
-        item = Gtk.MenuItem(text)
-        item.connect("activate", callback, data)
-        item.show()
-        return item
+    def get_clip_kftool_editable_params_data(self):
+        return get_clip_kftool_editable_params_data(edit_data["clip"])
 
+    def get_playback_follows(self):
+        return _playhead_follow_kf
+
+    def get_snapping_value(self):
+        return _snapping
+
+    def _set_playhead_follows(self):
+        global _playhead_follow_kf
+        _playhead_follow_kf = widget.get_active()
+            
     # ---------------------------------- Modify kf curve between two kf ------------------
     def prev_frame_line(self, lx):
         """Find the index of the keyframe before the event.x."""
