@@ -17,6 +17,19 @@
     You should have received a copy of the GNU General Public License
     along with Flowblade Movie Editor.  If not, see <http://www.gnu.org/licenses/>.
 """
+
+"""
+Thís module handels creating, editing and cloning of
+Generators.
+
+Generators are container clips that use Fluxity Scripts to render 
+clips on timeline, see e.g. fluxity.py, containerclip.py, containeractions.py and simpleeditors.py.
+
+Naming was changed to 'Generator' from 'Media Plugin' at the last moment before release,
+so all code here refers to 'media plugins' instead of 'generators'.
+"""  
+
+
 from gi.repository import Gtk, GObject, Pango, Gio
 
 import cairo
@@ -75,11 +88,12 @@ _preview_canvas = None
 # --------------------------------------------------------- plugin
 class MediaPlugin:
     
-    def __init__(self, folder, script_file, name, category):
+    def __init__(self, folder, script_file, name, category, default_render):
         self.folder = folder
-        self.script_file = script_file
+        self.script_file = script_file # This is a fluxity script.
         self.name = name
         self.category = category
+        self.default_render = default_render
     
     def get_screenshot_file(self):
         return respaths.MEDIA_PLUGINS_PATH + self.folder + "/screenshot.png"
@@ -101,7 +115,7 @@ def init():
     global _plugins
     plugins_list = plugins_obj["plugins"]
     for plugin_data in plugins_list:
-        plugin = MediaPlugin(plugin_data["folder"], plugin_data["scriptfile"], plugin_data["name"], plugin_data["category"])
+        plugin = MediaPlugin(plugin_data["folder"], plugin_data["scriptfile"], plugin_data["name"], plugin_data["category"], plugin_data["defaultrender"])
         _plugins.append(plugin)
     
     load_groups = {}
@@ -184,7 +198,7 @@ def _add_media_plugin():
     if _add_plugin_window.import_select.get_active() == 0:
         _close_window()
         # Add as Container Clip
-        containerclip.create_fluxity_media_item_from_plugin(script_file, screenshot_file, _current_plugin_data_object)
+        containerclip.create_fluxity_media_item_from_plugin(script_file, screenshot_file, _current_plugin_data_object, None, _selected_plugin.default_render)
     else:
         # Add as Rendered Clip.
         _close_window()
@@ -228,7 +242,9 @@ def _clone_properties_callback(dialog, response_id, data, length_spin, name_entr
     new_plugin_edit_data["name"] = name
     new_plugin_edit_data["length"] = int(new_length)
 
-    containerclip.create_fluxity_media_item_from_plugin(old_cd.program, screenshot_file, new_plugin_edit_data)
+    default_render = toolsencoding.get_render_type_from_render_data(old_cd.render_data)
+ 
+    containerclip.create_fluxity_media_item_from_plugin(old_cd.program, screenshot_file, new_plugin_edit_data, None, default_render)
 
 def create_plugin_assests_for_media_import(old_cd):
     md_str = hashlib.md5(str(os.urandom(32)).encode('utf-8')).hexdigest()
@@ -453,6 +469,19 @@ class AddMediaPluginWindow(Gtk.Window):
         _current_screenshot_surface = new_selected_plugin.get_screenshot_surface()
         self.length_spin.set_value(_current_plugin_data_object["length"])
 
+        # Create and set default render options data.
+        global _current_render_data
+        default_render = _selected_plugin.default_render
+        if default_render == appconsts.DEFAULT_RENDER_CLIP:
+            _current_render_data = toolsencoding.create_container_clip_default_render_data_object(current_sequence().profile)
+            _current_render_data.do_video_render = True
+        elif default_render == appconsts.DEFAULT_RENDER_ALPHA_CLIP:
+            _current_render_data = toolsencoding.create_container_clip_default_alpha_render_data_object(current_sequence().profile)
+            _current_render_data.do_video_render = True
+        else:
+            _current_render_data = toolsencoding.create_container_clip_default_render_data_object(current_sequence().profile)
+        self._display_current_render_data()
+        
         self.producer.set_frame(0)
         self.tc_display.set_frame(0)
         self.producer.set_length(_current_plugin_data_object["length"])
@@ -509,7 +538,14 @@ class AddMediaPluginWindow(Gtk.Window):
     def _show_preview(self):
         global _selected_plugin, _current_screenshot_surface
         fctx = self._render_preview(int(self.producer.frame()))
-        _current_screenshot_surface = fctx.priv_context.frame_surface
+        
+        if fctx.error == None:
+            _current_screenshot_surface = fctx.priv_context.frame_surface
+        else:
+           _current_screenshot_surface = _selected_plugin.get_screenshot_surface()
+           primary_txt = _("Generator Render Error")
+           secondary_txt = str(fctx.error)
+           dialogutils. warning_message(primary_txt, secondary_txt, self)
 
         self.screenshot_canvas.queue_draw()
 
@@ -574,6 +610,10 @@ class AddMediaPluginWindow(Gtk.Window):
 
 
 class PosBarProducer:
+    """
+    Object of this class pretends to a MLT.Producer clip to make position bar and
+    timecode display widgets work here.
+    """
     
     def __init__(self):
         self.length = 200
