@@ -39,7 +39,6 @@ import titler
 import tlineaction
 import updater
 import undo
-import workflow
 
 # Default button orders for different layouts.
 DEFAULT_BUTTONS_TIMECODE_LEFT = ['undo_redo', 'zoom_buttons', 'edit_buttons', 'edit_buttons_2', 'edit_buttons_3', 'monitor_insert_buttons']
@@ -189,7 +188,7 @@ def _create_buttons(editor_window):
     # Aug-2019 - SvdB - BB
     prefs = editorpersistance.prefs
     size_adj = 1
-    if prefs.double_track_hights:
+    if guiutils.double_icon_size():
        size_adj = 2
 
     editor_window.big_TC = Gtk.Stack()
@@ -292,11 +291,10 @@ def _create_buttons(editor_window):
 
 def create_tool_selector(editor_window):
     size_adj = 1
-    if editorpersistance.prefs.double_track_hights:
+    if guiutils.double_icon_size():
        size_adj = 2
       
     tool_selector = guicomponents.ToolSelector(editor_window.tline_cursor_manager.mode_selector_pressed, m_pixbufs, 40*size_adj, 22*size_adj)
-    tool_selector.connect_launched_menu(workflow._tools_menu)
     return tool_selector
 
 def re_create_tool_selector(editor_window):
@@ -340,7 +338,7 @@ def fill_with_TC_LEFT_pattern(buttons_row, window):
     
     buttons_row.pack_start(get_buttons_group(5), False, True, 0)
     buttons_row.pack_start(Gtk.Label(), True, True, 0)
-    
+
 def fill_with_TC_MIDDLE_pattern(buttons_row, window):
     if editorstate.screen_size_small_width() == False:
         buttons_row.set_homogeneous(True)
