@@ -40,14 +40,28 @@ import updater
 
 TICKER_DELAY = 250 # in millis
 RENDER_TICKER_DELAY = 0.05
+SLOWMO_FRAME_DELAY = 0.1
+
+SDL_1 = 1
+SDL_2 = 2
+
+_sdl_consumer_version = SDL_1
+
+def set_sdl_consumer_version(consumer_version):
+    global _sdl_consumer_version
+    _sdl_consumer_version = consumer_version
+
+def get_sdl_consumer_version():
+    return _sdl_consumer_version
 
 class Player:
     
     def __init__(self, profile):
 
         self.init_for_profile(profile)
-        
-        self.start_ticker()
+        self.last_slowmo_seektime = 0.0
+        self.slowmo_ticker = None
+        self.consumer = None
             
     def init_for_profile(self, profile):
         # Get profile and create ticker for playback GUI updates
@@ -58,54 +72,43 @@ class Player:
         self.loop_start = -1
         self.loop_end = -1
         self.is_looping = False
-
-        # JACK audio
-        self.jack_output_filter = None
         
     def create_sdl_consumer(self):
         """
         Creates consumer with sdl output to a gtk+ widget.
         """
-        # SDL 2 consumer is created after
-        #if editorstate.get_sdl_version() == editorstate.SDL_2:
-        #    print "refuse SDL1 consumer"
-        #    return
 
-        print("Create SDL1 consumer...")
         # Create consumer and set params
-        self.consumer = mlt.Consumer(self.profile, "sdl")
+        if _sdl_consumer_version == SDL_2:
+            print("Create SDL2 consumer...")
+            self.consumer = mlt.Consumer(self.profile, "sdl2")
+            self.consumer.set("window_id", self.window_xid)
+            w = gui.tline_display.get_allocated_width()
+            h = gui.tline_display.get_allocated_height()
+            self.consumer.set("window_width", w)
+            self.consumer.set("window_height", h)
+        else:
+            print("Create SDL1 consumer...")
+            self.consumer = mlt.Consumer(self.profile, "sdl")
+            # SDL 1 consumer uses env param to communicate xid and 
+            # that has already been set.
         self.consumer.set("real_time", 1)
         self.consumer.set("rescale", "bicubic") # MLT options "nearest", "bilinear", "bicubic", "hyper"
         self.consumer.set("resize", 1)
         self.consumer.set("progressive", 1)
 
+        self.start_ticker()
+        
         # Hold ref to switch back from rendering
         self.sdl_consumer = self.consumer 
 
-    """
-    def create_sdl2_video_consumer(self):
+    def display_resized(self):
+        if self.consumer != None and _sdl_consumer_version == SDL_2:
+            w = gui.tline_display.get_allocated_width()
+            h = gui.tline_display.get_allocated_height()
+            self.consumer.set("window_width", w)
+            self.consumer.set("window_height", h)
 
-        widget = gui.editor_window.tline_display
-        self.set_sdl_xwindow(widget)
-        
-        # Create consumer and set params
-        self.consumer = mlt.Consumer(self.profile, "sdl")
-        self.consumer.set("real_time", 1)
-        self.consumer.set("rescale", "bicubic") # MLT options "nearest", "bilinear", "bicubic", "hyper"
-        self.consumer.set("resize", 1)
-        self.consumer.set("progressive", 1)
-        self.consumer.set("window_id", str(self.xid))
-        alloc = gui.editor_window.tline_display.get_allocation()
-        self.consumer.set("window_width", str(alloc.width))
-        self.consumer.set("window_height", str(alloc.height))
-        self.consumer.set("window_type", "widget")
-        self.consumer.set("renderer_type", "software")
-        # Hold ref to switch back from rendering
-        self.sdl_consumer = self.consumer 
-        
-        self.connect_and_start()
-    """
-    
     def set_scrubbing(self, scrubbing_active):
         if scrubbing_active == True:
             self.consumer.set("scrub_audio", 1)
@@ -116,9 +119,10 @@ class Player:
         """
         Connects SDL output to display widget's xwindow
         """
-        os.putenv('SDL_WINDOWID', str(widget.get_window().get_xid()))
-        #self.xid = widget.get_window().get_xid()
-        Gdk.flush()
+        self.window_xid = widget.get_window().get_xid()
+        if _sdl_consumer_version == SDL_1:
+            os.putenv('SDL_WINDOWID', str(widget.get_window().get_xid()))
+            Gdk.flush()
 
     def set_tracktor_producer(self, tractor):
         """
@@ -132,25 +136,18 @@ class Player:
         self.connect_and_start()
 
     def refresh(self): # Window events need this to get picture back
-        self.consumer.stop()
-        self.consumer.start()
-   
-        """
         if self.consumer == None:
             return 
-        if editorstate.get_sdl_version() == editorstate.SDL_2:
-            alloc = gui.editor_window.tline_display.get_allocation()
-            self.consumer.set("window_width", str(alloc.width))
-            self.consumer.set("window_height", str(alloc.height))
-            
-            self.consumer.stop()
-            self.consumer.start()
-        else:
-            self.consumer.stop()
-            self.consumer.start()
-        """
+
+        self.stop_timer_slowmo_playback()
+
+        self.consumer.stop()
+        self.consumer.start()
         
     def is_stopped(self):
+        if self.slowmo_ticker != None:
+            return False
+            
         return (self.producer.get_speed() == 0)
         
     def stop_consumer(self):
@@ -161,9 +158,8 @@ class Player:
         """
         Connects current procer and consumer and
         """
-        #if self.consumer == None:
-        #    return 
-
+        self.stop_timer_slowmo_playback()
+        
         self.consumer.purge()
         self.producer.set_speed(0)
         self.consumer.connect(self.producer)
@@ -172,23 +168,56 @@ class Player:
     def start_playback(self):
         """
         Starts playback from current producer
-        """        
+        """
+        self.stop_timer_slowmo_playback()
+        
         self.producer.set_speed(1)
         self.stop_ticker()
         self.start_ticker()
+
+    def get_speed(self):
+        if self.slowmo_ticker != None:
+            return self.slowmo_ticker.data
         
+        return self.producer.get_speed()
+
     def start_variable_speed_playback(self, speed):
         """
         Starts playback from current producer
         """
+        self.stop_timer_slowmo_playback()
+        
         self.producer.set_speed(speed)
         self.stop_ticker()
         self.start_ticker()
 
+    def start_timer_slowmo_playback(self, speed, fps):
+        self.stop_timer_slowmo_playback()
+        
+        frames = speed * fps
+        update_delay = abs(int(1000.0 / frames))
+        
+        self.slowmo_ticker = utilsgtk.GtkTicker(self._slowmo_plyback_ticker_event, update_delay, speed)
+        self.slowmo_ticker.start_ticker()
+    
+    def _slowmo_plyback_ticker_event(self, speed):
+        if speed > 0.0:
+            self.seek_delta_from_timer_slowmo(1)
+        else:
+            self.seek_delta_from_timer_slowmo(-1)
+
+    def stop_timer_slowmo_playback(self):
+        if self.slowmo_ticker != None:
+            self.stop_ticker()
+            self.slowmo_ticker.destroy_ticker()
+            self.slowmo_ticker = None
+
     def stop_playback(self):
         """
         Stops playback from current producer
         """
+        self.stop_timer_slowmo_playback()
+        
         self.loop_start = -1 # User possibly goes into marks looping but stops without Control key.
         self.loop_end = -1
         self.is_looping = False
@@ -198,6 +227,8 @@ class Player:
         updater.update_frame_displayers(self.producer.frame())
 
     def start_loop_playback(self, cut_frame, loop_half_length, track_length):
+        self.stop_timer_slowmo_playback()
+        
         self.loop_start = cut_frame - loop_half_length
         self.loop_end = cut_frame + loop_half_length
         if self.loop_start < 0:
@@ -211,6 +242,8 @@ class Player:
         self.start_ticker()
 
     def start_loop_playback_range(self, range_in, range_out):
+        self.stop_timer_slowmo_playback()
+        
         seq_len = self.producer.get_length()
         if range_in >= seq_len:
             return
@@ -230,6 +263,8 @@ class Player:
         """
         Stops playback from current producer
         """
+        self.stop_timer_slowmo_playback()
+        
         self.loop_start = -1
         self.loop_end = -1
         self.is_looping = False
@@ -244,16 +279,39 @@ class Player:
         return self.producer.frame()
     
     def seek_position_normalized(self, pos, length):
+        self.stop_timer_slowmo_playback()
+        
         frame_number = pos * length
         self.seek_frame(int(frame_number)) 
 
     def seek_delta(self, delta):
+        self.stop_timer_slowmo_playback()
+        
         # Get new frame
         frame = self.producer.frame() + delta
         # Seek frame
         self.seek_frame(frame)
-    
-    def seek_frame(self, frame, update_gui=True):
+
+    def seek_delta_from_timer_slowmo(self, delta):
+        # Get new frame
+        frame = self.producer.frame() + delta
+        # Seek frame
+        self.seek_frame(frame, True, False)
+        
+    def slowmo_seek_delta(self, delta):
+        self.stop_timer_slowmo_playback()
+        
+        now = time.time()
+        if (now - self.last_slowmo_seektime) > SLOWMO_FRAME_DELAY:
+            self.last_slowmo_seektime = now
+            frame = self.producer.frame() + delta
+            # Seek frame
+            self.seek_frame(frame)
+
+    def seek_frame(self, frame, update_gui=True, stop_slowmo_timer=True):
+        if stop_slowmo_timer == True:
+            self.stop_timer_slowmo_playback()
+        
         # Force range
         length = self.get_active_length()
         if frame < 0:
@@ -270,11 +328,15 @@ class Player:
             updater.update_frame_displayers(frame)
 
     def seek_end(self, update_gui=True):
+        self.stop_timer_slowmo_playback()
+        
         length = self.get_active_length()
         last_frame = length - 1
         self.seek_frame(last_frame, update_gui)
 
     def seek_and_get_rgb_frame(self, frame, update_gui=True):
+        self.stop_timer_slowmo_playback()
+        
         # Force range
         length = self.get_active_length()
         if frame < 0:
@@ -302,6 +364,9 @@ class Player:
             self.seek_frame(new_seq_len)
 
     def is_playing(self):
+        if self.slowmo_ticker != None:
+            return True
+
         return (self.producer.get_speed() != 0)
 
     def _ticker_event(self, unused_data):
@@ -316,7 +381,8 @@ class Player:
 
         # Stop ticker if playback has stopped.
         if (self.consumer.is_stopped() or self.producer.get_speed() == 0):
-            self.stop_ticker()
+            if self.slowmo_ticker == None:
+                self.stop_ticker()
 
         # If we're out of active range seek end.
         if current_frame >= self.get_active_length():
@@ -350,6 +416,7 @@ class Player:
 
     def shutdown(self):
         self.stop_ticker()
+        self.stop_timer_slowmo_playback()
         self.producer.set_speed(0)
         self.consumer.stop()
 
@@ -359,3 +426,10 @@ class Player:
     
     def stop_ticker(self):
         self.ticker.destroy_ticker()
+
+
+    
+    
+
+
+        
