@@ -22,10 +22,10 @@
 Module has methods that build panels from widgets. Created panels
 are used to build gui at callsites.
 """
-from gi.repository import Gtk, Gdk, Pango
+from gi.repository import Gtk, Pango
 
 import appconsts
-import editorpersistance # Aug-2019 - SvdB - BB
+import editorpersistance
 from editorstate import current_sequence
 import gui
 import guicomponents
@@ -50,7 +50,7 @@ def get_media_files_panel(media_list_view, add_cb, del_cb, col_changed_cb, hambu
     # Aug-2019 - SvdB - BB
     size_adj = 1
     prefs = editorpersistance.prefs
-    if prefs.double_track_hights:
+    if guiutils.double_icon_size():
         size_adj = 2
     hamburger_launcher = guicomponents.HamburgerPressLaunch(hamburger_launch_pressed)
     hamburger_launcher.do_popover_callback = True
