@@ -18,20 +18,22 @@
     along with Flowblade Movie Editor.  If not, see <http://www.gnu.org/licenses/>.
 """
 
+import multiprocessing
 import os
 
 from gi.repository import Gtk
 
 import appconsts
+import databridge
 import dialogutils
 import editorpersistance
 import gui
 import guiutils
+import gtkbuilder
 import mltprofiles
-import multiprocessing
-import utils
 import utilsgtk
 
+
 PREFERENCES_WIDTH = 730
 PREFERENCES_HEIGHT = 440
 PREFERENCES_LEFT = 410
@@ -39,7 +41,7 @@ PREFERENCES_LEFT = 410
 def preferences_dialog():
 
     dialog = Gtk.Dialog(_("Editor Preferences"), None,
-                    Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
+                    None,
                     (_("Cancel"), Gtk.ResponseType.REJECT,
                     _("OK"), Gtk.ResponseType.ACCEPT))
 
@@ -48,6 +50,7 @@ def preferences_dialog():
     playback_prefs_panel, playback_prefs_widgets  = _playback_prefs_panel()
     view_pres_panel, view_pref_widgets = _view_prefs_panel()
     performance_panel, performance_widgets = _performance_panel()
+    jog_shuttle_panel, jog_shuttle_widgets = _jog_shuttle_panel()
 
     notebook = Gtk.Notebook()
     notebook.set_size_request(PREFERENCES_WIDTH, PREFERENCES_HEIGHT)
@@ -56,11 +59,13 @@ def preferences_dialog():
     notebook.append_page(playback_prefs_panel, Gtk.Label(label=_("Playback")))
     notebook.append_page(view_pres_panel, Gtk.Label(label=_("View")))
     notebook.append_page(performance_panel, Gtk.Label(label=_("Performance")))
+    notebook.append_page(jog_shuttle_panel, Gtk.Label(label=_("Jog/Shuttle")))
     guiutils.set_margins(notebook, 4, 24, 6, 0)
 
     dialog.connect('response', _preferences_dialog_callback, 
                     (gen_opts_widgets, edit_prefs_widgets, playback_prefs_widgets, 
-                    view_pref_widgets, performance_widgets))
+                    view_pref_widgets, performance_widgets, jog_shuttle_widgets))
+
     dialog.vbox.pack_start(notebook, True, True, 0)
     dialogutils.set_outer_margins(dialog.vbox)
     dialogutils.default_behaviour(dialog)
@@ -118,15 +123,15 @@ def _general_options_panel():
     load_order_combo.append_text(_("Absolute paths only"))
     load_order_combo.set_active(prefs.media_load_order)
 
-    render_folder_select = Gtk.FileChooserButton.new (_("Select Default Render Folder"), Gtk.FileChooserAction.SELECT_FOLDER)
+    render_folder_select = gtkbuilder.get_file_chooser_button(_("Select Default Render Folder"), Gtk.FileChooserAction.SELECT_FOLDER)
     if prefs.default_render_directory == None or prefs.default_render_directory == appconsts.USER_HOME_DIR \
         or (not os.path.exists(prefs.default_render_directory)) \
         or (not os.path.isdir(prefs.default_render_directory)):
-        render_folder_select.set_current_folder_uri(os.path.expanduser("~") + "/")
+        render_folder_select.set_current_folder(os.path.expanduser("~") + "/")
     else:
-        render_folder_select.set_current_folder_uri(prefs.default_render_directory)
+        render_folder_select.set_current_folder(prefs.default_render_directory)
 
-    disk_cache_warning_combo  = Gtk.ComboBoxText()
+    disk_cache_warning_combo = Gtk.ComboBoxText()
     disk_cache_warning_combo.append_text(_("Off"))
     disk_cache_warning_combo.append_text(_("500 MB"))
     disk_cache_warning_combo.append_text(_("1 GB"))
@@ -270,14 +275,6 @@ def _playback_prefs_panel():
     ffwd_rev_shift_spin.set_adjustment(spin_adj)
     ffwd_rev_shift_spin.set_numeric(True)
 
-    if hasattr(prefs, 'ffwd_rev_ctrl'):
-        spin_adj = Gtk.Adjustment(value=prefs.ffwd_rev_ctrl, lower=1, upper=10, step_increment=1)
-    else:
-        spin_adj = Gtk.Adjustment(value=10, lower=1, upper=10, step_increment=1)
-    ffwd_rev_ctrl_spin = Gtk.SpinButton()
-    ffwd_rev_ctrl_spin.set_adjustment(spin_adj)
-    ffwd_rev_ctrl_spin.set_numeric(True)
-
     if hasattr(prefs, 'ffwd_rev_caps'):
         spin_adj = Gtk.Adjustment(value=prefs.ffwd_rev_caps, lower=1, upper=10, step_increment=1)
     else:
@@ -299,8 +296,6 @@ def _playback_prefs_panel():
         "E.g. if Shift is set to " + str(prefs.ffwd_rev_shift) + " and Ctrl to " + str(prefs.ffwd_rev_ctrl) + \
         ", holding Shift + Ctrl will result in up to " + str(prefs.ffwd_rev_shift * prefs.ffwd_rev_ctrl) + "x speed.\n" \
         "(Effective maximum speed depends on underlying software and/or hardware limitations)"))
-    row15 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Fast Forward / Reverse Speed for Control Key:")), ffwd_rev_ctrl_spin, PREFERENCES_LEFT))
-    row15.set_tooltip_text(_("Speed of Forward / Reverse will be multiplied by this value if Ctrl Key is held (Only using KEYS)."))
     row16 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Fast Forward / Reverse Speed for Caps Lock Key:")), ffwd_rev_caps_spin, PREFERENCES_LEFT))
     row16.set_tooltip_text(_("Speed of Forward / Reverse will be multiplied by this value if Caps Lock is set (Only using KEYS)."))
     row17 = _row(guiutils.get_checkbox_row_box(follow_move_range, Gtk.Label(label=_("Move Timeline to follow Playback"))))
@@ -314,7 +309,6 @@ def _playback_prefs_panel():
     vbox.pack_start(row10, False, False, 0)
     vbox.pack_start(row11, False, False, 0)
     vbox.pack_start(row14, False, False, 0)
-    vbox.pack_start(row15, False, False, 0)
     vbox.pack_start(row16, False, False, 0)
 
     vbox.pack_start(Gtk.Label(), True, True, 0)
@@ -323,7 +317,7 @@ def _playback_prefs_panel():
 
     return vbox, (auto_center_on_stop,
                   play_pause_button, timeline_start_end_button, auto_center_on_updown,
-                  ffwd_rev_shift_spin, ffwd_rev_ctrl_spin, ffwd_rev_caps_spin, follow_move_range, loop_clips)
+                  ffwd_rev_shift_spin, ffwd_rev_caps_spin, follow_move_range, loop_clips)
 
 def _view_prefs_panel():
     prefs = editorpersistance.prefs
@@ -368,16 +362,15 @@ def _view_prefs_panel():
         window_mode_combo.set_active(1)
 
     tracks_combo = Gtk.ComboBoxText()
-    tracks_combo.append_text(_("Normal - 50px, 25px"))
-    tracks_combo.append_text(_("Double for HiDPI - 100px, 50px"))
-    # Aug-2019 - SvdB - BB
-    tracks_combo.set_active(prefs.double_track_hights)
+    tracks_combo.append_text(_("Default - 50px, 25px"))
+    tracks_combo.append_text(_("1.5 x - 75px, 37px"))
+    tracks_combo.append_text(_("2 x - 100px, 50px"))
+    tracks_combo.set_active(prefs.tracks_scale)
 
-    # DEPRECATED, remove.
-    top_row_layout = Gtk.ComboBoxText()
-    top_row_layout.append_text(_("3 panels if width (1450px+) available"))
-    top_row_layout.append_text(_("2 panels always"))
-    top_row_layout.set_active(prefs.top_row_layout)
+    icons_combo = Gtk.ComboBoxText()
+    icons_combo.append_text(_("Default"))
+    icons_combo.append_text(_("Double"))
+    icons_combo.set_active(prefs.icons_scale)
 
     monitors_data = utilsgtk.get_display_monitors_size_data()
     layout_monitor = Gtk.ComboBoxText()
@@ -391,28 +384,58 @@ def _view_prefs_panel():
             layout_monitor.append_text(_("Monitor ") + str(monitor_index) + ": " + str(monitor_w) + " x " + str(monitor_h))
     layout_monitor.set_active(prefs.layout_display_index)
 
+    spin_adj = Gtk.Adjustment(value=prefs.filter_select_width, lower=editorpersistance.FILTER_SELECT_WIDTH_MIN, upper=editorpersistance.FILTER_SELECT_WIDTH_MAX, step_increment=1)
+    filter_select_width_spin = Gtk.SpinButton.new_with_range(editorpersistance.FILTER_SELECT_WIDTH_MIN, editorpersistance.FILTER_SELECT_WIDTH_MAX, 1)
+    filter_select_width_spin.set_adjustment(spin_adj)
+    filter_select_width_spin.set_numeric(True)
+
+    spin_adj = Gtk.Adjustment(value=prefs.project_panel_width, lower=editorpersistance.PROJECT_PANEL_WIDTH_MIN, upper=editorpersistance.PROJECT_PANEL_WIDTH_MAX, step_increment=1)
+    project_panel_width_spin = Gtk.SpinButton.new_with_range(editorpersistance.PROJECT_PANEL_WIDTH_MIN, editorpersistance.PROJECT_PANEL_WIDTH_MAX, 1)
+    project_panel_width_spin.set_adjustment(spin_adj)
+    project_panel_width_spin.set_numeric(True)
+
+    spin_adj = Gtk.Adjustment(value=prefs.editor_panel_width, lower=editorpersistance.EDIT_PANEL_WIDTH_MIN, upper=editorpersistance.EDIT_PANEL_WIDTH_MAX, step_increment=1)
+    edit_panel_width_spin = Gtk.SpinButton.new_with_range(editorpersistance.EDIT_PANEL_WIDTH_MIN, editorpersistance.EDIT_PANEL_WIDTH_MAX, 1)
+    edit_panel_width_spin.set_adjustment(spin_adj)
+    edit_panel_width_spin.set_numeric(True)
+
+    spin_adj = Gtk.Adjustment(value=prefs.media_panel_width, lower=editorpersistance.MEDIA_PANEL_WIDTH_MIN, upper=editorpersistance.MEDIA_PANEL_WIDTH_MAX, step_increment=1)
+    media_panel_width_spin = Gtk.SpinButton.new_with_range(editorpersistance.MEDIA_PANEL_WIDTH_MIN, editorpersistance.MEDIA_PANEL_WIDTH_MAX, 1)
+    media_panel_width_spin.set_adjustment(spin_adj)
+    media_panel_width_spin.set_numeric(True)
+    
     row00 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Application window mode:")), window_mode_combo, PREFERENCES_LEFT))
     row9 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Force Language:")), force_language_combo, PREFERENCES_LEFT))
     row1 = _row(guiutils.get_checkbox_row_box(display_splash_check, Gtk.Label(label=_("Display splash screen"))))
+    row6 = _row(guiutils.get_checkbox_row_box(show_full_file_names, Gtk.Label(label=_("Show Full File names"))))
     row7 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Tracks Heights:")), tracks_combo, PREFERENCES_LEFT))
-    # Feb-2017 - SvdB - For full file names
-    row6 =  _row(guiutils.get_checkbox_row_box(show_full_file_names, Gtk.Label(label=_("Show Full File names"))))
+    row8 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Icons Size:")), icons_combo, PREFERENCES_LEFT))
+
     row10 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Do GUI layout based on:")), layout_monitor, PREFERENCES_LEFT))
+    row13 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Edit panel width:")), edit_panel_width_spin, PREFERENCES_LEFT))
+    row12 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Project panel width:")), project_panel_width_spin, PREFERENCES_LEFT))
+    row11 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Filter Select panel width:")), filter_select_width_spin, PREFERENCES_LEFT))
+    row14 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Media panel width:")), media_panel_width_spin, PREFERENCES_LEFT))
     
     vbox = Gtk.VBox(False, 2)
     vbox.pack_start(row00, False, False, 0)
     vbox.pack_start(row10, False, False, 0)
     vbox.pack_start(row9, False, False, 0)
     vbox.pack_start(row1, False, False, 0)
-    vbox.pack_start(row7, False, False, 0)
-    # Feb-2017 - SvdB - For full file names
     vbox.pack_start(row6, False, False, 0)
+    vbox.pack_start(row7, False, False, 0)
+    vbox.pack_start(row8, False, False, 0)
+    vbox.pack_start(row14, False, False, 0)
+    vbox.pack_start(row13, False, False, 0)
+    vbox.pack_start(row12, False, False, 0)
+    vbox.pack_start(row11, False, False, 0)
     vbox.pack_start(Gtk.Label(), True, True, 0)
     
     guiutils.set_margins(vbox, 12, 0, 12, 12)
 
     return vbox, (force_language_combo, display_splash_check, window_mode_combo, show_full_file_names,
-                  tracks_combo, top_row_layout, layout_monitor)
+                  tracks_combo, icons_combo, project_panel_width_spin, edit_panel_width_spin, media_panel_width_spin,
+                  layout_monitor, filter_select_width_spin)
 
 
 
@@ -457,6 +480,49 @@ def _performance_panel():
 
     return vbox, (perf_render_threads, perf_drop_frames)
 
+def _jog_shuttle_panel():
+    prefs = editorpersistance.prefs
+
+    # Widgets
+
+    # usbhid_enabled
+    usbhid_enabled_check = Gtk.CheckButton()
+    usbhid_enabled_check.set_active(prefs.usbhid_enabled)
+
+    # usbhid_config
+    # stored as a base name equal to what is in usbhid_config_metadata.device_config_name
+    # we generate a combo select box with "None" as index 0, and everything else starting
+    # from index 1.
+    usbhid_config = prefs.usbhid_config
+    usbhid_config_combo_index = 0
+    usbhid_config_combo_selected_index = 0
+
+    usbhid_config_combo = Gtk.ComboBoxText()
+    usbhid_config_combo.append_text("None")
+    for usbhid_config_metadata in databridge.usbhid_get_usb_hid_device_config_metadata_list():
+        usbhid_config_combo_index += 1
+        usbhid_config_combo.append_text(usbhid_config_metadata.name)
+
+        # if this is the config that was already selected in the prefs,
+        # highlight it as the selected option in the combo box
+        if usbhid_config is not None:
+            if usbhid_config == usbhid_config_metadata.device_config_name:
+                usbhid_config_combo_selected_index = usbhid_config_combo_index
+
+    usbhid_config_combo.set_active(usbhid_config_combo_selected_index)
+
+    # Layout
+    row1 = _row(guiutils.get_checkbox_row_box(usbhid_enabled_check, Gtk.Label(label=_("USB Jog/Shuttle Enabled"))))
+    row2 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Device")), usbhid_config_combo, PREFERENCES_LEFT))
+
+    vbox = Gtk.VBox(False, 2)
+    vbox.pack_start(row1, False, False, 0)
+    vbox.pack_start(row2, False, False, 0)
+
+    guiutils.set_margins(vbox, 12, 0, 12, 12)
+
+    return vbox, (usbhid_enabled_check, usbhid_config_combo)
+
 def _row(row_cont):
     row_cont.set_size_request(10, 26)
     return row_cont
