@@ -24,7 +24,6 @@ Load, save, add media file, etc...
 """
 
 import datetime
-import glob
 import hashlib
 try:
     import mlt7 as mlt
@@ -36,6 +35,7 @@ from os.path import isfile, join, expanduser
 from PIL import Image
 import re
 import shutil
+import subprocess
 import time
 import threading
 
@@ -44,13 +44,14 @@ from gi.repository import Gtk
 from gi.repository import Gdk
 from gi.repository import GLib
 
-import app
 import audiowaveformrenderer
 import appconsts
 import batchrendering
+import callbackbridge
 import clipeffectseditor
 import compositeeditor
 import containerclip
+import databridge
 import dialogs
 import dialogutils
 import gui
@@ -80,6 +81,7 @@ import projectinfogui
 import projectmediaimport
 import propertyparse
 import projectdatavault
+import projectdatavaultgui
 import proxyediting
 import render
 import renderconsumer
@@ -104,6 +106,7 @@ force_proxy = False
 # This is needed to pass only one event for double click, double init for monitor click possibly somewhat unstable
 _media_panel_double_click_counter = 0
 
+_popover_media_file = None
 
 #--------------------------------------- worker threads
 class LoadThread(threading.Thread):
@@ -171,7 +174,7 @@ class LoadThread(threading.Thread):
 
     def _do_project_open(self, project):
         
-        app.open_project(project) # <-- HERE
+        callbackbridge.app_open_project(project) # <-- HERE
 
         if self.block_recent_files: # naming flipped ????
             editorpersistance.add_recent_project_path(self.filename)
@@ -193,7 +196,7 @@ class LoadThread(threading.Thread):
                 editorstate.project.name = os.path.basename(self.filename)
                 _save_project_in_last_saved_path()
         else:
-            print("autosave load")
+            pass
 
         self.dialog.destroy()
         gui.tline_canvas.connect_mouse_events() # mouse events during load cause crashes because there is no data to handle
@@ -241,7 +244,7 @@ class LoadThread(threading.Thread):
                                                 panels)
 
         # We exit to same state as when app is first opened.
-        app.open_project(projectdata.get_default_project())
+        callbackbridge.app_open_project(projectdata.get_default_project())
 
     def _exit_on_profile_file_not_found_error(self, e, ticker, old_project):
         self._error_stop(self.dialog, ticker)
@@ -380,7 +383,39 @@ class AddMediaFilesThread(threading.Thread):
     def _not_valid_producer(self, err):
         dialogs.not_valid_producer_dialog(err, gui.editor_window.window)
 
-                    
+
+
+class AddTitleItemThread(threading.Thread):
+    
+    def __init__(self, name, filepath, title_data, completed_callback):
+        threading.Thread.__init__(self)
+        self.name = name
+        self.filepath = filepath
+        self.title_data = title_data
+        self.completed_callback = completed_callback
+
+    def run(self): 
+        target_bin = PROJECT().c_bin
+        PROJECT().add_title_item(self.name, self.filepath, self.title_data, target_bin)
+
+        self.list_view_update_done = False
+        GLib.idle_add(self._list_view_update)
+        while self.list_view_update_done == False:
+            time.sleep(0.05)
+
+        project_event = projectdata.ProjectEvent(projectdata.EVENT_MEDIA_ADDED, str(1))
+        PROJECT().events.append(project_event)
+
+        self.completed_callback()
+
+    def _list_view_update(self):
+        gui.media_list_view.fill_data_model()
+        max_val = gui.editor_window.media_scroll_window.get_vadjustment().get_upper()
+        gui.editor_window.media_scroll_window.get_vadjustment().set_value(max_val)
+        self.list_view_update_done = True
+
+
+
 class UpdateMediaLengthsThread(threading.Thread):
     
     def __init__(self, dialog):
@@ -479,7 +514,7 @@ def _new_project_dialog_callback(dialog, response_id, profile_combo, tracks_sele
 
         dialog.destroy()
                 
-        app.new_project(profile_index, v_tracks, a_tracks)
+        callbackbridge.app_new_project(profile_index, v_tracks, a_tracks)
 
         project_event = projectdata.ProjectEvent(projectdata.EVENT_CREATED_BY_NEW_DIALOG, None)
         PROJECT().events.append(project_event)
@@ -503,7 +538,7 @@ def close_project():
     if was_edited_since_last_save() == False:
         _close_dialog_callback(None, None, True)
     else:
-        dialogs.close_confirm_dialog(_close_dialog_callback, app.get_save_time_msg(), gui.editor_window.window, editorstate.PROJECT().name)
+        dialogs.close_confirm_dialog(_close_dialog_callback, databridge.app_get_save_time_msg(), gui.editor_window.window, editorstate.PROJECT().name)
 
 def _close_dialog_callback(dialog, response_id, no_dialog_project_close=False):
     if no_dialog_project_close == False:
@@ -529,8 +564,11 @@ def _close_dialog_callback(dialog, response_id, no_dialog_project_close=False):
     sequence.VIDEO_TRACKS_COUNT = appconsts.INIT_V_TRACKS
 
     new_project = projectdata.get_default_project()
-    app.open_project(new_project)
-    
+    vault_folder = projectdatavault.get_active_vault_folder()
+    new_project.create_vault_folder_data(vault_folder)
+    callbackbridge.app_open_project(new_project)
+    projectdatavault.create_project_data_folders()
+
 def actually_load_project(filename, block_recent_files=False, is_first_video_load=False, is_autosave_load=False, replace_media_file_path=None):
     gui.tline_canvas.disconnect_mouse_events() # mouse events dutring load cause crashes because there is no data to handle
     updater.set_info_icon("document-open")
@@ -618,8 +656,8 @@ def _save_as_dialog_callback(dialog, response_id):
             p_event = projectdata.ProjectEvent(projectdata.EVENT_SAVED_AS, (PROJECT().name, PROJECT().last_save_path))
             PROJECT().events.append(p_event)
             
-        app.stop_autosave()
-        app.start_autosave()
+        callbackbridge.app_stop_autosave()
+        callbackbridge.app_start_autosave()
         
         global save_icon_remove_event_id
         save_icon_remove_event_id = GLib.timeout_add(500, remove_save_icon)
@@ -767,8 +805,75 @@ def was_edited_since_last_save():
     
     return True
 
-def view_project_events():
-    projectinfogui.show_project_events_dialog()
+def show_project_info():
+
+    data_folder_path = projectdatavault.get_project_data_folder()
+    folder_handle = projectdatavault.ProjectDataFolderHandle(data_folder_path)
+    savefile_path, len_savefiles, date_time = folder_handle.get_save_info()
+
+    profile = PROJECT().profile
+    info_box = guicomponents.get_profile_info_box(profile)
+    
+    pvbox = Gtk.VBox()
+    pvbox.pack_start(info_box, False, True, 0)
+    profile_panel = guiutils.get_named_frame(_("Profile"), pvbox, 4)
+    profile_panel.set_margin_bottom(12)
+
+    creation_label = guiutils.bold_label(_("Created:"))
+    creation_label.set_margin_end(4)
+    try:
+        data_file = PROJECT().vault_folder + "/" + PROJECT().project_data_id
+        completed_process = subprocess.run(["stat", "-c", "'%w'", data_file], capture_output=True, check=True, text=True)
+        creation_date = completed_process.stdout
+        creation_date = creation_date.strip("'").split(".")[0]
+    except:
+        creation_date = "N/A"
+    creation_row = guiutils.get_left_justified_box([creation_label, Gtk.Label(label=creation_date)])
+
+    save_name_label = guiutils.bold_label(_("Last Saved File:"))
+    save_name_label.set_margin_end(4)
+    try:
+        save_file_name = Gtk.Label(label=str(os.path.basename(savefile_path)))
+    except:
+        save_file_name = Gtk.Label(label=_("Not Saved"))
+    save_name_row = guiutils.get_left_justified_box([save_name_label, save_file_name])
+
+    save_label = guiutils.bold_label(_("Last Saved:"))
+    save_label.set_margin_end(4)
+    if str(date_time) != "0":
+        save_date = Gtk.Label(label=str(date_time).split(".")[0])
+    else:
+        save_date = Gtk.Label(label=_("Not Saved"))
+    save_date_row = guiutils.get_left_justified_box([save_label, save_date])
+
+    save_times_label = guiutils.bold_label(_("Saved Times:"))
+    save_times_label.set_margin_end(4)
+    save_times_row = guiutils.get_left_justified_box([save_times_label, Gtk.Label(label=str(len_savefiles))])
+    save_times_row.set_margin_bottom(12)
+
+    media_files_label = guiutils.bold_label(_("Media Items:"))
+    media_files_label.set_margin_end(4)
+    media_files_row = guiutils.get_left_justified_box([media_files_label, Gtk.Label(label=str(len(PROJECT().media_files)))])
+
+    bins_label = guiutils.bold_label(_("Bins:"))
+    bins_label.set_margin_end(4)
+    bins_row = guiutils.get_left_justified_box([bins_label, Gtk.Label(label=str(len(PROJECT().bins)))])
+
+    sequences_label = guiutils.bold_label(_("Sequences:"))
+    sequences_label.set_margin_end(4)
+    sequences_row = guiutils.get_left_justified_box([sequences_label, Gtk.Label(label=str(len(PROJECT().bins)))])
+    
+    vbox = Gtk.VBox(False, 2)
+    vbox.pack_start(save_name_row, False, False, 0)
+    vbox.pack_start(creation_row, False, False, 0)
+    vbox.pack_start(save_date_row, False, False, 0)
+    vbox.pack_start(save_times_row, False, False, 0)
+    vbox.pack_start(profile_panel, False, False, 0)
+    vbox.pack_start(media_files_row, False, False, 0)
+    vbox.pack_start(bins_row, False, False, 0)
+    vbox.pack_start(sequences_row, False, False, 0)
+
+    projectdatavaultgui.show_current_project_data_store_info_window(vbox)
 
 # ---------------------------------- rendering
 def do_rendering():
@@ -917,6 +1022,15 @@ def _hamburger_menu_item_selected(action, variant, msg):
         append_all_media_clips_into_timeline()
     elif msg == "append selected":
         append_selected_media_clips_into_timeline()
+    elif msg == "reverse order":
+        reverse_current_bin_media_order()
+    elif msg == "move to clicked":
+        gui.media_list_view.init_move()
+    elif msg == "set bin default":
+        current_default_length = PROJECT().get_current_bin_graphics_default_length()
+        dialogs.set_bin_grfx_default_length_dialog( current_bin(),
+                                                    current_default_length,
+                                                    _set_bin_grfx_default_length_callback)
     else:
         target_bin_index = int(msg)
         
@@ -1204,7 +1318,31 @@ def open_next_media_item_in_monitor():
     gui.media_list_view.update_selected_bg_colors()
     updater.set_and_display_monitor_media_file(media_file)
     gui.pos_bar.widget.grab_focus()
-            
+
+def open_prev_media_item_in_monitor():
+    # Get id for prev media file
+    selection = gui.media_list_view.get_selected_media_objects()
+    if len(selection) < 1:
+        try: # Nothing selected, get first media item
+            next_media_file_id = current_bin().file_ids[0]
+        except:
+            return # bin is empty
+    else: # Get prev media item from selection
+        current_media_file_id = selection[0].media_file.id
+        prev_media_index = current_bin().file_ids.index(current_media_file_id) - 1
+        if prev_media_index == -1:
+            prev_media_index = 0
+
+        prev_media_file_id = current_bin().file_ids[prev_media_index]
+
+    # Get media file, select it and show in monitor
+    media_file = PROJECT().media_files[prev_media_file_id]
+
+    gui.media_list_view.select_media_file(media_file)
+    gui.media_list_view.update_selected_bg_colors()
+    updater.set_and_display_monitor_media_file(media_file)
+    gui.pos_bar.widget.grab_focus()
+
 def display_media_file_rename_dialog(media_file):
     dialogs.new_media_name_dialog(media_file_name_edited, media_file)
 
@@ -1278,38 +1416,22 @@ def _display_file_info(media_file):
     dialogs.file_properties_dialog((media_file, img, size, length, vcodec, acodec, channels, frequency, fps, match_profile_name, matches_project_profile))
 
 def remove_unused_media():
-    unused = unused_media()
+    unused = PROJECT().get_unused_media()
     # It is most convenient to do remove via gui object
     gui.media_list_view.select_media_file_list(unused)
     delete_media_files()
 
-def unused_media():
-    # Create path -> media item dict
-    path_to_media_object = {}
-    for key, media_item in list(PROJECT().media_files.items()):
-        if hasattr(media_item, "path") and media_item.path != "" and media_item.path != None:
-            path_to_media_object[media_item.path] = media_item
-    
-    # Remove all items from created dict that have a clip with same path on any of the sequences
-    for seq in PROJECT().sequences:
-        for track in seq.tracks:
-            for clip in track.clips:
-                try:
-                    path_to_media_object.pop(clip.path)
-                except:
-                    pass
-    
-    # Create a list of unused media objects
-    unused = []
-    for path, media_item in list(path_to_media_object.items()):
-        unused.append(media_item)
-    return unused
-
 def media_filtering_select_pressed(launcher, widget, event):
     guipopover.file_filter_popover_show(launcher, widget,  _media_filtering_selector_item_activated)
 
 def _media_filtering_selector_item_activated(action, new_value_variant):
-    index = int( new_value_variant.get_string())
+    try:
+        index = int( new_value_variant.get_string())
+    except:
+        # This is ratings selection
+        _ratings_filtering_item_activated(action, new_value_variant)
+        return
+
     gui.media_view_filter_selector.set_pixbuf(index)
     
     # Const values at appconst,py correspond with indexes here.
@@ -1318,7 +1440,21 @@ def _media_filtering_selector_item_activated(action, new_value_variant):
 
     action.set_state(new_value_variant)
     guipopover._file_filter_popover.hide()
-    
+
+def _ratings_filtering_item_activated(action, new_value_variant):
+    if new_value_variant.get_string() == "hide_bad":
+        editorstate.media_view_ratings_filter = appconsts.MEDIA_RATINGS_HIDE_BAD
+    elif new_value_variant.get_string() == "show_favorites":
+        editorstate.media_view_ratings_filter = appconsts.MEDIA_RATINGS_SHOW_FAVORITES
+    else:
+        editorstate.media_view_ratings_filter = appconsts.MEDIA_RATINGS_SHOW_ALL
+
+    gui.media_list_view.fill_data_model()
+
+    action.set_state(new_value_variant)
+    guipopover._file_filter_popover.hide()
+    gui.editor_window.bin_info.display_bin_info() 
+
 def columns_count_launch_pressed(launcher, widget, event):
     guipopover.columns_count_popupover_show(launcher, widget, _columns_count_item_selected)
     
@@ -1537,7 +1673,9 @@ def _append_media_files(media_files):
         new_clip.clip_in = 0
         new_clip.clip_out = new_clip.get_length() - 1
         if new_clip.media_type == appconsts.IMAGE:
-            in_fr, out_fr, default_grfx_length = editorpersistance.get_graphics_default_in_out_length()
+            default_grfx_length = PROJECT().get_current_bin_graphics_default_length()
+            in_fr = (new_clip.get_length() - 1) // 2 - (default_grfx_length // 2)
+            out_fr = in_fr + default_grfx_length - 1
             new_clip.clip_in = in_fr
             new_clip.clip_out = out_fr
 
@@ -1696,7 +1834,18 @@ def move_files_to_bin(new_bin, moved_files):
     
     gui.editor_window.bin_info.display_bin_info()
     
-    
+def reverse_current_bin_media_order():
+    current_bin().file_ids.reverse()
+    gui.media_list_view.fill_data_model()
+
+
+def _set_bin_grfx_default_length_callback(dialog, response_id, value_spin):
+    if response_id == Gtk.ResponseType.ACCEPT:
+        new_default_len = value_spin.get_value()
+        PROJECT().set_current_bin_graphics_default_length(int(new_default_len))
+
+    dialog.destroy()
+
 # ------------------------------------ sequences
 def change_edit_sequence():
     selection = gui.sequence_list_view.treeview.get_selection()
@@ -1712,7 +1861,7 @@ def change_edit_sequence():
     # therefore is not saved.
     movemodes.clear_selected_clips()
     
-    app.change_current_sequence(row)
+    callbackbridge.app_change_current_sequence(row)
 
 def sequences_hamburger_pressed(widget, event):
     sequence_panel_popup_requested(widget, event)
@@ -1762,7 +1911,6 @@ def _add_new_sequence_dialog_callback(dialog, response_id, widgets):
     sequence.VIDEO_TRACKS_COUNT = v_tracks
 
     # Add new sequence.
-    print("new seq", name)
     PROJECT().add_named_sequence(name)
 
     gui.sequence_list_view.fill_data_model()
@@ -1770,7 +1918,7 @@ def _add_new_sequence_dialog_callback(dialog, response_id, widgets):
     if open_right_away == False:
         selection.select_path(str(row)) # Keep previous selection
     else:
-        app.change_current_sequence(len(PROJECT().sequences) - 1)
+        callbackbridge.app_change_current_sequence(len(PROJECT().sequences) - 1)
         gui.editor_window.init_compositing_mode_menu()
     
     dialog.destroy()
@@ -1828,7 +1976,7 @@ def _delete_confirm_callback(dialog, response_id):
     
     # If we deleted current sequence, open first sequence
     if row == current_index:
-        app.change_current_sequence(0)
+        callbackbridge.app_change_current_sequence(0)
     
     _enable_save()
 
@@ -1915,8 +2063,6 @@ def _add_delete_track_dialog_callback(dialog, response_id, v_tracks, a_tracks):
         dialog.destroy()
         return
 
-    print(v_tracks)
-
     dialog.destroy()
     _do_tracks_count_change(v_tracks, a_tracks)
 
@@ -1934,7 +2080,7 @@ def _do_tracks_count_change(v_tracks, a_tracks):
 
     PROJECT().sequences.insert(cur_seq_index, new_seq)
     PROJECT().sequences.pop(cur_seq_index + 1)
-    app.change_current_sequence(cur_seq_index)
+    callbackbridge.app_change_current_sequence(cur_seq_index)
 
     if current_sequence().compositing_mode == appconsts.COMPOSITING_MODE_STANDARD_FULL_TRACK:
         # Put track compositors back
@@ -2160,10 +2306,14 @@ def do_compositing_mode_change(new_compositing_mode):
 
 # --------------------------------------------------------- pop-up menus
 def media_file_popover_mouse_right_pressed(widget, media_file, event):
-    guipopover.media_file_popover_show(media_file, widget, event.x, event.y, media_file_menu_item_selected)
+    global _popover_media_file # we need this for media_file_menu_rating_item_selected()
+    _popover_media_file = media_file 
+    guipopover.media_file_popover_show(media_file, widget, event.x, event.y, media_file_menu_item_selected, media_file_menu_rating_item_selected)
     
 def media_file_menu_item_selected(action, variant, data):
-    item_id, media_file = data
+    item_id, unused = data
+    global _popover_media_file
+    media_file = _popover_media_file
     if item_id == "File Properties":
         _display_file_info(media_file)
     if item_id == "Open in Clip Monitor":
@@ -2186,7 +2336,21 @@ def media_file_menu_item_selected(action, variant, data):
         media_file.create_icon()
     if item_id == "Replace":
         replace_media_file(media_file)
-        
+
+def media_file_menu_rating_item_selected(action, variant):
+    global _popover_media_file
+    if variant.get_string() == "unrated":
+        _popover_media_file.rating = appconsts.MEDIA_FILE_UNRATED
+    elif variant.get_string() == "favorite":
+        _popover_media_file.rating = appconsts.MEDIA_FILE_FAVORITE
+    else:
+        _popover_media_file.rating = appconsts.MEDIA_FILE_BAD
+
+    action.set_state(variant)
+    editorpersistance.save()
+    guipopover._media_file_popover.hide()
+    gui.media_list_view.widget.queue_draw()
+
 def _select_treeview_on_pos_and_return_row_and_column_title(event, treeview):
     selection = treeview.get_selection()
     path_pos_tuple = treeview.get_path_at_pos(int(event.x), int(event.y))
