@@ -21,6 +21,7 @@
 """
 Module contains objects used to capture project data.
 """
+
 import cairo
 import datetime
 try:
@@ -84,7 +85,8 @@ class Project:
         self.profile = profile
         self.profile_desc = profile.description()
         self.bins = []
-        self.media_files = {} # MediaFile.id(key) -> MediaFile object(value)
+        self.bins_graphics_default_lengths = {} # Bin.uid -> default length
+        self.media_files = {} # MediaFile.id(key) -> MediaFile object.
         self.sequences = []
         self.next_media_file_id = 0 
         self.next_bin_number = 1 # This is for creating name for new bin 
@@ -97,6 +99,7 @@ class Project:
         self.update_media_lengths_on_load = False # old projects < 1.10 had wrong media length data which just was never used.
                                                   # 1.10 needed that data for the first time and required recreating it correctly for older projects
         self.project_properties = {} # Key value pair for misc persistent properties, dict is used that we can add these without worrying loading
+        self.tracking_data = {}
 
         self.vault_folder = None
         self.project_data_id = None
@@ -176,6 +179,26 @@ class Project:
         
         return media_object
 
+    def add_title_item(self, name, file_path, title_data, target_bin):
+        (directory, file_name) = os.path.split(file_path)
+        (fn, ext) = os.path.splitext(file_name)
+
+        # Get media type
+        media_type = appconsts.IMAGE
+
+        # Get length and icon
+        (icon_path, length, info) = thumbnailer.write_image(file_path)
+        
+        clip_name = name
+
+        # Create media file object
+        media_object = MediaFile(self.next_media_file_id, file_path, 
+                                 clip_name, media_type, length, icon_path, info)
+        media_object.ttl = None
+        media_object.titler_data = title_data
+
+        self._add_media_object(media_object, target_bin)
+
     def add_pattern_producer_media_object(self, media_object):
         self._add_media_object(media_object)
         
@@ -249,6 +272,29 @@ class Project:
 
         self.c_bin.file_ids.pop(media_file.id)
 
+
+    def get_unused_media(self):
+        # Create path -> media item dict
+        path_to_media_object = {}
+        for key, media_item in list(self.media_files.items()):
+            if hasattr(media_item, "path") and media_item.path != "" and media_item.path != None:
+                path_to_media_object[media_item.path] = media_item
+        
+        # Remove all items from created dict that have a clip with same path on any of the sequences
+        for seq in self.sequences:
+            for track in seq.tracks:
+                for clip in track.clips:
+                    try:
+                        path_to_media_object.pop(clip.path)
+                    except:
+                        pass
+    
+        # Create a list of unused media objects
+        unused = []
+        for path, media_item in list(path_to_media_object.items()):
+            unused.append(media_item)
+        return unused
+    
     def get_current_proxy_paths(self):
         paths_dict = {}
         for idkey, media_file in list(self.media_files.items()):
@@ -276,7 +322,16 @@ class Project:
         """
         name = _("sequence_") + str(self.next_seq_number)
         self.add_named_sequence(name)
-        
+
+    def get_current_bin_graphics_default_length(self):
+        try:
+            return self.bins_graphics_default_lengths[self.c_bin.uid]
+        except:
+            return editorpersistance.prefs.default_grfx_length
+
+    def set_current_bin_graphics_default_length(self, default_length):
+        self.bins_graphics_default_lengths[self.c_bin.uid] = default_length
+            
     def add_named_sequence(self, name):
         seq = sequence.Sequence(self.profile, appconsts.COMPOSITING_MODE_STANDARD_FULL_TRACK)
         seq.create_default_tracks()
@@ -366,7 +421,26 @@ class Project:
     def set_project_property(self, property_name, value):
         self.project_properties[property_name] = value
 
-            
+    def add_tracking_data(self, data_label, data_file_path):
+        data_uid = utils.get_uid_str()
+        
+        # Get unique data label
+        add_number = 1
+        final_label = data_label
+        while self._tracking_data_label_exists(final_label):
+            final_label = data_label + "-" + str(add_number)
+            add_number += 1
+        
+        self.tracking_data[data_uid] = (final_label, data_file_path)
+        return final_label
+
+    def _tracking_data_label_exists(self, data_label):
+        for item in self.tracking_data.items():
+            label, data_path = item
+            if data_label == label:
+                return True
+        return False
+
 class MediaFile:
     """
     Media file that can added to and edited in Sequence.
@@ -391,9 +465,11 @@ class MediaFile:
         self.current_frame = 0
 
         self.container_data = None
-
+        self.titler_data = None
+        
         self.info = info
-
+        self.rating = appconsts.MEDIA_FILE_UNRATED
+        
         # Set default length for graphics files
         (f_name, ext) = os.path.splitext(self.name)
         if utils.file_extension_is_graphics_file(ext) and self.type != appconsts.IMAGE_SEQUENCE:
@@ -502,12 +578,13 @@ class Bin:
     Group of media files
     """
     def __init__(self, name="name"):
-        self.name  = name # Displayed name
+        self.uid = hashlib.md5(str(os.urandom(32)).encode('utf-8')).hexdigest()
+        self.name = name # Displayed name
         self.file_ids = [] # List of media files ids in the bin.
                            # Ids are increasing integers given in 
                            # Project.add_media_file(...)
-        
-        
+
+
 class ProducerNotValidError(Exception):
     def __init__(self, value, file_path):
         self.value = value
