@@ -25,13 +25,13 @@ NOTE: 'vault' in code is presented to user as 'Data Store'.
 """
 from gi.repository import Gtk, GObject, GLib, Gdk
 
-import copy
 import datetime
 import hashlib
 import os
 from os.path import isfile, join, isdir
 import pickle
 import shutil
+import traceback
 
 import atomicfile
 from editorstate import PROJECT
@@ -42,6 +42,7 @@ import diskcachemanagement
 import gui
 import guicomponents
 import guiutils
+import gtkbuilder
 import persistance
 import projectdatavault
 import userfolders
@@ -66,7 +67,7 @@ def _close_window():
     _project_data_manager_window.set_visible(False)
     _project_data_manager_window.destroy()
 
-def show_current_project_data_store_info_window():
+def show_current_project_data_store_info_window(info_panel):
     if PROJECT().vault_folder == None:
         primary_txt = _("Project created with version 2.10 or lower!")
         secondary_txt = _("This feature available only for projects created with version 2.12 or higher.")
@@ -75,7 +76,7 @@ def show_current_project_data_store_info_window():
         
     global _current_project_info_window
     
-    _current_project_info_window = CurrenProjectDataInfoWindow()
+    _current_project_info_window = CurrenProjectDataInfoWindow(info_panel)
 
 def _close_info_window():
     global _current_project_info_window
@@ -135,7 +136,17 @@ def show_create_data_dialog(callback, data=None):
         widgets = (data_folder_button, name_entry, data)
     dialogutils.panel_ok_cancel_dialog(_("Create Data Store"), vbox, _("Create Data Store"), callback, widgets)
 
-        
+def get_current_vault_name():
+    if PROJECT().vault_folder == projectdatavault.get_default_vault_folder():
+        vault_name = _("Default XDG Data Store")
+    else:
+        vaults = projectdatavault.get_vaults_object()
+        vault_name = vaults.get_user_vault_folder_name(PROJECT().vault_folder)
+        if vault_name == None:
+            vault_name = _("Project Data Store is not active")
+
+    return vault_name
+
 # --------------------------------------------------------- window classes
 class AbstractDataStoreWindow(Gtk.Window):
 
@@ -144,33 +155,34 @@ class AbstractDataStoreWindow(Gtk.Window):
         
         self.default_vault_name = _("Default XDG Data Store")
     
-    def folder_properties_panel(self, folder_handle):
+    def folder_properties_panel(self, folder_handle, show_save_data=True):
         
         info = folder_handle.data_folders_info()
         
         vbox = Gtk.VBox(False, 2)
 
-        savefile, times_saved, last_date = folder_handle.get_save_info()
+        if show_save_data == True:
+            savefile, times_saved, last_date = folder_handle.get_save_info()
 
-        save__name_label = guiutils.bold_label(_("Last Saved File:"))
-        save__name_label.set_margin_end(4)
-        try:
-            save_file_name = Gtk.Label(label=str(os.path.basename(savefile)))
-        except:
-            save_file_name = Gtk.Label(label=_("Not Saved"))
-        row = guiutils.get_left_justified_box([save__name_label, save_file_name])
-        vbox.pack_start(row, False, False, 0)
-        
-        save_label = guiutils.bold_label(_("Last Saved Time:"))
-        save_label.set_margin_end(4)
-        if str(last_date) != "0":
-            save_date = Gtk.Label(label=str(last_date))
-        else:
-            save_date = Gtk.Label(label=_("Not Saved"))
+            save__name_label = guiutils.bold_label(_("Last Saved File:"))
+            save__name_label.set_margin_end(4)
+            try:
+                save_file_name = Gtk.Label(label=str(os.path.basename(savefile)))
+            except:
+                save_file_name = Gtk.Label(label=_("Not Saved"))
+            row = guiutils.get_left_justified_box([save__name_label, save_file_name])
+            vbox.pack_start(row, False, False, 0)
+            
+            save_label = guiutils.bold_label(_("Last Saved Time:"))
+            save_label.set_margin_end(4)
+            if str(last_date) != "0":
+                save_date = Gtk.Label(label=str(last_date))
+            else:
+                save_date = Gtk.Label(label=_("Not Saved"))
 
-        row = guiutils.get_left_justified_box([save_label, save_date])
-        row.set_margin_bottom(12)
-        vbox.pack_start(row, False, False, 0)
+            row = guiutils.get_left_justified_box([save_label, save_date])
+            row.set_margin_bottom(12)
+            vbox.pack_start(row, False, False, 0)
         
         data_id_label = guiutils.bold_label(_("Data ID:"))
         data_id_label.set_margin_end(4)
@@ -274,7 +286,8 @@ class ProjectDataManagerWindow(AbstractDataStoreWindow):
         vbox = Gtk.VBox(False, 2)
         vbox.pack_start(vaults_frame, False, False, 0)
         vbox.pack_start(guiutils.pad_label(24, 24), False, False, 0)
-        vbox.pack_start(selections_frame, True, True, 0)
+        vbox.pack_start(selections_frame, False, False, 0)
+        vbox.pack_start(Gtk.Label(), True, True, 0)
         vbox.pack_start(close_hbox, False, False, 0)
 
         vbox = guiutils.set_margins(vbox, 12, 12, 12, 12)
@@ -688,7 +701,7 @@ class ProjectDataManagerWindow(AbstractDataStoreWindow):
 
 class CurrenProjectDataInfoWindow(AbstractDataStoreWindow):
     
-    def __init__(self):
+    def __init__(self, info_panel):
         AbstractDataStoreWindow.__init__(self)
 
         data_folder_path = projectdatavault.get_project_data_folder()
@@ -696,13 +709,7 @@ class CurrenProjectDataInfoWindow(AbstractDataStoreWindow):
 
         vbox = Gtk.VBox(False, 2)
 
-        if PROJECT().vault_folder == projectdatavault.get_default_vault_folder():
-            vault_name = self.default_vault_name 
-        else:
-            vaults = projectdatavault.get_vaults_object()
-            vault_name = vaults.get_user_vault_folder_name(PROJECT().vault_folder)
-            if vault_name == None:
-                vault_name = _("Project Data Store is not active")
+        vault_name = get_current_vault_name()
         store_name_label = guiutils.bold_label(_("Project Data Store:"))
         store_name_label.set_margin_end(4)
         row = guiutils.get_left_justified_box([store_name_label, Gtk.Label(label=vault_name)])
@@ -720,26 +727,34 @@ class CurrenProjectDataInfoWindow(AbstractDataStoreWindow):
         row.set_margin_bottom(24)
         vbox.pack_start(row, False, False, 0)
         
-        vbox.pack_start(self.folder_properties_panel(folder_handle), False, False, 0)
+        vbox.pack_start(self.folder_properties_panel(folder_handle, False), False, False, 0)
+        
+        guiutils.set_margins(info_panel, 12, 12, 12, 12)
+        guiutils.set_margins(vbox, 12, 12, 12, 12)
+        
+        notebook = Gtk.Notebook()                 
+        notebook.append_page(info_panel, Gtk.Label(label=_("Info")))
+        notebook.append_page(vbox, Gtk.Label(label=_("Data Store")))
 
         close_button = Gtk.Button(_("Close"))
         close_button.connect("clicked", lambda w: _close_info_window())
-
         close_hbox = Gtk.HBox(False, 2)
         close_hbox.pack_start(Gtk.Label(), True, True, 0)
         close_hbox.pack_start(close_button, False, False, 0)
         close_hbox.set_margin_top(18)
         
-        vbox.pack_start(close_hbox, False, False, 0)
+        vbox2 = Gtk.VBox(False, 2)
+        vbox2.pack_start(notebook, True, True, 0)
+        vbox2.pack_start(close_hbox, False, False, 0)
         
-        pane = guiutils.set_margins(vbox, 12, 12, 12, 12)
-        pane.set_size_request(400, 250)
+        guiutils.set_margins(vbox2, 0, 12, 12, 12)
+        vbox2.set_size_request(400, 220)
 
         self.set_transient_for(gui.editor_window.window)
-        self.set_title(_("Project Data"))
+        self.set_title(_("Project Info and Data"))
         self.connect("delete-event", lambda w, e:_close_info_window())
 
-        self.add(pane)
+        self.add(vbox2)
         self.set_position(Gtk.WindowPosition.CENTER)
         self.show_all()
 
@@ -762,6 +777,7 @@ class ProjectCloneWindow(Gtk.Window):
         GObject.GObject.__init__(self)
 
         self.clone_project_path = None
+        self.default_vault_name = _("Default XDG Data Store")
 
         # Create Data store selection combo. This is a bit involved because default valult and uservaults 
         # are not in the same data structure.
@@ -872,11 +888,17 @@ class ProjectCloneWindow(Gtk.Window):
             # Copy Project data 
             md_key = str(datetime.datetime.now()) + str(os.urandom(16))
             clone_project_data_id_str = hashlib.md5(md_key.encode('utf-8')).hexdigest()
-            clone_vault_name = self.vaults_combo.get_active_text()
-            clone_vault_path = projectdatavault.get_vaults_object().get_vault_path_for_name(clone_vault_name)
+            if self.vaults_combo.get_active() != 0:
+                clone_vault_name = self.vaults_combo.get_active_text()
+                clone_vault_path = projectdatavault.get_vaults_object().get_vault_path_for_name(clone_vault_name)
+            else:
+                # Default XDG Data Store
+                clone_vault_path = projectdatavault.get_default_vault_folder()
+                
             clone_project_data_folder_path = join(clone_vault_path, clone_project_data_id_str) + "/"
             source_data_folder = projectdatavault.get_project_data_folder()
 
+                
             shutil.copytree(source_data_folder, clone_project_data_folder_path)
 
             # Create copy of project via saving to temp file.
@@ -900,6 +922,7 @@ class ProjectCloneWindow(Gtk.Window):
                 filehandle = open(cloneproject.last_save_path, 'w')
                 filehandle.close()
             except IOError as ioe:
+                print(traceback.format_exc())
                 primary_txt = "I/O error({0})".format(ioe.errno)
                 secondary_txt = "Project cloning failed:" + ioe.strerror + "."
                 dialogutils.warning_message(primary_txt, secondary_txt, self, is_info=False)
@@ -940,6 +963,7 @@ class ProjectCloneWindow(Gtk.Window):
             GLib.timeout_add(750, self.cloning_completed)
 
         except Exception as e:
+            print(traceback.format_exc())
             primary_txt = "Cloning failed"
             secondary_txt = "Project cloning failed: " + str(e) + "."
             dialogutils.warning_message(primary_txt, secondary_txt, self, is_info=False)
