@@ -28,12 +28,14 @@ import cairo
 
 import appconsts
 import cairoarea
+import callbackbridge
 from editorstate import PROJECT
 from editorstate import PLAYER
 from editorstate import current_sequence
 import extraeditors
 import gui
 import guiutils
+import gtkbuilder
 import keyframeeditor
 import mltfilters
 import mlttransitions
@@ -60,7 +62,6 @@ COLOR_SELECT = "color_select"                               # Gtk.ColorButton
 GEOMETRY_EDITOR = "geometry_editor"                         # keyframeeditor.GeometryEditor
 FILTER_RECT_GEOM_EDITOR = "filter_rect_geometry_editor"     # keyframeeditor.FilterRectGeometryEditor
 FILTER_ROTATION_GEOM_EDITOR = "filter_rotation_geometry_editor" # Creates a single editor for multiple geometry values for using in filter 
-WIPE_SELECT = "wipe_select"                                 # Gtk.Combobox with options from mlttransitions.wipe_lumas, possible to select luma from file system
 FILTER_WIPE_SELECT = "filter_wipe_select"                   # Gtk.Combobox with options from mlttransitions.wipe_lumas
 COMBO_BOX_OPTIONS = "cbopts"                                # List of options for combo box editor displayed to user
 LADSPA_SLIDER = "ladspa_slider"                             # Gtk.HScale, does ladspa update for release changes(disconnect, reconnect)
@@ -70,7 +71,6 @@ CR_CURVES = "crcurves"                                      # Curves color edito
 COLOR_BOX = "colorbox"                                      # One band color editor with color box interface
 COLOR_LGG = "colorlgg"                                      # Editor for ColorLGG filter
 FILE_SELECTOR = "file_select"                               # File selector button for selecting single files from
-IMAGE_MEDIA_FILE_SELECTOR = "image_media_file_select"       # File selector button for selecting single files from
 FILE_TYPES = "file_types"                                   # list of files types with "." characters, like ".png.tga.bmp"
 FADE_LENGTH = "fade_length"                                 # Autofade compositors fade length
 TEXT_ENTRY = "text_entry"                                   # Text editor
@@ -92,8 +92,7 @@ SCALE_DIGITS = "scale_digits"                               # Number of decimal
 # are created deterministically from those and FilterObject.info.property_args data. So we need to save data here on change request to make the change happen.
 # This data needs to be erased always after use.
 changing_slider_to_kf_property_name = None
-re_init_editors_for_slider_type_change_func = None # monkeypatched in at app.py
-show_rotomask_func =  None # monkeypatched in, it is rotomask.py, gmic won't launch if we import
+
 
 def _p(name):
     try:
@@ -279,7 +278,7 @@ class SliderEditor:
         if self.editor_type == SLIDER: # slider -> kf editor
             global changing_slider_to_kf_property_name
             changing_slider_to_kf_property_name = self.editable_property.name
-            re_init_editors_for_slider_type_change_func()
+            callbackbridge.clipeffectseditor_refresh_clip()
         else: # kf editor -> slider
             # Save value as single keyframe or PROP_INT or PROP_FLOAT and
             # drop all but first keyframe.
@@ -314,7 +313,7 @@ class SliderEditor:
 
                 self.editable_property.write_value("0=" + str(float(first_kf_val)))
 
-            re_init_editors_for_slider_type_change_func()
+            callbackbridge.clipeffectseditor_refresh_clip()
 
 
 class KeyframesToggler:
@@ -619,81 +618,6 @@ def _maybe_untoggle_picker_botton(editable_property, picker_button):
         picker_button.set_active(False)
         picker_button.handler_unblock(editable_property.picker_toggled_id)
 
-def _get_wipe_selector(editable_property):
-    """
-    Returns GUI component for selecting wipe type.
-    """
-    # Preset luma
-    combo_box = Gtk.ComboBoxText()
-            
-    # Get options
-    keys = list(mlttransitions.wipe_lumas.keys())
-    # translate here
-    keys.sort()
-    for k in keys:
-        combo_box.append_text(k)
- 
-    # Set initial value
-    k_index = -1
-    tokens = editable_property.value.split("/")
-    test_value = tokens[len(tokens) - 1]
-    for k,v in mlttransitions.wipe_lumas.items():
-        if v == test_value:
-            k_index = keys.index(k)
-    
-    combo_box.set_active(k_index)
-    preset_luma_row = _get_two_column_editor_row(editable_property.get_display_name(), combo_box)
-    
-    # User luma
-    use_preset_luma_combo = Gtk.ComboBoxText()
-    use_preset_luma_combo.append_text(_("Preset Luma"))
-    use_preset_luma_combo.append_text(_("User Luma"))
-        
-    dialog = Gtk.FileChooserDialog(_("Select Luma File"), None, 
-                                   Gtk.FileChooserAction.OPEN, 
-                                   (_("Cancel"), Gtk.ResponseType.CANCEL,
-                                    _("OK"), Gtk.ResponseType.ACCEPT))
-    dialog.set_action(Gtk.FileChooserAction.OPEN)
-    dialog.set_select_multiple(False)
-    file_filter = Gtk.FileFilter()
-    file_filter.add_pattern("*.png")
-    file_filter.add_pattern("*.pgm")
-    file_filter.set_name(_("Wipe Luma files"))
-    dialog.add_filter(file_filter)
-        
-    user_luma_select = Gtk.FileChooserButton(dialog)
-    user_luma_select.set_size_request(210, 28)
-    
-    user_luma_label = Gtk.Label(label=_("Luma File:"))
-
-    if k_index == -1:
-        use_preset_luma_combo.set_active(1)
-        combo_box.set_sensitive(False)
-        combo_box.set_active(0)
-        user_luma_select.set_filename(editable_property.value)
-    else:
-        use_preset_luma_combo.set_active(0)
-        user_luma_select.set_sensitive(False)
-        user_luma_label.set_sensitive(False)
-    
-    user_luma_row = Gtk.HBox(False, 2)
-    user_luma_row.pack_start(use_preset_luma_combo, False, False, 0)
-    user_luma_row.pack_start(Gtk.Label(), True, True, 0)
-    user_luma_row.pack_start(user_luma_label, False, False, 2)
-    user_luma_row.pack_start(user_luma_select, False, False, 0)
-
-    editor_pane = Gtk.VBox(False)
-    editor_pane.pack_start(preset_luma_row, False, False, 4)
-    editor_pane.pack_start(user_luma_row, False, False, 4)
-
-    widgets = (combo_box, use_preset_luma_combo, user_luma_select, user_luma_label, keys)
-    
-    combo_box.connect("changed", editable_property.combo_selection_changed, keys)
-    use_preset_luma_combo.connect("changed", _wipe_preset_combo_changed, editable_property, widgets)
-    user_luma_select.connect('file-set', _wipe_lumafile_dialog_response, editable_property, widgets)
-    
-    return editor_pane
-
 def _get_filter_wipe_selector(editable_property):
     # Preset luma
     combo_box = Gtk.ComboBoxText()
@@ -760,28 +684,7 @@ class FadeLengthEditor(Gtk.HBox):
 def _get_fade_length_editor(editable_property):
     return FadeLengthEditor(editable_property)
 
-def _wipe_preset_combo_changed(widget, ep, widgets):
-    combo_box, use_preset_luma_combo, user_luma_select, user_luma_label, keys = widgets
-    if widget.get_active() == 1:
-        combo_box.set_sensitive(False)
-        user_luma_select.set_sensitive(True)
-        user_luma_label.set_sensitive(True)
-        file_name = user_luma_select.get_filename()
-        if file_name != None:
-            ep.write_value(file_name)
-    else:
-        user_luma_select.set_sensitive(False)
-        user_luma_label.set_sensitive(False)
-        combo_box.set_sensitive(True)
-        ep.combo_selection_changed(combo_box, keys)
 
-def _wipe_lumafile_dialog_response(button, ep, widgets):
-    combo_box, use_preset_luma_combo, user_luma_select, user_luma_label, keys = widgets
-    file_name = user_luma_select.get_filename()
-
-    if file_name != None:
-        ep.combo_selection_changed(combo_box, keys)
-        ep.write_value(file_name)
 
 
 def _get_file_select_editor(editable_property):
@@ -793,7 +696,8 @@ def _get_file_select_editor(editable_property):
                                    (_("Cancel"), Gtk.ResponseType.CANCEL,
                                     _("OK"), Gtk.ResponseType.ACCEPT))
     dialog.set_action(Gtk.FileChooserAction.OPEN)
-    dialog.set_select_multiple(False)
+    dialog.set_transient_for(gui.editor_window.window)
+    #dialog.set_select_multiple(False)
 
     try:
         file_types_args_list = editable_property.args[FILE_TYPES].split(".")
@@ -808,8 +712,9 @@ def _get_file_select_editor(editable_property):
         # We will interpret missing as decision to add no file filter.
         pass
         
-    file_select_button = Gtk.FileChooserButton.new_with_dialog(dialog)
+    file_select_button = gtkbuilder.get_file_chooser_button_with_dialog(dialog)
     file_select_button.set_size_request(210, 28)
+    # TODO: check this out
     if hasattr(editable_property, "value") and editable_property.value != '' and editable_property.value != '""':
         file_select_button.set_uri(GLib.filename_to_uri(editable_property.value))
 
@@ -823,34 +728,6 @@ def _get_file_select_editor(editable_property):
     dialog.connect('response', editable_property.dialog_response_callback)
 
     return editor_row
-
-def _get_image_file_select_editor(editable_property):
-    """
-    Returns GUI component for selecting image producing file
-    """
-    dialog = Gtk.FileChooserDialog(_("Select Image Producing File"), None, 
-                                   Gtk.FileChooserAction.OPEN, 
-                                   (_("Cancel"), Gtk.ResponseType.CANCEL,
-                                    _("OK"), Gtk.ResponseType.ACCEPT))
-    dialog.set_action(Gtk.FileChooserAction.OPEN)
-    dialog.set_select_multiple(False)
-
-    file_filter = utilsgtk.get_media_source_file_filter(False)    
-    dialog.add_filter(file_filter)
-        
-    file_select_button = Gtk.FileChooserButton.new_with_dialog(dialog)
-    file_select_button.set_size_request(210, 28)
-    
-    file_select_label = Gtk.Label(label=editable_property.get_display_name())
-
-    editor_row = Gtk.HBox(False, 2)
-    editor_row.pack_start(file_select_label, False, False, 2)
-    editor_row.pack_start(guiutils.get_pad_label(3, 5), False, False, 2)
-    editor_row.pack_start(file_select_button, False, False, 0)
-
-    dialog.connect('response', editable_property.dialog_response_callback)
-    
-    return editor_row
     
 def _create_composite_editor(clip, editable_properties):
     aligned = [ep for ep in editable_properties if ep.name == "aligned"][0]
@@ -1153,7 +1030,7 @@ def _create_infotips_editor(filt, editable_properties, editor_name, track, clip_
     return hbox
 
 def _roto_lauch_pressed(filt, editable_properties, property_editor_widgets_create_func, value_labels):
-    show_rotomask_func(filt, editable_properties, property_editor_widgets_create_func, value_labels)
+    callbackbridge.rotomask_show_rotomask(filt, editable_properties, property_editor_widgets_create_func, value_labels)
 
 def _create_rotomask_property_editor_widgets(editable_properties):
     # NOTE: EditanbleParam objects for are usually created in  propertyedit.get_filter_editable_properties(), this a deviation from normal pipeline
@@ -1311,12 +1188,10 @@ EDITOR_ROW_CREATORS = { \
     KEYFRAME_EDITOR_RELEASE: lambda ep : _get_keyframe_editor_release(ep),
     GEOMETRY_EDITOR: lambda ep : _get_geometry_editor(ep),
     COLOR_SELECT: lambda ep: _get_color_selector(ep),
-    WIPE_SELECT: lambda ep: _get_wipe_selector(ep),
     FILTER_WIPE_SELECT:  lambda ep: _get_filter_wipe_selector(ep),
     LADSPA_SLIDER: lambda ep: _get_ladspa_slider_row(ep),
     CLIP_FRAME_SLIDER: lambda ep: _get_clip_frame_slider(ep),
     FILE_SELECTOR: lambda ep: _get_file_select_editor(ep),
-    IMAGE_MEDIA_FILE_SELECTOR: lambda ep: _get_image_file_select_editor(ep),
     FADE_LENGTH: lambda ep: _get_fade_length_editor(ep),
     NO_EDITOR: lambda ep: _get_no_editor(),
     COMPOSITE_EDITOR_BUILDER: lambda comp, editable_properties: _create_composite_editor(comp, editable_properties),
