@@ -26,12 +26,12 @@ import math
 import os
 
 import dialogutils
-import editorpersistance # Aug-2019- SvdB - BB
 import editorstate
 import gui
 import guicomponents
 import guipopover
 import guiutils
+import gtkbuilder
 from editorstate import current_sequence
 import renderconsumer
 import utils
@@ -66,7 +66,7 @@ def render_progress_dialog(callback, parent_window, frame_rates_match=True):
 
     if frame_rates_match == False:
         # Aug-2019 - SvdB - BB
-        if editorpersistance.prefs.double_track_hights:
+        if guiutils.double_icon_size():
             warning_icon = Gtk.Image.new_from_icon_name("dialog-warning", Gtk.IconSize.DND)
         else:
             warning_icon = Gtk.Image.new_from_icon_name("dialog-warning", Gtk.IconSize.MENU)
@@ -175,8 +175,8 @@ def show_slowmo_dialog(media_file, default_range_render, _response_callback):
 
     name, ext = os.path.splitext(file_name)
         
-    dialog = Gtk.Dialog(_("Render Slow/Fast Motion Video File"), gui.editor_window.window,
-                        Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
+    dialog = Gtk.Dialog(_("Render Slow/Fast Motion Video File"), None,
+                        None,
                         (Gtk.STOCK_CANCEL, Gtk.ResponseType.REJECT,
                         _("Render"), Gtk.ResponseType.ACCEPT))
 
@@ -208,7 +208,7 @@ def show_slowmo_dialog(media_file, default_range_render, _response_callback):
     name_row.pack_start(fb_widgets.file_name, True, True, 0)
     name_row.pack_start(fb_widgets.extension_label, False, False, 4)
     
-    fb_widgets.out_folder = Gtk.FileChooserButton(_("Select Target Folder"))
+    fb_widgets.out_folder = gtkbuilder.get_file_chooser_button(_("Select Target Folder"))
     fb_widgets.out_folder.set_action(Gtk.FileChooserAction.SELECT_FOLDER)
     fb_widgets.out_folder.set_current_folder(folder)
     
@@ -298,6 +298,192 @@ def show_slowmo_dialog(media_file, default_range_render, _response_callback):
     dialog.connect('response', _response_callback, fb_widgets, media_file)
     dialog.show_all()
 
+def show_tline_clip_slowmo_dialog(clip, track, completed_callback, _response_callback):
+    folder, file_name = os.path.split(clip.path)
+
+    name, ext = os.path.splitext(file_name)
+        
+    dialog = Gtk.Dialog(_("Clip Slow/Fast Motion"), gui.editor_window.window,
+                        Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
+                        (Gtk.STOCK_CANCEL, Gtk.ResponseType.REJECT,
+                        _("Render"), Gtk.ResponseType.ACCEPT))
+
+    media_file_label = Gtk.Label(label=_("Clip: "))
+    media_name = Gtk.Label(label="<b>" + clip.name + "</b>")
+    media_name.set_use_markup(True)
+    SOURCE_PAD = 8
+    SOURCE_HEIGHT = 20
+    mf_row = guiutils.get_left_justified_box([media_file_label,  guiutils.pad_label(SOURCE_PAD, SOURCE_HEIGHT), media_name])
+    
+    fb_widgets = utils.EmptyClass()
+
+    label = Gtk.Label(label=_("Speed %:"))
+
+    if clip.slowmo_data == None:
+        current_speed = 1.0
+    else:
+        slowmo_type, orig_file_path, slowmo_clip_media_area, current_speed, orig_media_in, orig_media_out = clip.slowmo_data
+
+    adjustment = Gtk.Adjustment(value=float(100 * current_speed), lower=float(1), upper=float(2900), step_increment=float(1))
+    fb_widgets.adjustment = adjustment
+
+    spin = Gtk.SpinButton()
+    spin.set_numeric(True)
+    spin.set_adjustment(adjustment)
+    spin.set_digits(0)
+
+    fb_widgets.extension_label = Gtk.Label()
+    fb_widgets.extension_label.set_size_request(45, 20)
+    
+    objects_list = Gtk.TreeStore(str, bool)
+    objects_list.append(None, [_("Current Clip Content"), True])
+    objects_list.append(None, [_("Full Media"), True])
+
+    fb_widgets.render_range = Gtk.ComboBox.new_with_model(objects_list)
+    renderer_text = Gtk.CellRendererText()
+    fb_widgets.render_range.pack_start(renderer_text, True)
+    fb_widgets.render_range.add_attribute(renderer_text, "text", 0)
+    fb_widgets.render_range.add_attribute(renderer_text, 'sensitive', 1)
+    fb_widgets.render_range.show()
+    fb_widgets.render_range.set_active(0)
+
+    slider_hbox = Gtk.HBox(False, 4)
+    slider_hbox.pack_start(Gtk.Label(), True, True, 0)
+    slider_hbox.pack_start(spin, False, False, 4)
+    slider_hbox.pack_start(Gtk.Label(), True, True, 0)
+    slider_hbox.set_size_request(450,35)
+
+    hbox = Gtk.HBox(False, 2)
+    hbox.pack_start(guiutils.pad_label(8, 8), False, False, 0)
+    hbox.pack_start(slider_hbox, False, False, 0)
+
+    profile_selector = ProfileSelector()
+    profile_selector.set_initial_selection()
+    profile_selector.widget.set_sensitive(True)
+    fb_widgets.categories_combo = profile_selector.categories_combo
+
+    quality_selector = RenderQualitySelector()
+    fb_widgets.quality_cb = quality_selector.widget
+    
+    # Encoding
+    encoding_selector = RenderEncodingSelector(quality_selector, fb_widgets.extension_label, None)
+    encoding_selector.encoding_selection_changed()
+    fb_widgets.encodings_cb = encoding_selector.widget
+
+    # Build gui
+    vbox = Gtk.VBox(False, 2)
+    vbox.pack_start(mf_row, False, False, 0)
+    vbox.pack_start(guiutils.pad_label(18, 12), False, False, 0)
+    vbox.pack_start(label, False, False, 0)
+    vbox.pack_start(hbox, False, False, 0)
+    vbox.pack_start(guiutils.pad_label(24, 12), False, False, 0)
+    vbox.pack_start(guiutils.get_two_column_box(Gtk.Label(label=_("Encoding:")), fb_widgets.encodings_cb, 200), False, False, 0)
+    vbox.pack_start(guiutils.get_two_column_box(Gtk.Label(label=_("Quality:")), fb_widgets.quality_cb, 200), False, False, 0)
+    vbox.pack_start(guiutils.pad_label(18, 12), False, False, 0)
+    vbox.pack_start(guiutils.get_two_column_box(Gtk.Label(label=_("Render Range:")), fb_widgets.render_range, 180), False, False, 0)
+    
+    alignment = guiutils.set_margins(vbox, 6, 24, 24, 24)
+    
+    dialog.vbox.pack_start(alignment, True, True, 0)
+    dialogutils.set_outer_margins(dialog.vbox)
+    dialogutils.default_behaviour(dialog)
+    dialog.connect('response', _response_callback, fb_widgets, clip, track, completed_callback)
+    dialog.show_all()
+
+def show_tline_clip_reverse_dialog(clip, track, completed_callback, _response_callback):
+    folder, file_name = os.path.split(clip.path)
+
+    name, ext = os.path.splitext(file_name)
+        
+    dialog = Gtk.Dialog(_("Clip Reverse Motion"), gui.editor_window.window,
+                        Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
+                        (Gtk.STOCK_CANCEL, Gtk.ResponseType.REJECT,
+                        _("Render"), Gtk.ResponseType.ACCEPT))
+
+    media_file_label = Gtk.Label(label=_("Clip: "))
+    media_name = Gtk.Label(label="<b>" + clip.name + "</b>")
+    media_name.set_use_markup(True)
+    SOURCE_PAD = 8
+    SOURCE_HEIGHT = 20
+    mf_row = guiutils.get_left_justified_box([media_file_label,  guiutils.pad_label(SOURCE_PAD, SOURCE_HEIGHT), media_name])
+    
+    fb_widgets = utils.EmptyClass()
+
+    label = Gtk.Label(label=_("Reverse Speed %:"))
+
+    if clip.slowmo_data == None:
+        current_speed = 1.0
+    else:
+        slowmo_type, orig_file_path, slowmo_clip_media_area, current_speed, orig_media_in, orig_media_out = clip.slowmo_data
+        current_speed = abs(current_speed) # We're using positive value for SpinButton display, but need negative value for clip creation. 
+
+    adjustment = Gtk.Adjustment(value=float(100 * current_speed), lower=float(1), upper=float(600), step_increment=float(1))
+
+    fb_widgets.adjustment = adjustment
+
+    spin = Gtk.SpinButton()
+    spin.set_numeric(True)
+    spin.set_adjustment(adjustment)
+    spin.set_digits(0)
+
+    fb_widgets.extension_label = Gtk.Label()
+    fb_widgets.extension_label.set_size_request(45, 20)
+    
+    objects_list = Gtk.TreeStore(str, bool)
+    objects_list.append(None, [_("Current Clip Content"), True])
+    objects_list.append(None, [_("Full Media"), True])
+
+    fb_widgets.render_range = Gtk.ComboBox.new_with_model(objects_list)
+    renderer_text = Gtk.CellRendererText()
+    fb_widgets.render_range.pack_start(renderer_text, True)
+    fb_widgets.render_range.add_attribute(renderer_text, "text", 0)
+    fb_widgets.render_range.add_attribute(renderer_text, 'sensitive', 1)
+    fb_widgets.render_range.show()
+    fb_widgets.render_range.set_active(0)
+
+    slider_hbox = Gtk.HBox(False, 4)
+    slider_hbox.pack_start(Gtk.Label(), True, True, 0)
+    slider_hbox.pack_start(spin, False, False, 4)
+    slider_hbox.pack_start(Gtk.Label(), True, True, 0)
+    slider_hbox.set_size_request(450,35)
+
+    hbox = Gtk.HBox(False, 2)
+    hbox.pack_start(guiutils.pad_label(8, 8), False, False, 0)
+    hbox.pack_start(slider_hbox, False, False, 0)
+
+    profile_selector = ProfileSelector()
+    profile_selector.set_initial_selection()
+    profile_selector.widget.set_sensitive(True)
+    fb_widgets.categories_combo = profile_selector.categories_combo
+
+    quality_selector = RenderQualitySelector()
+    fb_widgets.quality_cb = quality_selector.widget
+    
+    # Encoding
+    encoding_selector = RenderEncodingSelector(quality_selector, fb_widgets.extension_label, None)
+    encoding_selector.encoding_selection_changed()
+    fb_widgets.encodings_cb = encoding_selector.widget
+
+    # Build gui
+    vbox = Gtk.VBox(False, 2)
+    vbox.pack_start(mf_row, False, False, 0)
+    vbox.pack_start(guiutils.pad_label(18, 12), False, False, 0)
+    vbox.pack_start(label, False, False, 0)
+    vbox.pack_start(hbox, False, False, 0)
+    vbox.pack_start(guiutils.pad_label(24, 12), False, False, 0)
+    vbox.pack_start(guiutils.get_two_column_box(Gtk.Label(label=_("Encoding:")), fb_widgets.encodings_cb, 200), False, False, 0)
+    vbox.pack_start(guiutils.get_two_column_box(Gtk.Label(label=_("Quality:")), fb_widgets.quality_cb, 200), False, False, 0)
+    vbox.pack_start(guiutils.pad_label(18, 12), False, False, 0)
+    vbox.pack_start(guiutils.get_two_column_box(Gtk.Label(label=_("Render Range:")), fb_widgets.render_range, 180), False, False, 0)
+    
+    alignment = guiutils.set_margins(vbox, 6, 24, 24, 24)
+    
+    dialog.vbox.pack_start(alignment, True, True, 0)
+    dialogutils.set_outer_margins(dialog.vbox)
+    dialogutils.default_behaviour(dialog)
+    dialog.connect('response', _response_callback, fb_widgets, clip, track, completed_callback)
+    dialog.show_all()
+    
 def _slomo_speed_changed(adjustment, media_file, range_combo, length_label):
     clip_length = _get_rendered_slomo_clip_length(media_file, range_combo, adjustment.get_value())
     length_label.set_text(utils.get_tc_string(clip_length))
@@ -321,8 +507,8 @@ def show_reverse_dialog(media_file, default_range_render, _response_callback):
 
     name, ext = os.path.splitext(file_name)
         
-    dialog = Gtk.Dialog(_("Render Reverse Motion Video File"), gui.editor_window.window,
-                        Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
+    dialog = Gtk.Dialog(_("Render Reverse Motion Video File"), None,
+                        None,
                         (Gtk.STOCK_CANCEL, Gtk.ResponseType.REJECT,
                         _("Render"), Gtk.ResponseType.ACCEPT))
 
@@ -354,7 +540,7 @@ def show_reverse_dialog(media_file, default_range_render, _response_callback):
     name_row.pack_start(fb_widgets.file_name, True, True, 0)
     name_row.pack_start(fb_widgets.extension_label, False, False, 4)
     
-    fb_widgets.out_folder = Gtk.FileChooserButton(_("Select Target Folder"))
+    fb_widgets.out_folder = gtkbuilder.get_file_chooser_button(_("Select Target Folder"))
     fb_widgets.out_folder.set_action(Gtk.FileChooserAction.SELECT_FOLDER)
     fb_widgets.out_folder.set_current_folder(folder)
     
@@ -617,7 +803,8 @@ def get_render_panel_left(render_widgets, render_clicked_cb, to_queue_clicked_cb
             render_panel.pack_start(Gtk.Label(), True, True, 0)
     else:
         render_panel.pack_start(encoding_panel, False, False, 0)
-        render_panel.pack_start(Gtk.Label(), True, True, 0)
+        if editorstate.SCREEN_HEIGHT > 800:
+            render_panel.pack_start(Gtk.Label(), True, True, 0)
     
     # Stop panel **width** popping when/if GPU renders added.
     render_panel.set_size_request(300, 30)
@@ -682,7 +869,7 @@ class RenderFilePanel():
 
     def __init__(self):
 
-        self.out_folder = Gtk.FileChooserButton(_("Select Folder"))
+        self.out_folder = gtkbuilder.get_file_chooser_button(_("Select Folder"))
         self.out_folder.set_action(Gtk.FileChooserAction.SELECT_FOLDER)
         self.out_folder.set_current_folder(os.path.expanduser("~") + "/")
         gui.render_out_folder = self.out_folder
@@ -705,6 +892,7 @@ class RenderFilePanel():
         self.out_folder.set_tooltip_text(_("Select folder to place rendered file in"))
         self.movie_name.set_tooltip_text(_("Give name for rendered file"))
 
+        
 
 class RenderRangePanel():
     
@@ -756,7 +944,7 @@ class RenderEncodingPanel():
     
     def __init__(self, extension_label):
         self.quality_selector = RenderQualitySelector()
-        self.quality_selector.widget.set_size_request(110, 34)
+        self.quality_selector.widget.set_size_request(110, 20)
         self.quality_selector.update_quality_selection(0)
         self.audio_desc = Gtk.Label()
         self.encoding_selector = RenderEncodingSelector(self.quality_selector,
@@ -1005,6 +1193,7 @@ class RenderArgsRow():
         guiutils.set_margins(self.pop_over_pane, 4,4,4,4)
         self.pop_over_pane.show_all()
         
+        
         # ----------------------------------------- build GUI
         self.box = Gtk.VBox(False, 2)
         self.box.pack_start(use_opts_row , False, False, 0)
@@ -1273,6 +1462,4 @@ class RenderArgsEditWindow(Gtk.Window):
         self.set_title(_("Render Args"))
         self.show_all()
         self.set_resizable(False)
-        self.set_keep_above(True) # Perhaps configurable later
 
-    
