@@ -46,6 +46,9 @@ PHANTOM_JAR = None
 FLUXITY_EMPTY_BG_RES_PATH = None
 MEDIA_PLUGINS_PATH = None
 FLUXITY_API_DOC = None
+USBHID_DRIVERS_PATH = None
+INFO_TIPS_DOC = None
+
 
 def set_paths(root_path):
     # Apr-2017 - SvdB - Added SHORTCUTS_PATH for keyboard shortcuts files
@@ -56,13 +59,13 @@ def set_paths(root_path):
     LAUNCH_DIR, REPLACEMENTS_XML_DOC, GMIC_SCRIPTS_DOC,  \
     PHANTOM_JAR, PHANTOM_DIR, DEVELOPERS_DOC, CONTRIBUTORS_DOC, \
     SHORTCUTS_PATH, FLUXITY_EMPTY_BG_RES_PATH, MEDIA_PLUGINS_PATH, \
-    FLUXITY_API_DOC
-    
+    FLUXITY_API_DOC, USBHID_DRIVERS_PATH, INFO_TIPS_DOC
+
     ROOT_PATH = root_path
     IMAGE_PATH = ROOT_PATH + "/res/darktheme/"
     WIPE_RESOURCES_PATH = root_path + "/res/filters/wipes/"
     PROFILE_PATH = root_path + "/res/profiles/"
-    BLACK_IMAGE_PATH = root_path + "/res/img/black.jpg"
+    BLACK_IMAGE_PATH = root_path + "/res/darktheme/black.jpg"
     FILTERS_XML_DOC = root_path + "/res/filters/filters.xml"
     COMPOSITORS_XML_DOC = root_path + "/res/filters/compositors.xml"
     REPLACEMENTS_XML_DOC = root_path + "/res/filters/replace.xml"
@@ -78,8 +81,16 @@ def set_paths(root_path):
     LAUNCH_DIR = root_path + "/launch/"
     GMIC_SCRIPTS_DOC = root_path + "/res/gmic/gmic2scripts.xml"
     PHANTOM_JAR = root_path + "/phantom2d/Phantom2D.jar"
-    # Apr-2017 - SvdB
     SHORTCUTS_PATH = root_path + "/res/shortcuts/"
     FLUXITY_EMPTY_BG_RES_PATH = root_path + "/res/scripttool/not_rendered.png"
     MEDIA_PLUGINS_PATH = root_path + "/res/mediaplugins/"
     FLUXITY_API_DOC = root_path + "/res/mediaplugins/fluxity.html"
+    INFO_TIPS_DOC = root_path + "/res/help/en/infotips.html"
+    USBHID_DRIVERS_PATH = root_path + "/res/usbhid/"
+    
+def apply_dark_theme():
+    global IMAGE_PATH
+    IMAGE_PATH = ROOT_PATH + "/res/darktheme/"
+
+
+
