@@ -81,9 +81,9 @@ def set_track_counts(project):
     VIDEO_TRACKS_COUNT = AUDIO_TRACKS_COUNT + 1
 
 # Output modes. These correspond to option indexes in guicomponents.get_monitor_view_select_combo()
-PROGRAM_OUT_MODE = 0
-VECTORSCOPE_MODE = 1
-RGB_PARADE_MODE = 2
+PROGRAM_OUT_MODE = appconsts.PROGRAM_OUT_MODE
+VECTORSCOPE_MODE = appconsts.VECTORSCOPE_MODE
+RGB_PARADE_MODE = appconsts.RGB_PARADE_MODE
 
 # Black clip
 black_track_clip = None
@@ -260,7 +260,7 @@ class Sequence:
         
         return new_track
 
-    def _add_track_attributes(self, track, type):                
+    def _add_track_attributes(self, track, type):
         # Add data attr
         track.type = type
         track.sequence = self
@@ -497,12 +497,15 @@ class Sequence:
         clip.sync_data = None 
         clip.mute_filter = None #
         clip.stream_indexes = None # a, v stream indexes when not muted
-        clip.clip_length = lambda: _clip_length(clip) # MLT get_length gives wrong values for blanks
+        clip.clip_length = lambda: _clip_length(clip) # MLT producer.get_length() gives producer max length.
         clip.waveform_data = None
         clip.color = None # None means that clip type default color is displayed
         clip.markers = []
         clip.container_data = None
-        
+        clip.titler_data = None # titler_data != None defines clips as a title clip that can be edited in Titler.
+        clip.slowmo_data = None # tuple (slowmo_type, orig_media_file_path, slowmo_clip_media_area, slowmo_speed_data,
+                                #        orig_media_in, orig_media_out)
+
     def clone_track_clip(self, track, index):
         orig_clip = track.clips[index]
         if orig_clip.is_blanck_clip == True:
@@ -533,7 +536,11 @@ class Sequence:
             clone_filter = mltfilters.clone_filter_object(f, self.profile)
             clone_clip.attach(clone_filter.mlt_filter)
             clone_clip.filters.append(clone_filter)
-    
+
+        # Image clips longer then 15000 frames need to have their length property set.
+        if clip.media_type == appconsts.IMAGE and clip.clip_out > 15000:
+            clone_clip.set("length", int(clip.clip_out - 1)) # -1, out is inclusive
+
     def copy_filters(self, clip, clone_clip):
         for f in clip.filters:
             clone_filter = mltfilters.clone_filter_object(f, self.profile)
@@ -738,7 +745,7 @@ class Sequence:
         """
         Compositor order must be from top to bottom or will not work.
         """
-        if self.compositing_mode != appconsts.COMPOSITING_MODE_STANDARD_AUTO_FOLLOW and self.compositing_mode != appconsts.COMPOSITING_MODE_STANDARD_FULL_TRACK:
+        if self.compositing_mode != appconsts.COMPOSITING_MODE_STANDARD_FULL_TRACK:
             self.compositors.sort(key=_sort_compositors_comparator, reverse=True)
         else:
             self.compositors.sort(key=_sort_compositors_comparator)
