@@ -24,19 +24,14 @@ Module handles button edit events from buttons in the middle bar and other non-t
 
 
 
-from gi.repository import Gtk
-from gi.repository import Gdk
+from gi.repository import Gtk, GLib
 
 import copy
-import hashlib
-import os
 from operator import itemgetter
-import threading
-import time
 
-import app
 import appconsts
 import boxmove
+import callbackbridge
 import clipeffectseditor
 import compositeeditor
 import compositormodes
@@ -45,9 +40,7 @@ import dialogs
 import dialogutils
 import glassbuttons
 import gui
-import guicomponents
 import guipopover
-import guiutils
 import edit
 import editevent
 import editorpersistance
@@ -61,15 +54,10 @@ from editorstate import MONITOR_MEDIA_FILE
 from editorstate import EDIT_MODE
 import movemodes
 import multimovemode
-import mlttransitions
 import projectaction
-import render
-import renderconsumer
-import respaths
 import sequence
 import syncsplitevent
 import updater
-import userfolders
 import utils
 
 
@@ -283,7 +271,7 @@ def split_confirmed(dialog, response_id):
     name = _("sequence_") + str(PROJECT().next_seq_number)
     sequence.VIDEO_TRACKS_COUNT, sequence.AUDIO_TRACKS_COUNT = current_sequence().get_track_counts()
     PROJECT().add_named_sequence(name)
-    app.change_current_sequence(len(PROJECT().sequences) - 1)
+    callbackbridge.app_change_current_sequence(len(PROJECT().sequences) - 1)
     # New sequence needs to have same compositing mode as current.
     projectaction.do_compositing_mode_change(current_sequence().compositing_mode)
 
@@ -916,6 +904,16 @@ def _view_mode_menu_item_item_activated(action, new_value_variant):
     action.set_state(new_value_variant)
     guipopover._monitorview_popover.hide()
 
+def set_monitor_display_mode(display_mode):
+    editorstate.current_sequence().set_output_mode(display_mode)
+    editorstate.tline_view_mode = display_mode
+    action = editorstate.APP().lookup_action("monitor.viewimage")
+    new_value_variant = GLib.Variant.new_string(str(display_mode))
+    try:
+        action.set_state(new_value_variant)
+    except:
+        pass # this action gets create at first menu launch.
+
 def _opacity_menu_item_item_activated(action, new_value_variant):
     msg = int(new_value_variant.get_string())
     mix_value_index = msg - 3 ## this just done in a bit hackish way, 
