@@ -24,14 +24,13 @@ import gi
 gi.require_version('Gtk', '3.0')
 
 from gi.repository import GObject, GLib, Gio
-from gi.repository import Gtk, Gdk, GdkPixbuf
+from gi.repository import Gtk, GdkPixbuf
 
 try:
     import mlt7 as mlt
 except:
     import mlt
 import hashlib
-import locale
 import os
 from os import listdir
 from os.path import isfile, join
@@ -52,14 +51,13 @@ import editorstate
 import editorpersistance
 import gui
 import guiutils
+import gtkbuilder
 import mltinit
 import mltprofiles
-import processutils
 import persistance
 import respaths
 import renderconsumer
 import toolguicomponents
-import translations
 import userfolders
 import utils
 
@@ -408,7 +406,7 @@ class BatchRenderApp(Gtk.Application):
         self.connect("activate", self.on_activate)
 
     def on_activate(self, data=None):
-        gui.apply_theme(editorpersistance.prefs.theme)
+        gui.apply_theme()
 
         mltinit.init_with_translations()
         
@@ -1132,8 +1130,8 @@ def show_change_render_item_path_dialog(callback, render_item):
     cancel_str = _("Cancel")
     ok_str = _("Change Path")
     dialog = Gtk.Dialog(_("Change Render Item Path"),
-                        batch_window.window,
-                        Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
+                        None,
+                        None,
                         (cancel_str, Gtk.ResponseType.CANCEL,
                         ok_str, Gtk.ResponseType.YES))
 
@@ -1141,7 +1139,7 @@ def show_change_render_item_path_dialog(callback, render_item):
     
     folder, f_name = os.path.split(render_item.render_path)
 
-    out_folder = Gtk.FileChooserButton(_("Select target folder"))
+    out_folder = gtkbuilder.get_file_chooser_button(_("Select target folder"))
     out_folder.set_action(Gtk.FileChooserAction.SELECT_FOLDER)
     out_folder.set_current_folder(folder)
     
@@ -1279,7 +1277,7 @@ class SingleRenderApp(Gtk.Application):
         self.connect("activate", self.on_activate)
 
     def on_activate(self, data=None):
-        gui.apply_theme(editorpersistance.prefs.theme)
+        gui.apply_theme()
 
         mltinit.init_with_translations()
     
