@@ -200,6 +200,7 @@
     |:--------|:--------|
     | 1 | Initial release. |
     | 2 | Added methods FluxityContext.required_api_version(), FluxityContext.add_editor_group() |
+    | 3 | Added method FluxityContext.get_editors_values_clone_dict(), Added editor EDITOR_HTML_LINK, added global variable LOCAL_ROOT |
 
     # FLUXITY API
 """
@@ -326,6 +327,8 @@ EDITOR_PANGO_FONT = 10
 """ Editor for setting pango font properties."""
 EDITOR_TEXT_AREA = 11
 """ Editor for creating multiline text."""
+EDITOR_HTML_LINK = 12
+""" Editor for opening a HTML page, generally to make some additional documentation available."""
 
 EDITOR_PANGO_FONT_DEFAULT_VALUES = ("Liberation Serif", "Regular", 80, Pango.Alignment.LEFT, (1.0, 1.0, 1.0, 1.0), \
               True, (0.3, 0.3, 0.3, 1.0) , False, 2, False, (0.0, 0.0, 0.0), \
@@ -368,7 +371,10 @@ KEYFRAME_SMOOTH = 1
 KEYFRAME_DISCRETE = 2
 """Value after keyframe of this type is value at keyframe."""
 
-API_VERSION = 2
+LOCAL_ROOT = "##LOCAL##"
+"""This value is replaced with an absolute path to local installation root directory when creating a path to a local resource."""
+
+API_VERSION = 3
 """API version number, increasing integer for each Flowblade release with changes."""
 
 # ---------------------------------------------------------- script object
@@ -419,6 +425,8 @@ class FluxityScript:
         """
         try:
             self.namespace['init_render'](fctx)
+        except FluxityUserMessageError as e:
+            _raise_message_error(str(e))
         except Exception as e:
             _raise_fluxity_error("error calling function 'init_render()':\n\n" + str(e))
           
@@ -428,6 +436,8 @@ class FluxityScript:
         """
         try:
             self.namespace['render_frame'](frame, fctx, w, h)
+        except FluxityUserMessageError as e:
+            _raise_message_error(str(e))
         except Exception as e:
             _raise_fluxity_error("error calling function 'render_frame()':\n\n" + str(e))
 
@@ -602,6 +612,8 @@ class FluxityContext:
           
           * `EDITOR_INT_RANGE`(3-tuple with int values, (default, min, max))
           
+          * `EDITOR_HTML_LINK`((str) A path to a HTML resource, either local or in the web),
+          
           * `EDITOR_PANGO_FONT` (17-tuple (font_family, font_face, font_size, alignment, color_rgba,
                   fill_on, outline_color_rgba, outline_on, outline_width, shadow_on, shadow_color_rgb, 
                   shadow_opacity, shadow_xoff, shadow_yoff, shadow_blur, 
@@ -613,6 +625,8 @@ class FluxityContext:
         
         Data describing editors can be accessed with *get_script_data(self)*. Edited values are made available for script with *set_editors_data(self, editors_data_json)*.
         
+        EDITOR_HTML_LINK can use global variable fluxity.LOCAL_ROOT to point to installation root directory as part of a path to an installed resource.
+        
         Must be called in script method *init_script()*.
         """
         self.priv_context.error_on_wrong_method("add_editor()", METHOD_INIT_SCRIPT)
@@ -651,7 +665,9 @@ class FluxityContext:
           * `EDITOR_FLOAT_RANGE`(3-tuple with float values, (default, min, max)), 
           
           * `EDITOR_INT_RANGE`(3-tuple with int values, (default, min, max))
-
+          
+          * `EDITOR_HTML_LINK`(str),
+          
           * `EDITOR_PANGO_FONT`(17-tuple (font_family, font_face, font_size, alignment, color_rgba,
                   fill_on, outline_color_rgba, outline_on, outline_width, shadow_on, shadow_color_rgb, 
                   shadow_opacity, shadow_xoff, shadow_yoff, shadow_blur,
@@ -670,6 +686,27 @@ class FluxityContext:
             exception_msg = "No editor for name '" + name + "' found."
             _raise_fluxity_error(exception_msg)
 
+    def get_editors_values_clone_dict(self):
+        """         
+        Return dictionary containing values of all edited data.
+        
+        **Returns:** (dict) Dictionary {name:value} containing all editors.
+        """
+        editor_values = {}
+        
+        for name, value_item in self.editors.items():
+            type, value = value_item
+            if type == EDITOR_INT_RANGE or type == EDITOR_FLOAT_RANGE:
+                val, min, max = value
+                value = val 
+            elif type == EDITOR_OPTIONS:
+                selected_index, options = value
+                value = selected_index
+            
+            editor_values[name] = value
+        
+        return editor_values
+
     def get_script_data(self):
         """             
         Returns data of all editors and their default values, and script metadata like script author and version. 
@@ -1339,6 +1376,19 @@ def _raise_compile_error(exception_msg):
 def _raise_exec_error(exception_msg):
     raise FluxityError("Error on doing exec() to create script code object:\n" + exception_msg)
 
+
+class FluxityUserMessageError(Exception):
+    """
+    Errors used to communicate particular info to user without traceback.
+    """
+    def __init__(self, value):
+        self.value = value
+    def __str__(self):
+        return self.value
+
+def _raise_message_error(exception_msg):
+    raise FluxityUserMessageError(exception_msg)
+            
 # ------------------------------------------------------ rendering
 def render_preview_frame(script, script_file, frame, generator_length, out_folder, profile_file_path, editors_data_json=None):
     """
@@ -1358,7 +1408,7 @@ def render_preview_frame(script, script_file, frame, generator_length, out_folde
     
     **Returns:** (FluxityContext) Object created during rendering. This object has attributes *error* and *log_msg* providing error and logging information.
     """
- 
+
     try:
         # Init script and context.
         error_msg, results = _init_script_and_context(script, script_file, out_folder, profile_file_path)
@@ -1388,6 +1438,9 @@ def render_preview_frame(script, script_file, frame, generator_length, out_folde
         w, h = fctx.get_dimensions()
         fscript.call_render_frame(frame, fctx, w, h)
 
+        return fctx
+    except FluxityUserMessageError as e:
+        fctx.error = str(e)
         return fctx
     except Exception as e:
         fctx.error = str(e) + traceback.format_exc(6,True)
