@@ -26,9 +26,7 @@ from gi.repository import GLib, Gio
 from gi.repository import Gtk, Gdk, GdkPixbuf
 from gi.repository import Pango
 
-import copy
 import cairo
-import locale
 try:
     import mlt7 as mlt
 except:
@@ -48,13 +46,12 @@ import dialogutils
 import editorstate
 import editorpersistance
 import gui
-import guicomponents
 import guiutils
 import glassbuttons
+import gtkbuilder
 import mltinit
 import mltprofiles
 import positionbar
-import processutils
 import respaths
 import renderconsumer
 import toolguicomponents
@@ -206,7 +203,7 @@ class GMicApplication(Gtk.Application):
             MONITOR_HEIGHT = 400 # initial value, this gets changed when material is loaded
 
         # Themes
-        gui.apply_theme(editorpersistance.prefs.theme)
+        gui.apply_theme()
         
         # Init mlt.
         repo = mltinit.init_with_translations()
@@ -341,8 +338,19 @@ def _finish_clip_open(use_default_profile):
 
     global _player, _frame_writer
     _player = gmicplayer.GmicPlayer(_current_path)
+    _player.set_display_widget(_window.monitor)
+    _window.monitor.connect("size-allocate", lambda w, e:_player.display_resized())
+ 
     _frame_writer = gmicplayer.PreviewFrameWriter(_current_path)
 
+    # Set SDL consumer version to be used.
+    #if editorstate.mlt_version_is_greater_correct("7.28.0") or editorstate.force_sdl2 == True \
+    #    or editorstate.app_running_from == editorstate.RUNNING_FROM_FLATPAK:
+    if editorstate.force_sdl2 == True:
+        gmicplayer.set_sdl_consumer_version(gmicplayer.SDL_2)
+    else:
+        gmicplayer.set_sdl_consumer_version(gmicplayer.SDL_1)
+            
     _window.set_fps()
     _window.set_monitor_sizes()
     if use_default_profile == False:
@@ -531,8 +539,8 @@ def _encode_settings_clicked():
     align = dialogutils.get_default_alignment(_encoding_panel)
     
     dialog = Gtk.Dialog(_("Video Encoding Settings"),
-                        _window,
-                        Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
+                        None,
+                        None,
                         (_("Cancel"), Gtk.ResponseType.REJECT,
                          _("Set Encoding"), Gtk.ResponseType.ACCEPT))
     dialog.vbox.pack_start(align, True, True, 0)
@@ -574,11 +582,11 @@ class GmicWindow(Gtk.Window):
         app_icon = GdkPixbuf.Pixbuf.new_from_file(respaths.IMAGE_PATH + "flowbladetoolicon.png")
         self.set_icon(app_icon)
         hamburger_launcher_surface = guiutils.get_double_scaled_cairo_image("hamburger.png")
-        if editorpersistance.prefs.double_track_hights == False:
+        if guiutils.double_icon_size() == False:
             psize = 22
         else:
             psize = 44
-        self.hamburger_launcher = guicomponents.PressLaunch(self.hamburger_launch_pressed, hamburger_launcher_surface, psize, psize)
+        self.hamburger_launcher = toolguicomponents.PressLaunchSurface(self.hamburger_launch_pressed, hamburger_launcher_surface, psize, psize)
         #self.hamburger_launcher.connect_launched_menu(_hamburger_menu)
         
         # Load media row
@@ -631,7 +639,7 @@ class GmicWindow(Gtk.Window):
         monitors_panel.pack_start(right_vbox, False, False, 0)
 
         # Control row
-        self.tc_display = guicomponents.MonitorTCDisplay()
+        self.tc_display = toolguicomponents.MonitorTCDisplay()
         self.tc_display.use_internal_frame = True
         self.tc_display.widget.set_valign(Gtk.Align.CENTER)
         self.tc_display.use_internal_fps = True
@@ -645,7 +653,7 @@ class GmicWindow(Gtk.Window):
         pos_bar_frame.set_margin_start(6)
         pos_bar_frame.set_margin_end(2)
     
-        if editorpersistance.prefs.double_track_hights == True:
+        if guiutils.double_icon_size():
             pos_bar_frame_temp = Gtk.VBox(False, 0)
             pos_bar_frame_temp.pack_start(Gtk.Label(), True, True, 0)
             pos_bar_frame_temp.pack_start(pos_bar_frame, False, False, 0)
@@ -750,9 +758,9 @@ class GmicWindow(Gtk.Window):
         marks_row.pack_start(out_row, True, True, 0)
         marks_row.pack_start(length_row, True, True, 0)
 
-        self.out_folder = Gtk.FileChooserButton(_("Select Folder"))
+        self.out_folder = gtkbuilder.get_file_chooser_button(_("Select Folder"))
         self.out_folder.set_action(Gtk.FileChooserAction.SELECT_FOLDER)
-        self.out_folder.connect("selection-changed", self.folder_selection_changed) 
+        self.out_folder.connect_selection_changed(None, self.folder_selection_changed)
         self.out_label = Gtk.Label(label=_("Frames Folder:"))
         
         self.frame_name = Gtk.Entry()
@@ -920,7 +928,7 @@ class GmicWindow(Gtk.Window):
             self.render_status_info.set_markup("<small>" + info_str +  "</small>")
             self.render_button.set_sensitive(True)
             
-    def folder_selection_changed(self, chooser):
+    def folder_selection_changed(self, chooser, data):
         self.update_render_status_info()
 
     def hamburger_launch_pressed(self, widget, event):
