@@ -27,13 +27,11 @@ from gi.repository import Pango
 
 import datetime
 import glob
-import locale
 try:
     import mlt7 as mlt
 except:
     import mlt
 import os
-import pickle
 import shutil
 import subprocess
 import sys
@@ -50,20 +48,18 @@ import editorpersistance
 import fluxity
 import fluxityheadless
 import gui
-import guicomponents
 import guiutils
 import glassbuttons
+import gtkbuilder
 import gmicplayer
 import mediaplugin
 import mltinit
 import mltprofiles
-import mltfilters
 import positionbar
-import processutils
 import respaths
 import renderconsumer
 import toolsencoding
-import translations
+import toolguicomponents
 import threading
 import userfolders
 import utils
@@ -112,8 +108,6 @@ _hamburger_popover = None
 
 #-------------------------------------------------- launch and inits
 def launch_scripttool(launch_data=None):
-    gui.save_current_colors()
-
     # no args yet
     args = ["profile_name:" + editorstate.PROJECT().profile.description()]
         
@@ -195,13 +189,11 @@ class ScriptToolApplication(Gtk.Application):
             MONITOR_HEIGHT = 400 # initial value, this gets changed when material is loaded
 
         # Themes
-        gui.apply_theme(editorpersistance.prefs.theme)
-        gui.load_current_colors()
+        gui.apply_theme()
 
         # Get launch profile and init player and display GUI params for it. 
         global _current_profile_name
         _current_profile_name = _get_arg_value(sys.argv, "profile_name")
-        _init_player_and_profile_data(_current_profile_name)
 
         # Show window.
         global _window
@@ -211,11 +203,32 @@ class ScriptToolApplication(Gtk.Application):
 
         os.putenv('SDL_WINDOWID', str(_window.monitor.get_window().get_xid()))
 
-        _init_playback()
-        update_length(_script_length)
+        # Set SDL consumer version to be used.
+        #if editorstate.mlt_version_is_greater_correct("7.28.0") or editorstate.force_sdl2 == True \
+        #    or editorstate.app_running_from == editorstate.RUNNING_FROM_FLATPAK:
+        if editorstate.force_sdl2 == True:
+            gmicplayer.set_sdl_consumer_version(gmicplayer.SDL_2)
+        else:
+            gmicplayer.set_sdl_consumer_version(gmicplayer.SDL_1)
         
+        # Launch player if we're using SDL 1 consumer.
+        if gmicplayer.get_sdl_consumer_version() == gmicplayer.SDL_1:
+            _init_player_and_profile_data(_current_profile_name)
+            _init_playback()
+            update_length(_script_length)
+        else:
+            # Launch SDL 2 player after data and gui exist if using that.
+            GLib.idle_add(_create_sdl_2_consumer)
+                        
         self.add_window(_window)
 
+# --------------------------------------- display
+def _create_sdl_2_consumer():
+
+    _init_player_and_profile_data(_current_profile_name)
+    _init_playback()
+    update_length(_script_length)
+    
 
 # ------------------------------------------------- folders init
 def _init_frames_dirs():
@@ -235,6 +248,8 @@ def _init_player_and_profile_data(profile_name):
     global _player, _ticker
     _ticker = utils.Ticker(_ticker_event, TICKER_DELAY)
     _player = gmicplayer.GmicPlayer(respaths.FLUXITY_EMPTY_BG_RES_PATH, _ticker)
+    _player.set_display_widget(_window.monitor)
+    _window.monitor.connect("size-allocate", lambda w, e:_player.display_resized())
 
 def _init_playback():
     _window.set_fps()
@@ -584,8 +599,8 @@ def _encode_settings_clicked():
     align = dialogutils.get_default_alignment(_encoding_panel)
     
     dialog = Gtk.Dialog(_("Video Encoding Settings"),
-                        _window,
-                        Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
+                        None,
+                        None,
                         (_("Cancel"), Gtk.ResponseType.REJECT,
                          _("Set Encoding"), Gtk.ResponseType.ACCEPT))
     dialog.vbox.pack_start(align, True, True, 0)
@@ -674,11 +689,11 @@ class ScriptToolWindow(Gtk.Window):
         app_icon = GdkPixbuf.Pixbuf.new_from_file(respaths.IMAGE_PATH + "flowbladetoolicon.png")
         self.set_icon(app_icon)
         hamburger_launcher_surface = guiutils.get_double_scaled_cairo_image("hamburger.png")
-        if editorpersistance.prefs.double_track_hights == False:
+        if guiutils.double_icon_size():
             psize = 22
         else:
             psize = 44
-        self.hamburger_launcher = guicomponents.PressLaunch(self.hamburger_launch_pressed_popover, hamburger_launcher_surface, psize, psize)
+        self.hamburger_launcher = toolguicomponents.PressLaunchSurface(self.hamburger_launch_pressed_popover, hamburger_launcher_surface, psize, psize)
         self.hamburger_launcher.widget.set_margin_bottom(7)
 
         self.reload_button = Gtk.Button(label=_("Reload Script"))
@@ -720,8 +735,6 @@ class ScriptToolWindow(Gtk.Window):
         # -------------------------------------------------------------------- RIGHT SIDE: MONITOR, CONTROLS, ENCODING 
         black_box = Gtk.EventBox()
         black_box.add(Gtk.Label())
-        bg_color = Gdk.Color(red=0.0, green=0.0, blue=0.0)
-        black_box.modify_bg(Gtk.StateType.NORMAL, bg_color)
         self.monitor = black_box  # This could be any GTK+ widget (that is not "windowless"), only its XWindow draw rect 
                                   # is used to position and scale SDL overlay that actually displays video.
         self.monitor.set_size_request(MONITOR_WIDTH, MONITOR_HEIGHT)
@@ -734,7 +747,7 @@ class ScriptToolWindow(Gtk.Window):
         self.monitors_switcher.set_visible_child_name(CAIRO_DRAW_MONITOR)
 
         # Control row
-        self.tc_display = guicomponents.MonitorTCDisplay(56)
+        self.tc_display = toolguicomponents.MonitorTCDisplay(56)
         self.tc_display.use_internal_frame = True
         self.tc_display.widget.set_valign(Gtk.Align.CENTER)
         self.tc_display.use_internal_fps = True
@@ -848,9 +861,9 @@ class ScriptToolWindow(Gtk.Window):
         marks_row.pack_start(out_row, True, True, 0)
         marks_row.pack_start(length_row, True, True, 0)
 
-        self.out_folder = Gtk.FileChooserButton(_("Select Folder"))
+        self.out_folder = gtkbuilder.get_file_chooser_button(_("Select Folder"))
         self.out_folder.set_action(Gtk.FileChooserAction.SELECT_FOLDER)
-        self.out_folder.connect("selection-changed", self.folder_selection_changed) 
+        self.out_folder.connect_selection_changed(None, self.folder_selection_changed)
         self.out_label = Gtk.Label(label=_("Frames Folder:"))
 
         self.frame_name = Gtk.Entry()
@@ -1058,7 +1071,7 @@ class ScriptToolWindow(Gtk.Window):
             self.render_status_info.set_markup("<small>" + info_str + "</small>")
             self.render_button.set_sensitive(True)
             
-    def folder_selection_changed(self, chooser):
+    def folder_selection_changed(self, chooser, data):
         self.update_render_status_info()
 
     def hamburger_launch_pressed_popover(self, widget, event):
