@@ -25,6 +25,7 @@ import os
 
 import appconsts
 import guiutils
+import gtkbuilder
 import mltprofiles
 import renderconsumer
 import utils
@@ -52,9 +53,10 @@ class ToolsRenderData():
     """
     def __init__(self):
         self.profile_index = None
-        self.use_default_profile = None # NOT USED, 'profile_index' is the meaningful data here, this one should not have been included in this data struct.
-        self.use_preset_encodings = None
-        self.presets_index = None
+        # The DEPRECATED values are part of save data and cannot be removed without braking save files.
+        self.use_default_profile = None # DEPRECATED, 'profile_index' is used.
+        self.use_preset_encodings = None  # DEPRECATED 'encoding_option_index', 'quality_option_index' are used.
+        self.presets_index = None  # DEPRECATED, 'encoding_option_index', 'quality_option_index' are used.
         self.encoding_option_index = None
         self.quality_option_index = None
         self.render_dir = None
@@ -67,9 +69,8 @@ class ToolsRenderData():
         self.frame_name = "frame"
         self.is_preview_render = False
         self.is_flatpak_render = False
-    
-    
-    
+
+
 def get_args_vals_list_for_render_data(render_data):
     profile = mltprofiles.get_profile_for_index(render_data.profile_index)
 
@@ -81,9 +82,8 @@ def get_args_vals_list_for_render_data(render_data):
 
     return args_vals_list
 
-
 def create_container_clip_default_render_data_object(profile):
-    # When ToolsRenderData is used by G'Mic tool we need to have default values be 'None', for container clis we need different
+    # When ToolsRenderData is used by G'Mic tool we need to have default values be 'None', for container clips we need different
     # default values.
     # 
     # When first render is attempted this created to have data available for render process
@@ -100,7 +100,37 @@ def create_container_clip_default_render_data_object(profile):
     render_data.file_extension = ".mp4"
 
     return render_data
+
+def create_container_clip_default_alpha_render_data_object(profile):
+    render_data = create_container_clip_default_render_data_object(profile)
+    encoding_option_index, quality_option_index, extension = renderconsumer.get_default_alpha_enc_quality_ext()
+    if encoding_option_index == -1:
+        return None
+
+    render_data.encoding_option_index = encoding_option_index
+    render_data.quality_option_index = quality_option_index
+    render_data.file_extension = "." + extension
+
+    return render_data
+
+
+def get_render_type_from_render_data(render_data):
+    if render_data == None:
+        return appconsts.DEFAULT_RENDER_FRAME_SEQUENCE
+    elif render_data.do_video_render == True:
+        alpha_encoding_option_index, quality_option_index, extension = renderconsumer.get_default_alpha_enc_quality_ext()
+        # NOTE: this breaks is we add more then 1 alpha render option,
+        # and user switches to use another one,
+        # a more rubust way would be to get list of all alpha encodings indexes.
+        if render_data.encoding_option_index == alpha_encoding_option_index:
+            return appconsts.DEFAULT_RENDER_ALPHA_CLIP
+        else:
+            return appconsts.DEFAULT_RENDER_CLIP
+    else:
+        return appconsts.DEFAULT_RENDER_FRAME_SEQUENCE
     
+    
+
 # ------------------------------------------------------------ GUI interface
 def create_widgets(def_profile_index, disable_audio=True, create_container_file_panel=False):
     """
@@ -212,7 +242,7 @@ def get_render_data_for_current_selections():
     render_data.quality_option_index = widgets.encoding_panel.quality_selector.widget.get_active()
     render_data.presets_index = 0 # presents rendering not available
     render_data.use_preset_encodings = False # presents rendering not available
-    render_data.render_dir = "/" + widgets.file_panel.out_folder.get_uri().lstrip("file:/")
+    render_data.render_dir = widgets.file_panel.out_folder.get_filename()
     render_data.file_name = widgets.file_panel.movie_name.get_text()
     render_data.file_extension = widgets.file_panel.extension_label.get_text()
     
@@ -242,25 +272,6 @@ def get_encoding_desc(args_vals_list):
 
 
 # ----------------------------------------------------------------- helper functions
-"""
-def _render_type_changed(w):
-    if w.get_active() == 0: # User Defined
-        widgets.render_type_panel.presets_selector.widget.set_sensitive(False)
-        widgets.encoding_panel.encoding_selector.encoding_selection_changed()
-    else: # Preset Encodings
-        widgets.render_type_panel.presets_selector.widget.set_sensitive(True)
-        _preset_selection_changed(widgets.render_type_panel.presets_selector.widget)
-
-def _preset_selection_changed(w):
-    encs = renderconsumer.non_user_encodings
-    if disable_audio_encoding == True:
-        encs = renderconsumer.get_video_non_user_encodigs()
-
-    enc_index = w.get_active()
-    ext = encs[enc_index].extension
-    widgets.file_panel.extension_label.set_text("." + ext)
-"""
-
 def _out_profile_changed(categorized_combo):
     profile = mltprofiles.get_profile(categorized_combo.get_selected())
     _fill_info_box(profile)
@@ -280,7 +291,7 @@ def _fill_info_box(profile):
 class RenderFilePanel():
 
     def __init__(self, create_container_file_panel):
-
+        
         if create_container_file_panel == True:
             self.render_location_combo = Gtk.ComboBoxText() # filled later when current sequence known
             self.render_location_combo.append_text(_("Save Rendered Container Media Internally"))
@@ -290,10 +301,9 @@ class RenderFilePanel():
         else:
             self.render_location_combo = None
             
-        self.out_folder = Gtk.FileChooserButton(_("Select Folder"))
+        self.out_folder = gtkbuilder.get_file_chooser_button(_("Select Folder"))
         self.out_folder.set_action(Gtk.FileChooserAction.SELECT_FOLDER)
         self.out_folder.set_current_folder(os.path.expanduser("~") + "/")
-        self.out_folder.set_local_only(True)
         self.out_folder_label = Gtk.Label(label=_("Folder:"))
         out_folder_row = guiutils.get_two_column_box(self.out_folder_label, self.out_folder, 60)
                               
