@@ -99,7 +99,7 @@ def init_languages():
         lang = gettext.translation(APP_NAME, locale_path, languages=langs, fallback=True)
 
     # Un-comment for translations tests
-    #lang = gettext.translation(APP_NAME, locale_path, languages=["ru"], fallback=True)
+    #lang = gettext.translation(APP_NAME, locale_path, languages=["es"], fallback=True)
 
     lang.install(APP_NAME) # makes _() a built-in available in all modules without imports
 
@@ -181,13 +181,11 @@ def load_filters_translations():
     filter_names["Alpha Gradient"] = _("Alpha Gradient")
     filter_names["Crop"] = _("Crop")
     filter_names["Alpha Shape"]= _("Alpha Shape")
-
     filter_names["Volume"]= _("Volume")
     filter_names["Pan"]= _("Pan")
     filter_names["Pan Keyframed"]= _("Pan Keyframed")
     filter_names["Mono to Stereo"]= _("Mono to Stereo")
     filter_names["Swap Channels"]= _("Swap Channels")
-
     filter_names["Pitchshifter - AM"]= _("Pitchshifter - AM")
     filter_names["Distort - Barry's Satan"]= _("Distort - Barry's Satan")
     filter_names["Frequency Shift - Bode/Moog"]= _("Frequency Shift - Bode/Moog")
@@ -212,17 +210,14 @@ def load_filters_translations():
     filter_names["Distort - Sinus Wavewrap"]= _("Distort - Sinus Wavewrap")
     filter_names["Vinyl Effect"]= _("Vinyl Effect")
     filter_names["Chorus - Multivoice"]= _("Chorus - Multivoice")
-
     filter_names["Charcoal"]= _("Charcoal")
     filter_names["Glow"]= _("Glow")
     filter_names["Old Film"]= _("Old Film")
     filter_names["Scanlines"]= _("Scanlines")
     filter_names["Cartoon"]= _("Cartoon")
-
     filter_names["Pixelize"]= _("Pixelize")
     filter_names["Blur"]= _("Blur")
     filter_names["Grain"]= _("Grain")
-
     filter_names["Grayscale"]= _("Grayscale")
     filter_names["Grayscale Luminance"]= _("Grayscale Luminance")
     filter_names["Contrast"]= _("Contrast")
@@ -234,7 +229,6 @@ def load_filters_translations():
     filter_names["Tint"]= _("Tint")
     filter_names["White Balance"]= _("White Balance")
     filter_names["Levels"]= _("Levels")
-
     filter_names["Color Clustering"]= _("Color Clustering")
     filter_names["Chroma Hold"]= _("Chroma Hold")
     filter_names["Three Layer"]= _("Three Layer")
@@ -243,29 +237,23 @@ def load_filters_translations():
     filter_names["Primaries"]= _("Primaries")
     filter_names["Color Distance"]= _("Color Distance")
     filter_names["Threshold"]= _("Threshold")
-
     filter_names["Waves"]= _("Waves")
     filter_names["Lens Correction"]= _("Lens Correction")
     filter_names["Flip"]= _("Flip")
     filter_names["Mirror"]= _("Mirror")
     filter_names["V Sync"]= _("V Sync")
-
     filter_names["Edge Glow"]= _("Edge Glow")
     filter_names["Sobel"]= _("Sobel")
-
     filter_names["Denoise"]= _("Denoise")
     filter_names["Sharpness"]= _("Sharpness")
     filter_names["Letterbox"]= _("Letterbox")
-
     filter_names["Baltan"]= _("Baltan")
     filter_names["Vertigo"]= _("Vertigo")
     filter_names["Nervous"]= _("Nervous")
     filter_names["Freeze"]= _("Freeze")
-
     filter_names["Rotate"]= _("Rotate")
     filter_names["Shear"]= _("Shear")
     filter_names["Translate"]= _("Translate")
-
     filter_names["Color Select"]= _("Color Select")
     filter_names["Alpha Modify"]= _("Alpha Modify")
     filter_names["Spill Suppress"]= _("Spill Suppress")
@@ -285,7 +273,6 @@ def load_filters_translations():
     filter_names["Posterize"]= _("Posterize")
     filter_names["Soft Glow"]= _("Soft Glow")
     filter_names["Newspaper"]= _("Newspaper")
-
     filter_names["Luma Key"] = _("Luma Key")
     filter_names["Chroma Key"] = _("Chroma Key")
     filter_names["Affine"] = _("Affine")
@@ -294,7 +281,6 @@ def load_filters_translations():
     filter_names["Curves"] = _("Curves")
     filter_names["Lift Gain Gamma"] = _("Lift Gain Gamma")
     filter_names["Image Grid"] = _("Image Grid")
-
     filter_names["Color Lift Gain Gamma"] = _("Color Lift Gain Gamma")
     filter_names["Color Channel Mixer"] = _("Color Channel Mixer")
     filter_names["Lens Correction AV"] = _("Lens Correction AV")
@@ -307,6 +293,7 @@ def load_filters_translations():
     filter_names["RotoMask"] = _("RotoMask")
     filter_names["Lens Defisher"] = _("Lens Defisher")
     filter_names["Position Scale"] = _("Position Scale")
+    filter_names["Position Scale Rotate"] = _("Position Scale Rotate")
     filter_names["Wipe"] = _("Wipe")
     filter_names["Mask - Begin"] = "\u21c9" + " " + _("Mask - Begin")
     filter_names["Mask - End"] = "\u21c7" + " "  + _("Mask - End")
@@ -334,7 +321,13 @@ def load_filters_translations():
     filter_names["Motion Tracking Apply"] = _("Motion Tracking Apply")
     filter_names["Motion Tracking Create"] = _("Motion Tracking Create")
     filter_names["Alpha Shape Motion Tracked"]= _("Alpha Shape Motion Tracked")
+    filter_names["Compressor"] = _("Compressor")
+    filter_names["Channel Copy"] = _("Channel Copy")
+
+
+
     # param names
+    # From version 2.10 onwards Media Plugins editable parameters names go here too.
     global param_names
 
     param_names["Position"] = _("Position")
@@ -648,7 +641,6 @@ def load_filters_translations():
     param_names["Glitch frequency"] = _("Glitch Frequency")
     param_names["Shift intensity"] = _("Shift Intensity")
     param_names["Color glitching intensity"] = _("Color Intensity")
-    # From version 2.10 onwards Media Plugins editable parameters names go here too.
     param_names["Speed"] = _("Speed")
     param_names["Speed Variation %"] = _("Speed Variation %")
     param_names["Number of Items"] = _("Number of Items")
@@ -718,8 +710,28 @@ def load_filters_translations():
     param_names["Analysis Accuracy"] = _("Analysis Accuracy")
     param_names["Analysis Shakiness"] = _("Analysis Shakiness")
     param_names["Tracking Coords Interpretation"] = _("Tracking Coords Interpretation")
+    param_names["Copy Channel From:"] = _("Copy Channel From:")
+    param_names["Copy Channel To:"] = _("Copy Channel To:") 
+    param_names["Show Info"] = _("Show Info")
+    param_names["Credits Layout"] = _("Credits Layout")
+    param_names["Center Gap"] = _("Center Gap")
+    param_names["Credit Block Gap"] = _("Credit Block Gap")
+    param_names["Justified X Position"] = _("Justified X Position")
+    param_names["Justified X Position Offset"] = _("Justified X Position Offset")
+    param_names["Name X Offset"] = _("Name X Offset")
+    param_names["Name Y Offset"] = _("Name Y Offset")
+    param_names["Section Title Alignment"] = _("Section Title Alignment")
+    param_names["Credit Font"] = _("Credit Font")
+    param_names["Credit Case"] = _("Credit Case")
+    param_names["Name Font"] = _("Name Font")
+    param_names["Name Case"] = _("Name Case")
+    param_names["Section Title Font"] = _("Section Title Font")
+    param_names["Section Title Case"] = _("Section Title Case")
+    param_names["Default Frames Per Page"] = _("Default Frames Per Page")
 
+     
     # Combo options
+    # From version 2.10 onwards Media Plugins combo options go here too.
     global combo_options
     combo_options["Shave"] = _("Shave")
     combo_options["Rectangle"] = _("Rectangle")
@@ -812,7 +824,6 @@ def load_filters_translations():
     combo_options["Bicubic Smooth"] = _("Bicubic Smooth")
     combo_options["Bicubic Sharp"] = _("Bicubic Sharp")
     combo_options["Spline"] = _("Spline")
-    # From version 2.10 onwards Media Plugins combo options go here too.
     combo_options["Mondrian"] = _("Mondrian")
     combo_options["Pastels"] = _("Pastels")
     combo_options["User Hue"] = _("User Hue")
@@ -865,6 +876,26 @@ def load_filters_translations():
     combo_options["Top Left Justified"] = _("Top Left Justified")
     combo_options["Not Scaled"] = _("Not Scaled")
     combo_options["Scaled To Box"] = _("Scaled To Box")
+    combo_options["Single Line Centered"] = _("Single Line Centered")
+    combo_options["Two Line Centered"] = _("Two Line Centered")
+    combo_options["Single Line Right Justified"] = _("Single Line Right Justified")
+    combo_options["Two Line Right Justified"] = _("Two Line Right Justified")
+    combo_options["Single Line Sides Justified"] = _("Single Line Sides Justified")
+    combo_options["Two Line Left Justified"] = _("Two Line Left Justified")
+    combo_options["Two Columns Centered"] = _("Two Columns Centered")
+    combo_options["Left Justified"] = _("Left Justified")
+    combo_options["Centered"] = _("Centered")
+    combo_options["Right Justified"] = _("Right Justified")
+    combo_options["Transparent"] = _("Transparent")
+    combo_options["Solid Color"] = _("Solid Color")
+    combo_options["No Changes"] = _("No Changes")
+    combo_options["Uppercase"] = _("Uppercase")
+    combo_options["Lowercase"] = _("Lowercase")
+    combo_options["Scrolled"] = _("Scrolled")
+    combo_options["Paged"] = _("Paged")
+    combo_options["Hexagon"] = _("Hexagon")
+    combo_options["Square"] = _("Square")
+    combo_options["Triangle"] = _("Triangle")
 
     # Added for 2.10
     global plugin_groups
@@ -874,14 +905,15 @@ def load_filters_translations():
     
     global plugin_names
     plugin_names["Lines Sweep"] = _("Lines Sweep")
-    plugin_names["Hex Colors"] = _("Hex Colors")
+    plugin_names["Color Polygons"] = _("Color Polygons")
     plugin_names["Floating Boxes"] = _("Floating Boxes")
     plugin_names["Typewriter"] = _("Typewriter")
-    plugin_names["Multiline Text"] = _("Multiline Text")
+    plugin_names["Multiline Animation"] = _("Multiline Animation")
     plugin_names["Hex Overlay"] = _("Hex Overlay")
 
     global plugin_editor_groups
     plugin_editor_groups["Text"] = _("Text")
     plugin_editor_groups["Animation"] = _("Animation")
     plugin_editor_groups["Background"] = _("Background")
-
+    plugin_editor_groups["Layout"] = _("Layout")
+    plugin_editor_groups["Fonts"] = _("Fonts")
\ No newline at end of file
