@@ -26,6 +26,8 @@ from gi.repository import Gtk
 from gi.repository import Gdk
 from gi.repository import GLib
 
+import os
+
 import appconsts
 import clipeffectseditor
 import compositeeditor
@@ -41,6 +43,7 @@ import editorpersistance
 import kftoolmode
 import monitorevent
 import mediaplugin
+import mltprofiles
 import utils
 import respaths
 import tlinewidgets
@@ -50,11 +53,14 @@ page_size = 99.0 # Gtk.Adjustment.get_page_size() wasn't there (wft?)
                  # so use this to have page size
 
 # Scale constants
-
 SCALE_MULTIPLIER = 0.66
 PIX_PER_FRAME_MAX = 20.0 * 1.0 / SCALE_MULTIPLIER
 PIX_PER_FRAME_MIN = 0.001
 
+# For adjusting how fast mouse scrolls timeline sideways
+MOUSE_SIDE_SCROLL_SPEED = 1.4
+MOUSE_SIDE_SCROLL_SPEED_LENGTH_CORRECTION_MULTI = 6000.0
+
 # Trim edit loop playback
 TRIM_EDIT_PRE_ROLL = 25
 TRIM_EDIT_POST_ROLL = 20
@@ -122,19 +128,23 @@ def refresh_player(e):
 
 # --------------------------------- window 
 def window_resized():
-    #try:
-    # Set page offset.
-    tlineypage.vertical_size_update(gui.tline_canvas.widget.get_allocation())
-    
-    # Place clips in the middle of timeline canvas after window resize
-    tlinewidgets.set_ref_line_y(gui.tline_canvas.widget.get_allocation())
-
-    gui.tline_column.init_listeners() # hit areas for track switches need to be recalculated
-    repaint_tline()
+    try:
+        # Set page offset.
+        tlineypage.vertical_size_update(gui.tline_canvas.widget.get_allocation())
+        
+        # Place clips in the middle of timeline canvas after window resize
+        tlinewidgets.set_ref_line_y(gui.tline_canvas.widget.get_allocation())
+        gui.tline_column.init_listeners() # hit areas for track switches need to be recalculated
+        repaint_tline()
+    except Exception as e:
+        msg = str(e)
+        print("updater.window_resized() failed with " + msg)
+        print("reattempting updater.window_resized()...")
+        GLib.idle_add(window_resized)
+        return False
 
     return False
 
-    
 # --------------------------------- timeline
 # --- REPAINT
 def repaint_tline():
@@ -320,7 +330,8 @@ def mouse_scroll_zoom(event):
 
     if do_scroll == True: # Uh, were doing scroll here.
         adj = gui.tline_scroll.get_adjustment()
-        incr = adj.get_step_increment() / 2.0 * (0.72 / tlinewidgets.pix_per_frame)
+        length_multi = 1.0 / current_sequence().get_length() * MOUSE_SIDE_SCROLL_SPEED_LENGTH_CORRECTION_MULTI
+        incr = (adj.get_step_increment() / 2.0 * (0.72 / tlinewidgets.pix_per_frame)) * MOUSE_SIDE_SCROLL_SPEED * length_multi
         if editorpersistance.prefs.scroll_horizontal_dir_up_forward == False:
             incr = -incr
         if event.direction == Gdk.ScrollDirection.UP:
@@ -464,6 +475,23 @@ def display_monitor_clip_name():#we're displaying length and range length also
     gui.editor_window.monitor_tc_info.set_source_name(MONITOR_MEDIA_FILE().name + " - ")
     gui.editor_window.monitor_tc_info.set_source_tc(clip_len)
 
+    if MONITOR_MEDIA_FILE().type == appconsts.VIDEO:
+        try:
+            best_media_profile_index = mltprofiles.get_closest_matching_profile_index(MONITOR_MEDIA_FILE().info)
+            profile = mltprofiles.get_profile_for_index(best_media_profile_index)
+            gui.editor_window.monitor_desc_label.set_text(profile.description())
+        except:
+            # We're being a bit paranoid here, this is going in late in the cycle.
+            print("except at display_monitor_clip_name:")
+            profile = PROJECT().profile
+            gui.editor_window.monitor_desc_label.set_text(profile.description())
+    elif MONITOR_MEDIA_FILE().type == appconsts.AUDIO or MONITOR_MEDIA_FILE().type == appconsts.IMAGE: 
+        (f_name, ext) = os.path.splitext(MONITOR_MEDIA_FILE().name)
+        gui.editor_window.monitor_desc_label.set_text(ext[1:])
+    elif MONITOR_MEDIA_FILE().type == appconsts.IMAGE_SEQUENCE:
+        (f_oath, ext) = MONITOR_MEDIA_FILE().path.split(".")
+        gui.editor_window.monitor_desc_label.set_text(ext + "(image sequence)")
+
     gui.editor_window.monitor_tc_info.set_range_info(*range_info)
 
 def display_sequence_in_monitor():
@@ -499,11 +527,14 @@ def update_seqence_info_text():
     prog_len = PLAYER().producer.get_length()
     if prog_len < 2: # # to 'fix' the single frame black frame at start, will bug for actual 1 frame sequences
         prog_len = 0
-    tc_info = utils.get_tc_string(prog_len)
+    len_tc = utils.get_tc_string(prog_len)
 
     gui.editor_window.monitor_tc_info.set_source_name(name + " - ")
-    gui.editor_window.monitor_tc_info.set_source_tc(tc_info)
+    gui.editor_window.monitor_tc_info.set_source_tc(len_tc)
 
+    profile = PROJECT().profile
+    gui.editor_window.monitor_desc_label.set_text(profile.description())
+    
     range_info = _get_marks_range_info(PLAYER().producer.mark_in, PLAYER().producer.mark_out)
     gui.editor_window.monitor_tc_info.set_range_info(*range_info)
 
