@@ -632,4 +632,9 @@ def get_disk_size_str(size):
         return str(int((size + 500) / 1000)) + " kB"
     else:
         return str(int(size)) + " B"
-            
\ No newline at end of file
+
+def create_render_session_uid():
+    return  get_uid_str()
+
+def get_uid_str():
+    return hashlib.md5(str(os.urandom(32)).encode('utf-8')).hexdigest()
